mode(-1)
//cone with level set at z=1 generated by symmetry
//x>0 (x+z)^2+y^2=4
//so at level z
//(x+z)^2+y^2=4z^2 solve for z to get z=(x+sqrt(4x^2+3y^2))/3;
clear n1 f1 n0 f0;
function z=n1(x,y)
    x=abs(x);//use symmetry
    z=(x+sqrt(4*x^2+3*y^2))/3;
endfunction

//cone with level set at z=1 generated by symmetry to lower right quadrant and
//x>0, y<0: 0<x<1, y=-2, z=-2; 1<x<2, -2<y<-1, z=(x-1)^2+(y+1)^2-1;x=2,-1<y<0, z=y
//so at level z
//0<x<z, y=-2*z, z=-2*y; z<x<2z, -2z<y<z, (x-z)^2+(y+z)^2-z^2=0; x=2z, -z<y<0, z=y [last 2 checked]
function z=n0(x,y)
    x=abs(x);y=-abs(y);//use symmetry to reduce all to lower right quadrant
    if y<=-2*x then
        z=-y/2;
    elseif y <= -x/2 then
        z=x-y-sqrt(-2*x*y);//x-y+sqrt(-2*x*y);
    else
        z=x/2;
    end;
endfunction

function z=f1(x,y),    z=n1(x,y).^2/2;endfunction
function z=f0(x,y),    z=n0(x,y).^2/2;endfunction
//f0=n0;f1=n1;

tic();

beginProgress();

xr=linspace(-5,5,150)';xc=xr;[Xr,Xc]=ndgrid(xr,xc);q=(Xr.^2+Xc.^2)/2;
sr=linspace(-10,10,200)';sc=sr;[Sr,Sc]=ndgrid(sr,sc);
xxr=linspace(-2,2,150)';//xxc=xxr;
xxc=linspace(-3,3,150)';
[XXr,XXc]=ndgrid(xxr,xxc);qq=(XXr.^2+XXc.^2)/2;
z0=feval(xr,xc,f0);l0=pl_lft_llt_2d(xr,xc,z0+q,sr,sc,%t);
z1=feval(xr,xc,f1);l1=pl_lft_llt_2d(xr,xc,z1+q,sr,sc,%t);
//nc=256;nc=128;nc=64;
nc=10;//for experimenting

fig=scf();
xset("fpf"," ");
fig.color_map = [jetcolormap(nc)];
clf();
colorbar(0,1);
hh=gce();//hh.parent.font_size=4;
xtitle("Proximal Average (strictly convex nonsmooth to nonstrictly convex smooth)", "x", "PA");

lset=linspace(0,1,nc);//lset=[0.5];
levels=[1];err=0;s(1,1:10)="_";
i=1;stp=ceil(length(lset)/10);
for lambda=lset,
    l=(1-lambda)*l0+lambda*l1;
    c=pl_lft_llt_2d(sr,sc,l,xxr,xxc,%t);
    pa=c-qq;
    pamin=min(pa);ii=find(pa<0);pa(ii)=zeros(ii);//remove negative parts due to numerical error
    err=min(err,pamin);
    na=2*sqrt(pa);
    contour2d(xxr,xxc,na,levels,style=ceil(lambda*nc)*ones(1, ...
						  length(levels)),strf="034",rect=[-3,-3,3,3]);
    //msg = sprintf("error=%f time=%f",-err,toc())
    msg = "Plot in progress";
    showProgress(i/length(lset), %f, msg);

    i=i+1;
    if modulo(i,stp)==0 then
	  drawnow();
	  drawlater();
    end;
end;

finishProgress();
printf("error=%f  time=%f",-err,toc())

//printf("] error=%f",-err);
h=gce();nchildren=size(h.children,"*");
for i=[1:nchildren],
    h.children(i).children.thickness=2;
end;
h.children(nchildren).children.line_mode="on";
h.box="off";
//h.font_size=4;
//printf(" time=%f\n",toc());

drawnow();
