mode(-1);

//time plq_coDirect, plq_coSplit, and opt_co

function a=displayTime(time,labels,N,s,qdirect,qsplit,copt,winID,strYlbl)
  [a,b]=dispEff(time);
  i=find(time<0.016);time(i)=0;
  scf(winID);clf();
  //points
  dispPoints=%t;
  if dispPoints then
    symbols=[-2,-11,-5];
    tdirect=time(:,1);tsplit=time(:,2);
    plot2d(N,tdirect,style=-2);plot2d(N,tsplit,style=-11);
    if size(time,2)>2 then 
      topt=time(:,3);plot2d(N,topt,style=-9);
      symbols=[symbols -9];  
    end;    
  end
  //regression lines
  for i=1:size(time,2)
    plot2d(N,a(i).*N+b(i),style=i);
  end
  if dispPoints then
    legends([labels labels],[[1:size(time,2)] symbols],opt="ul",font_size=3);  
  else
    legends(labels,[1:size(time,2)],opt="ul",font_size=3);
  end  
  xtitle("Comparison of PLQ convex hull algorithms","n",strYlbl);

  cdirect = plq_eval(qdirect,s);
  csplit = plq_eval(qsplit,s);
  er1=max(abs(csplit-cdirect));
  if size(time,2)>2 then er2=max(abs(cdirect-copt)); else er2=0;end;

  er=max([er1,er2]);
  printf("Max error: %f\n",er);    

//  scf(winID+1);clf();
//  plot2d(s,cllt,style=3);//llt
//  plot2d(s,cgph,style=1);//gph  
//  plot2d(s,cplq,style=2);//plq
//  legends(labels,[-2,-11,-5],opt="ul",font_size=3);  
endfunction


function [time,labels,N,s,qdirect,qsplit,copt]=runAndTimeCo(f,df,domf,K,includeOpt)
  //time the computation of the convex hull of function f
  //using plq_coDirect, plq_coSplit and opt_co functions
  //average over K computations
  //input:
  //dynamic scoping: f,df,domf,K
  //output:
  //time: Nx3 matrix storing computation times time(1,1)= computation time for plq_coDirect for N(1)
  //labels: 1x3 matrix storing algoritm names
  //N: vector containing number of points: N(i) means x will take N(i) values
  //s: vector on which to evaluate the convex hull
  //q: plq function equals to the convex envelope
  //cdirect: vector containing the values of the convex hull at s computed with coDirect
  //csplit: vector containing the values of the convex hull at s computed with coSplit
  //copt: vector containing the values of the convex hull at s computed with opt_co
  //includeOpt: boolean; %t if also time using OPT algorithms (much slower)
  beginProgress();
  tdirect=[];tsplit=[];topt=[];
  for n=N,
    //printf("n=%i\n",n);
    x=linspace(-5,5,n)';s = x;
    //printf("building...");
    //try
      p=plq_build(x,f,df,%t,%f,'soqs','bounded');
    //catch
    //  printf("\nError in building\n");pause;
    //end
    //printf("built\n");
    startTime();    
    for k=1:K,  
      //printf("n=%i, k=%i, p=%ix%i\n",n,k,size(p,1),size(p,2));
      //try
      //printf("AA\n");
      //  qdirect=plq_coDirect(p);
      //	printf("ZZ\n");
      //catch
      //  qdirect=%nan;//printf("Error in plq_coDirect\n");
      //  //pause
      //  //break;
      //end
      qdirect = plq_coDirect(p);
      if (isnan(qdirect)) then printf("%%nan for n=%d.\n", n); end;
    end
    tdirect=[tdirect,elapsedTime()/K];
    if isnan(qdirect) then tdirect($)=%nan;end;

    startTime();
    for k=1:K,
      qsplit = plq_coSplit(p);
    end;
    tsplit=[tsplit,elapsedTime()/K];

    if includeOpt then 
      startTime();
      for k=1:K,
        copt=opt_co(f,df,domf,s,0);
      end;
      topt=[topt,elapsedTime()/K];
    else copt=[];
    end;

    showProgress(n/N($));
  end
  finishProgress();
  time=[tdirect',tsplit'];labels =["direct","split"];
  if includeOpt then time=[time,topt'];labels=[labels,"opt"];end;
endfunction


function [a,b]=dispEff(time,labels)
  //compute slope of the least-square line
  //returns a, b such that Y=a X + b (WARNING: reversed from regress function)
  ieeeold=ieee();ieee(2);
  a=zeros(1,size(time,2));b=a;
  for i=1:size(time,2)
    coef=regress(N,time(:,i));
    cst(i)=coef(2);a(i)=coef(2);b(i)=coef(1);
  end;
  [s,k]=gsort(cst,'r','i');
  l=labels(k)';
  printf("Ranking (best first):\n");
  for i=1:size(time,2);printf("%s: C=%f\n",l(i),s(i));end;//print by decreasing efficiency
  printf("Improvement factor from %s on n=%i: %3.1f\n",labels(1),N($),time($,1)/time($,2));
  if size(time,2)>=3 then
    printf("Improvement factor from %s on n=%i: %3.1f\n",labels(3),N($),time($,3)/time($,2));
  end  
  ieee(ieeeold);
endfunction

//real time functions
function startTime(),tic();endfunction
function t=elapsedTime(),t=toc();endfunction

//build test functions
function y=f1(x), y= x.^4;endfunction
function y=df1(x), y= 4*x.^3;endfunction
function y=f2(x), y= -x.^4;endfunction
function y=df2(x), y= -4*x.^3;endfunction
function y=f3(x), y= x.^2;endfunction
function y=df3(x), y= 2*x;endfunction  
function y=f4(x), y= -x.^2;endfunction
function y=df4(x), y= -2*x;endfunction
//build a quadratic function going thru 3 points
//x=[x1;x2;x3];y=[y1;y2;y3];
function q=splineQuadratic(x,y)
  A = [x.^2 x [1;1;1]];
  q = [(inv(A)*y)'];
endfunction

//i: index 1,2,3,4,5
//f,df: functions and derivative
//n: number of points
//method: either "plq" or "soqs"
function p = myTime(i,f,df,n,method)
  a=-5;b=5;//bounds on the interval
  x=linspace(a,b,n);
  if i<5 then needEval=%t; else needEval=%f;end;
  exclude=%f;extrapolate="bounded";  
  p = plq_build(x,f,df,needEval,exclude,method,extrapolate);
  times = [];
  //printf("n = %d: ", n);
  tic();
  plqcoDirect = plq_coDirect(p);
  timeDirect = toc();
  tic();
  plqcoSplit = plq_coSplit(p);
  timeSplit = toc();
  times = [times; size(p,1), timeDirect, timeSplit];
  //printf("direct = %f, split = %f\n", timeDirect, timeSplit);
endfunction

//CPU time functions
//function startTime(),timer();endfunction
//function t=elapsedTime(),t=timer();endfunction

//functions f1,df1,f2,df2,f3,df3,f4,df4 are dedined in timing-functions.sci
s=stacksize();
if (s(1) < 5e6) then
  if (msscanf(getversion(),"%*7s%f") < 5.2) then
    stacksize(5e6);
  else
    stacksize('max');
  end;
end;
//define f5. df5=%f as it is not available.
p51=splineQuadratic([-3;-2;-1],[1;0;1]);
p52=splineQuadratic([-1;0;1],[1;3;0]);
p53=splineQuadratic([1;2;3],[0;-1;0]);
p5=[-1 p51;1 p52;%inf p53];
function y=f5(x), y=plq_eval(p5,x);y=y+sin(2*%pi*x)*y/2;endfunction
//functions to be tested:
//p=list();
//needEval1=%f;needEval2=%t;exclude=%f;method1="plq";method2="soqs";extrapolate="bounded";
//p($+1) = plq_build(x,f1,df1,needEval1,exclude,method1,extrapolate);
//p($+1) = plq_build(x,f1,df1,needEval1,exclude,method2,extrapolate);
//p($+1) = plq_build(x,f2,df2,needEval1,exclude,method1,extrapolate);
//p($+1) = plq_build(x,f2,df2,needEval1,exclude,method2,extrapolate);
//p($+1) = plq_build(x,f3,df3,needEval1,exclude,method1,extrapolate);
//p($+1) = plq_build(x,f3,df3,needEval1,exclude,method2,extrapolate);
//p($+1) = plq_build(x,f4,df4,needEval1,exclude,method1,extrapolate);
//p($+1) = plq_build(x,f4,df4,needEval1,exclude,method2,extrapolate);
//p($+1) = plq_build(x,f5,%f,needEval2,exclude,method1,extrapolate);
//p($+1) = plq_build(x,f5,%f,needEval2,exclude,method2,extrapolate);
//x=linspace(-5,5);y=feval(x,f5);clf();plot2d(x,y);//plq_plot(p5);

strTitle="";
//function to time: fime f1, f2, f3, f4 with dfi and f5 without df5 using method1 (piecewise linear) and method2 (piecewise quadratic)
f=f1;df=df1;domf=[-5,5];
tic();
winID=0;includeOpt=%f;
//============================================================
//USE SCILAB 5.1.1 for timing! (Faster than Scilab 4.2.1)
//============================================================
//N=100:200:2100;K=1;
//N=100:200:800;K=10;//time consuming
//N=100:50:600;K=1;//tentative
//N=100:100:300;K=1;//debugging 
//N=40:10:100;K=1;//debugging 
N=20:5:40;K=1;
[time,labels,N,s,qdirect,qsplit,copt]=runAndTimeCo(f,df,domf,K,includeOpt);
I = find(or(isnan(time), "c"));
time(I,:) = [];
N(I) = [];
//save('timecox',time,labels,N);
//load('timecox');//reload computed values; may need to adjust labels 
slopeCO=displayTime(time,labels,N,s,qdirect,qsplit,copt,winID,"Time");
//timePA=time;

printf("Elapsed time: %f\n",toc());
//xs2png(1,'time.png');
//xs2eps(1,'time.eps');//convert to pdf using acrobat under Windows for better quality
printf("done\n");

