//demo file: Compare 3 algorithms to compute the
//Legendre-Fenchel conjugate. Each algorithm is
//timed on the computation of conjugate of the absolute value
//function on the grid N={1,2,3,...,n} for increasing 
//values of n=100, 300, 500, ..., 4000 The 3 algorithms compared //are pl_lft_direct, pl_lft_llt using _pl_fusion, and pl_lft_llt 
//using _pl_fusionsci.
//Functions demoed: pl_lft_direct, pl_lft_llt

//Result: Except for small values of n, the direct
//algorithm is the loser. The llt algorithm with
//the option _pl_fusionsci is the winner for the range
//of values of n consider. For very large values
//of n (not shown on the graph), the llt algorithm
//with the option _pl_fusion will beat _pl_fusionsci.
mode(-1);

//increases the memory allocated to Scilab
s=stacksize();
if (s(1) < 5e7) then
  if (msscanf(getversion(),"%*7s%f") < 5.2) then
    stacksize(5e7);
  else
    stacksize("max");
  end;
end;
clear time;
//Grid containing all the increasing values of n
N=[100:200:4000]';
//boolean to verify all algorithms return the same results
isEqual = %t;

printf("Looping till n=%i\n",N(length(N)));
beginProgress();
for i=1:size(N,1)
	n=N(i);//size and upper-bound of the grid
	showProgress((i-1)/size(N,1), %f, sprintf("n=%4d", n));
	Xr=(1:1:n)';//primal grid is {1,2,...,n}
	Sr=Xr;//dual grid is equal to primal grid
	f=abs(Xr);//evalute the absolute value function on the primal grid
	//time the direct algorithm (quadratic complexity;for comparison only)
	timer();C1 = pl_lft_direct(Xr, f, Sr);time(i, 1) = timer();
	//time the llt algorithm using the _pl_fusion internal function
	//_pl_fusion uses explicit loops to achieve a linear complexity
	timer();C2 = pl_lft_llt(Xr,f,Sr,0);time(i, 2) = timer();
	//time the llt algorithm using the _pl_fusionsci internal function
	//_pl_fusionsci uses built-in functions but may not be linear-time
	timer();C3 = pl_lft_llt(Xr,f,Sr,1);time(i, 3) = timer();
	isEqual = isEqual & isequal(C1,C2,C3);
end
finishProgress();
//check that all 3 algorithms return the exact same results
if ~isEqual 
	printf("Error! For i=%i not all methods returned the same result",i);
end

//plot the time taken by each algorithm for each value of n
labels =["direct","fusion","fusionsci"];
scf();
plot2d(N,time);
p=get("hdl");
legends(labels,[1,2,3],opt="ul");
q=get("hdl");
thickness=[1;2;1];line_style=[1;2;4];
for i=1:3
	p.children(i).thickness=thickness(i);
	p.children(i).line_style=line_style(i);
	q.children(2*i).thickness=thickness(i);
	q.children(2*i).line_style=line_style(i);
end
xtitle('Comparison of pl_lft_direct vs pl_lft_llt[fusion] vs pl_lft_llt[fusionsci]', ...
       'n', 'CPU Seconds');
//reset the memory dedicated to Scilab to each previous size
stacksize(s(1));
clear s;
