mode(-1);

//time the plq, gph, llt, and opt for lft,me,pa
//WARNING: USE eps=1E-4 IN plq_clean FOR SOME TESTS

function [a,b,er]=displayTime(time,labels,N,s,q,Q,cllt,copt,winID,ttl,strYlbl)
  [a,b]=compEff(time,labels);
  i=find(time<0.016);time(i)=0;
  scf(winID);clf();
  //points
  dispPoints=%t;
  if dispPoints then
    symbols=[-2,-11,-5];
    tplq=time(:,1);tgph=time(:,2);tllt=time(:,3);
    plot2d(N',tplq,style=-2);plot2d(N',tgph,style=-11);plot2d(N',tllt,style=-5);
    if size(time,2)>3 then 
      topt=time(:,4);plot2d(N,topt,style=-9);
      symbols=[symbols -9];  
    end;    
  end
  //regression lines
  for i=1:size(time,2)
    plot2d(N,a(i).*N+b(i),style=i);
  end
  if dispPoints then
    legends([labels labels],[[1:size(time,2)] symbols],opt="ul",font_size=3);  
  else
    legends(labels,[1:size(time,2)],opt="ul",font_size=3);
  end  
  xtitle(ttl,"n",strYlbl);

  cgph=gph_eval(Q,s);
  cplq=plq_eval(q,s);
  er1=max(cgph-cplq);er2=max(cllt-cplq);
  if size(time,2)>3 then er3=max(cllt(2:$)-copt(2:$)); else er3=0;end;

  er=max([er1,er2,er3]);

//  scf(winID+1);clf();
//  plot2d(s,cllt,style=3);//llt
//  plot2d(s,cgph,style=1);//gph  
//  plot2d(s,cplq,style=2);//plq
//  legends(labels,[-2,-11,-5],opt="ul",font_size=3);  
endfunction


function [time,labels,N,s,q,Q,cllt,copt]=runAndTimeConj(includeOpt)
  //time the computation of the conjugate of function f
  //using plq, gph, llt, and opt librairies
  //average over K computations
  //input:
  //dynamic scoping: f,df,domf,K
  //output:
  //time: Nx4 matrix storing computation times time(1,1)= computation time for plq_lft for N(1)
  //labels: 1x4 matrix storing algoritm names
  //N: vector containing number of points: N(i) means x will take N(i) values
  //s: vector on which to evaluate the conjugate
  //q: plq function whose conjugate we compute
  //Q: qph function whose conjugate we compute
  //cllt: vector containing the values of the conjugate at s computed with llt
  //copt: vector containing the values of the conjugate at s computed with opt
  //includeOpt: boolean; %t if also time using OPT algorithms (much slower)
  beginProgress();
  tplq=[];tgph=[];tllt=[];topt=[];
  for n=N,
    x=linspace(-10,10,n)';
    p=plq_build(x,f,df,%t,%f,'soqs','bounded');
    startTime();
    for k=1:K,
      q=plq_lft(p,%t);
    end
    tplq=[tplq,elapsedTime()/K];
    P=gph_plq(p);
    startTime();
    for k=1:K,
      Q=gph_lft(P,%f);
    end;
    tgph=[tgph,elapsedTime()/K];
    y=f(x);s=df(x);
    startTime();
    for k=1:K,
      cllt=pl_lft_llt(x,y,s,%t);
    end;
    tllt=[tllt,elapsedTime()/K];
    if includeOpt then 
      startTime();
      for k=1:K,
        copt=opt_lft(f,df,domf,s,0);
      end;
      topt=[topt,elapsedTime()/K];
    else
      copt=cllt;//output parameters must be set in Scilab 5
    end;
    showProgress(n/N($));
  end
  finishProgress();
  time=[tplq',tgph',tllt'];labels =["plq","gph","llt"];
  if includeOpt then time=[time,topt'];labels=[labels,"opt"];end;
endfunction

function [time,labels,N,s,q,Q,cllt,copt]=runAndTimeMe(includeOpt)
  //time the computation of the conjugate of function f
  //using plq, gph, llt, and opt librairies
  //average over K computations
  //input:
  //dynamic scoping: f,df,domf,K
  //output:
  //time: Nx4 matrix storing computation times time(1,1)= computation time for plq_me for N(1)
  //labels: 1x4 matrix storing algoritm names
  //N: vector containing number of points: N(i) means x will take N(i) values
  //s: vector on which to evaluate the Moreau envelope
  //q: plq function whose Moreau envelope we compute
  //Q: qph function whose Moreau envelope we compute
  //cllt: vector containing the values of the Moreau envelope at s computed with llt
  //copt: vector containing the values of the Moreau envelope at s computed with opt
  //includeOpt: boolean; %t if also time using OPT algorithms (much slower)
  beginProgress();
  tplq=[];tgph=[];tllt=[];topt=[];
  lambda=0.5;
  for n=N,
    x=linspace(-10,10,n)';s = x;
    p=plq_build(x,f,df,%t,%f,'soqs','bounded');
    px=p(:,1);dpx=px(2:$)-px(1:$-1);
    startTime();    
    for k=1:K,  
//      printf("n=%i, k=%i, p=%ix%i\n",n,k,size(p,1),size(p,2));
      q=plq_me(p,lambda,%t);
    end
    tplq=[tplq,elapsedTime()/K];
    P=gph_plq(p);
    startTime();
    for k=1:K,
      Q=gph_me(P,lambda);
    end;
    tgph=[tgph,elapsedTime()/K];
    y=f(x);
    startTime();
    for k=1:K,
      cllt=pl_me_llt(x,y,s,%t);
    end;
    tllt=[tllt,elapsedTime()/K];
    if includeOpt then 
      startTime();
      for k=1:K,
        copt=opt_me(f,df,domf,s,lambda,0);
      end;
      topt=[topt,elapsedTime()/K];
    end;
    showProgress(n/N($));
  end
  finishProgress();
  time=[tplq',tgph',tllt'];labels =["plq","gph","llt"];
  if includeOpt then time=[time,topt'];labels=[labels,"opt"];end;
endfunction

function [time,labels,N,s,q,Q,cllt,copt]=runAndTimePa(includeOpt)
  //time the computation of the prox average of function f
  //using plq, gph, llt, and opt librairies
  //average over K computations
  //input:
  //dynamic scoping: f,df,domf,K
  //output:
  //time: Nx4 matrix storing computation times time(1,1)= computation time for plq_me for N(1)
  //labels: 1x4 matrix storing algoritm names
  //N: vector containing number of points: N(i) means x will take N(i) values
  //s: vector on which to evaluate the Moreau envelope
  //q: plq function whose Moreau envelope we compute
  //Q: qph function whose Moreau envelope we compute
  //cllt: vector containing the values of the Moreau envelope at s computed with llt
  //copt: vector containing the values of the Moreau envelope at s computed with opt
  //includeOpt: boolean; %t if also time using OPT algorithms (much slower)
  beginProgress();
  tplq=[];tgph=[];tllt=[];topt=[];
  lambda=0.5;
  for n=N,
    x=linspace(-10,10,n)';s = x;
    p=plq_build(x,f,df,%t,%f,'soqs','bounded');
    q=plq_build(x,g,dg,%t,%f,'soqs','bounded');
    startTime();    
    for k=1:K,  
      //printf("n=%i, k=%i, p=%ix%i\n",n,k,size(p,1),size(p,2));
      try
        r=plq_pa(p,q,lambda);
      catch
        r=%nan;printf("Error in plq_pa\n");
        //break;
      end
    end
    tplq=[tplq,elapsedTime()/K];
    if isnan(r) then tplq($)=%nan;end;
    P=gph_plq(p);Q=gph_plq(q);
    startTime();
    for k=1:K,
      R=gph_pa(P,Q,lambda);
    end;
    tgph=[tgph,elapsedTime()/K];
    y=f(x);
    startTime();
    for k=1:K,
      cllt=0;
    end;
    tllt=[tllt,elapsedTime()/K];
    if includeOpt then 
      startTime();
      for k=1:K,
        copt=opt_pa(f,df,domf,g,dg,domg,s,lambda,0);
      end;
      topt=[topt,elapsedTime()/K];
    end;
    showProgress(n/N($));
  end
  finishProgress();
  time=[tplq',tgph'];labels =["plq","gph"];
  if includeOpt then time=[time,topt'];labels=[labels,"opt"];end;
endfunction


function [a,b]=compEff(time,labels)
  //compute slope of the least-square line
  //returns a, b such that Y=a X + b (WARNING: reversed from regress function)
  ieeeold=ieee();ieee(2);
  a=zeros(1,size(time,2));b=a;
  for i=1:size(time,2)
    coef=regress(N,time(:,i));
    cst(i)=coef(2);a(i)=coef(2);b(i)=coef(1);
  end;
  ieee(ieeeold);
endfunction

function dispEff(time,labels,a,b)
  ieeeold=ieee();ieee(2);
  for i=1:size(time,2)
    coef=regress(N,time(:,i));
    cst(i)=coef(2);a(i)=coef(2);b(i)=coef(1);
  end;
  [s,k]=gsort(cst,'r','i');
  l=labels(k)';
  printf("Ranking (best first):\n");
  for i=1:size(time,2);printf("%s: C=%f\n",l(i),s(i));end;//print by decreasing efficiency
  printf("Improvement factor from %s on n=%i: %3.1f\n",labels(1),N($),time($,1)/time($,2));
  if size(time,2)>=3 then
    printf("Improvement factor from %s on n=%i: %3.1f\n",labels(3),N($),time($,3)/time($,2));
  end  
  ieee(ieeeold);
endfunction

//real time functions
function startTime(),tic();endfunction
function t=elapsedTime(),t=toc();endfunction

//CPU time functions
//function startTime(),timer();endfunction
//function t=elapsedTime(),t=timer();endfunction

//////////////////////////////////////////////////

if %t | (msscanf(getversion(),"%*7s%d") < 5) then
  stacksize(5E7);
else
  stacksize('max');
end

function y=f(x), y=x.^4;endfunction
function y=df(x), y=4*x.^3;endfunction
domf=[-%inf,%inf];
function y=g(x), y=exp(x);endfunction
function y=dg(x), y=exp(x);endfunction
domg=[-%inf,%inf];

strTitle="";

tic();
//============================================================
select x_choose([ ..
                  "Legendre-Fenchel transform", ..
                  "Moreau envelope", ..
                  "Proximal average"], ..
                "Which algorithm would you like to time?")
 case 1 then
  N=100:100:1000;K=1;//timeLFTa
  includeOpt=%t;
  tic();
  [time,labels,N,s,q,Q,cllt,copt]=runAndTimeConj(includeOpt);//recompute
  winID=scf();
  [a,b,er]=displayTime(time,labels,N,s,q,Q,cllt,copt,winID,"Legendre-Fenchel transform","Run time (sec)");
  winID=scf();
  displayTime(time(:,1:$-1),labels(1:$-1),N,s,q,Q,cllt,copt,winID,"Legendre-Fenchel transform (without OPT)","Run time (sec)");
  
  printf("Elapsed time: %f\n", toc());
  dispEff(time,labels,a,b);
  printf("Max error: %f\n",er);    

 case 2 then
  N=100:100:800;K=1;//timeLFTb, timeME
  includeOpt=%t;
  tic();
  [time,labels,N,s,q,Q,cllt,copt]=runAndTimeMe(includeOpt);
  winID=scf();
  [a,b,er]=displayTime(time,labels,N,s,q,Q,cllt,copt,winID,"Moreau envelope","Time");
  winID=scf();
  displayTime(time(:,1:$-1),labels(1:$-1),N,s,q,Q,cllt,copt,winID,"Moreau envelope (without OPT)","Time");
  printf("Elapsed time: %f\n", toc());
  dispEff(time,labels,a,b);
  printf("Max error: %f\n",er);    
   
 case 3 then
  N=100:100:600;K=1;//time consuming
  includeOpt = %t;
  [time,labels,N,s,q,Q,cllt,copt]=runAndTimePa(includeOpt);
  winID=scf();
  [a,b,er]=displayTime(time,labels,N,s,q,Q,cllt,copt,winID,"Proximal average","Run time (sec)");
  printf("Elapsed time: %f\n", toc());
  dispEff(time,labels,a,b);
  printf("Max error: %f\n",er);    
end;
