function [b,n] = _gph_check(gph,testConvex)
  //false if gph structure violates requirements; true otherwise
  //gph: gph function
  //testConvex: boolean; true means check gph is convex, false means do not test convexity
  //b: boolean; true is structure test passes; false otherwise
  //n: column dimension of gph
  eps=1e-6;
  if argn(2)<2 then testConvex=%t;end;
  n = size(gph, 2);
  if (size(gph,1) ~= 3) then
    b = %f;//gph should be a 3xn matrix
  elseif n<=1 then
    b=%f;//n should be n>1
  elseif n==2 then
    [x0,x1,s0,s1,f0,f1]=(gph(1,1),gph(1,2),gph(2,1),gph(2,2),gph(3,1),gph(3,2));
    if abs(x1-x0)>eps then
        b=%t;//full domain: OK
    else //indicator of singleton
      b = and([(abs(s0-s1)>eps&abs(f0-f1)<eps)]);
    end
  else
    //bounded domain should return %inf for function value
    b = and([(gph(1,1) == gph(1,2)) == (gph(3,1) == %inf); ..
       (gph(1,$-1) == gph(1,$)) == (gph(3,$) == %inf)]);
    //gph(1,:) should be nondecreasing
    b = and([b ;and(clean(gph(1,2:$) - gph(1,1:$-1),eps,eps) >= 0)]);
    //gph(2,:) should be nondecreasing for convex gph
    if testConvex then   
      b = and([b ;and(clean(gph(2,2:$) - gph(2,1:$-1),eps,eps) >= 0)]);
    end      
    //check continuity of f
    x=gph(1,:); s=gph(2,:); f=gph(3,:);
    d = f(2:$-1) + (x(3:$) - x(2:$-1)) .* (s(3:$) + s(2:$-1)) / 2;//integrate right for all pieces...
    e = f(2) + (x(1) - x(2)) * (s(1) + s(2)) / 2;                 //except the first piece
    pl_bb = [abs(e - f(1)) <= eps, abs(d - f(3:$)) <= eps];
    Er = [abs(e - f(1)), abs(d - f(3:$))];//error matrix
    if gph(3,1)==%inf & gph(1,1)==gph(1,2) then pl_bb(1)=%t;Er(1)=0; end;
    if gph(3,$)==%inf & gph(1,$)==gph(1,$-1) then pl_bb($)=%t;Er($)=0; end;
    //if ~and(pl_bb) then 
      //warning(sprintf("_gph_check: f is not continuous. Max error: %f greater than eps: %f\n",max(Er),eps));
    //end;
    b = b & and(pl_bb);
  end
endfunction