function gph = gph_add(gph1, gph2)
  oldIeee = ieee();
  ieee(2);

  // Special case: adding a point indicator function.
  if size(gph2,2) == 2 & gph2(1,1) == gph2(1,2) then
	  t=gph1; gph1=gph2; gph2=t;    // Put the indicator in gph1.
  end;
  if size(gph1,2) == 2 & gph1(1,1) == gph1(1,2) then
	  z = gph1(3,1) + gph_eval(gph2, gph1(1,1));
	  gph = gph1;
	  gph(3,:) = [z z];
	  return;
  end;

  // Ignore the boundary pieces, until the very end,
  // where we attach them to the resulting gph.
  X1 = gph1(1,2:$-1);
  X2 = gph2(1,2:$-1);
  X = unique([X1, X2]);
  // Find the index of each X in gph1 and gph2.
  I1 = _pl_fusionsci(X1', X');
  I2 = _pl_fusionsci(X2', X');
  // These are the slopes of the subdifferentials in the regions that each
  // X(i) maps into in both gph1 and gph2.
  m1 = (gph1(2,I1+1) - gph1(2,I1)) ./ (gph1(1,I1+1) - gph1(1,I1));
  m2 = (gph2(2,I2+1) - gph2(2,I2)) ./ (gph2(1,I2+1) - gph2(1,I2));
  // At each X value, compute the left and right limits of the sum.
  // Put the right limits "below" the left limits, then reshape to get "beside".
  SR = gph1(2,I1) + gph2(2,I2);
  J = find(X ~= gph1(1,I1)); SR(J) = SR(J) + m1(J) .* (X(J) - gph1(1,I1(J)));
  J = find(X ~= gph2(1,I2)); SR(J) = SR(J) + m2(J) .* (X(J) - gph2(1,I2(J)));
  // X matches up with gph1(:,I1) and gph2(:,I2).  If, for In > 1, X ==
  // gphn(1,In-1), then set SL += gphn(2,In-1) - gphn(2,In) to adjust SL to
  // have the left slope at X.
  SL = SR;
  J = find(I1 > 2);
  if J ~= [] then
    K = find(X(J) == gph1(1,I1(J)-1));
    if K ~= [] then
      SL(J(K)) = SL(J(K)) + gph1(2,I1(J(K))-1) - gph1(2,I1(J(K)));
    end;
  end;
  J = find(I2 > 2);
  if J ~= [] then
    K = find(X(J) == gph2(1,I2(J)-1));
    if K ~= [] then
      SL(J(K)) = SL(J(K)) + gph2(2,I2(J(K))-1) - gph2(2,I2(J(K)));
    end;
  end;
  // Build gph.
  F = (gph_eval(gph1, X') + gph_eval(gph2, X'))';
  gph = [X;SL;F ; X;SR;F];
  // Reshape gph from (6 x n) to (3 x 2n).
  gph = matrix(gph, 3, size(gph,2)*2);
  // It could be that gph.{s(i) = -%inf, f(i) = %inf} for all i < j if j > 1
  // and gph1(1,j) is below gph2's lower bound.  Or it may be that gph(1,1)
  // = -%inf if both gph1,gph2 are unbounded below.  In both cases, we want
  // to delete those columns.
  gph(:,find(or(isinf(gph), "r"))) = [];    // Remove columns with infinite x's.
  if (msscanf(getversion(),"%*7s%d") < 5) then//custom implementation for Scilab 4
    gph = _unique(gph);
  else//only works in Scilab 5
    gph = unique(gph, "c");
  end;

  // Add the unbounded regions.
  // First determine the lower and upper x bounds.
  lb = -%inf; ub = %inf;
  if gph1(1,1) == gph1(1,2) then lb = max(lb, gph1(1,1)); end;
  if gph2(1,1) == gph2(1,2) then lb = max(lb, gph2(1,1)); end;
  if gph1(1,$-1) == gph1(1,$) then ub = min(ub, gph1(1,$)); end;
  if gph2(1,$-1) == gph2(1,$) then ub = min(ub, gph2(1,$)); end;

  if lb ~= -%inf then
    gph = [gph(:,1) + [0; -1; %inf], gph];
  else
    // Both functions are unbounded below.
    x0 = gph(1,1) - 1;
    s0 = gph1(2,2) + (x0 - gph1(1,2)) * ((gph1(2,2) - gph1(2,1)) / ..
                                         (gph1(1,2) - gph1(1,1)));
    s0 = s0 + gph2(2,2) + (x0 - gph2(1,2)) * ((gph2(2,2) - gph2(2,1)) / ..
                                              (gph2(1,2) - gph2(1,1)));

    lower = [x0; s0; gph_eval(gph1,x0) + gph_eval(gph2,x0)];
    gph = [lower, gph];
  end;

  if ub ~= %inf then
    gph = [gph, gph(:,$) + [0; 1; %inf]];
  else
    // Both functions are unbounded above.
    x9 = gph(1,$) + 1;
    s9 = gph1(2,$-1) + (x9 - gph1(1,$-1)) * ((gph1(2,$) - gph1(2,$-1)) / ..
                                             (gph1(1,$) - gph1(1,$-1)));
    s9 = s9 + gph2(2,$-1) + (x9 - gph2(1,$-1)) * ((gph2(2,$) - gph2(2,$-1)) / ..
                                                  (gph2(1,$) - gph2(1,$-1)));
    upper = [x9; s9; gph_eval(gph1,x9) + gph_eval(gph2,x9)];
    gph = [gph, upper];
  end;

  ieee(oldIeee);
endfunction
