function Z = gph_eval(gph, X)
  //evaluate the gph function gph on the NONDECREASING column vector X
  Z=[];
  if X == [] then return; end;  
  isDebug=%f;
  if size(X,2) ~= 1 then
      cerror("gph_eval: X should be a column vector.");
  end;
  //do NOT test _gph_check since some gph functions are NOT convex e.g. proximal mapping is monotone nonconvex
//  if ~_gph_check(gph,%f) then cerror("gph struct pb in gph_eval");end;
  
  // Integrates the subdifferential gph to give the value
  // of the corresponding PLQ function at each X value.
  oldIeee = ieee();ieee(2);
  if X~=[] then
    n = size(gph, 2);
    if n == 2 & gph(1,1) == gph(1,2) then    // Special case: Point indicator.
      Z = zeros(X) + %inf;
      Z(find(X == gph(1,1))) = gph(3,1);
      return;
    end;

    I = _pl_fusionsci(gph(1,:)', X);
    X = X';  I = I';
    I(find(I == 1)) = 2;
    // Guarantee that X > gph(1,I) by incrementing I's as necessary.
    // _pl_fusion(sci) doesn't guarantee to be above the left bound.
    gph2 = [gph, [%nan;%nan;%nan]];
    J = find(X == gph2(1,I) & I < n+1);
    while J ~= []
        I(J) = I(J) + 1;
        J = find(X == gph2(1,I) & I < n+1);
    end;
    I(find(I == n + 1)) = n;
    // X(i) is between gph(1,I(i)-1) inclusive and gph(1,I(i)) exclusive.
    // Z(i) = gph(3,I(i)) - int_{X(i)<x<gph(1,I(i))} (height of gph at x)

    // Duplicate values in X, in gph(1,:), and between the two, are handled:
    // 1. X(i) == X(j) is fine because each X(i) is independantly assigned an 
    // I(i) from _pl_fusionsci, and the pieces the X(i)'s land in are integrated
    // separately for each X(i).
    // 2. We ensure by the loop above, that when X(i) == gph(1,j)
    // [or, gph(1,j-1)<=X(i)<gph(1,j)], that I(i)=j.
    // Integration is done rightward (subtracting) from
    // gph(3,j) to X(i), so gph(1,j-1)-X(i)=0 is never calculated, because
    // gph(1,j)-X(i)>0 is used instead.
    // 3. Finally, gph(1,j-1) == gph(1,j) is fine because _pl_fusionsci will never
    // find an X(i) strictly between two equal numbers.
    m = (gph(2,I) - gph(2,I-1)) ./ (gph(1,I) - gph(1,I-1));
    S = gph(2,I) - m .* (gph(1,I) - X);
    area = (gph(2,I) + S) .* (gph(1,I) - X) / 2;
    Z = (gph(3,I) - area)';

    // Enforce lower-semicontinuity if upper-bounded.
    if gph(3,$) == %inf then
      Z(find(X == gph(1,$))) = gph(3,$-1);
    end
    // Enforce lower-semicontinuity if lower-bounded.
    if gph(3,1) == %inf then
      Z(find(X == gph(1,1))) = gph(3,2);
    end
  end

  ieee(oldIeee);
  if isDebug then 
    p=plq_gph(gph);
    y=plq_eval(p,X);
    if ~isEqual(y,Z) then printf("G eval pb\n");end;
  end
    
endfunction