// =================================================
// Generate a regular grid from scattered input data
// according to a quadratic 2D-interpolation
// =================================================
//           (c) 2007 - Tibault Reveyrand
//                  www.reveyrand.fr
//http://www.reveyrand.fr/INTERPOLATE.html
// contact : reveyrand@gmail.com
// =================================================
function output=interpolate_grid(M,scaling_factor)
	X_min=min(M(:,1));X_max=max(M(:,1));
	Y_min=min(M(:,2));Y_max=max(M(:,2));
	Z_min=min(M(:,3));Z_max=max(M(:,3));
	step=min(abs((M(1:$-1,1)-M(2:$,1))+%i*(M(1:$-1,2)-M(2:$,2))))/scaling_factor;
	X_nbp=1+int((X_max-X_min)/step);
	Y_nbp=1+int((Y_max-Y_min)/step);
	x=linspace(0,1,X_nbp).';
	y=linspace(0,1,Y_nbp).';
	z=zeros(X_nbp,Y_nbp);
	M(:,1)=(M(:,1)-X_min)./(X_max-X_min);
	M(:,2)=(M(:,2)-Y_min)./(Y_max-Y_min);
	M(:,3)=(M(:,3)-Z_min)./(Z_max-Z_min);
	for i=1:X_nbp,
		for j=1:Y_nbp,
			d=abs(M(:,1)-x(i)+%i*(M(:,2)-y(j)))./max(abs(M(:,1)-x(i)+%i*(M(:,2)-y(j))));
			w=(((1-(d)).^2).*(1+2.*(d)))./(1-(((1-(d)).^2).*(1+2.*(d)))+((1-(((1-(d)).^2).*(1+2.*(d))))==0)*%eps);
			MT=M(find(w~=0),:);
			w=w(find(w~=0));
			_system=[w.*MT(:,1).^2,w.*MT(:,2).^2,w.*MT(:,1).*MT(:,2),w.*MT(:,1),w.*MT(:,2),w.*((MT(:,1).*0)+1)];
			_coeff=pinv(_system)*(w.*MT(:,3));
			z(i,j)=(_coeff(1)*x(i)^2+_coeff(2)*y(j)^2+_coeff(3)*x(i)*y(j)+_coeff(4)*x(i)+_coeff(5)*y(j)+_coeff(6));
		end;
	end;
	output=list(x.*(X_max-X_min)+X_min,y.*(Y_max-Y_min)+Y_min,z.*(Z_max-Z_min)+Z_min);
endfunction