//internal function: Parabolic envelope of a function
//represented as a matrix. Useful for computing distance transforms
function out=me_pe_d(f)
// lower bound of f definition is 0, upper is n-1
	// asterisked comments (//*) are as in the alg at Felzenswalb, p. 7
	n = length(f);
	k=1 //* Index of rightmost parabola in lower envelope
	v(1)=1 //* Locations of parabolas in lower envelope
	z(1)=-%inf //* Locations of boundaries between parabolas
	z(2)=%inf
	for q=2:n //* Compute lower envelope
	        // find intersection between para at x=q and para at x=v(k)
		// (we're searching for the leftmost lower envelope parabola that intersects one at (q,f(q))
		// at a point to the left of the lower envelope's last intersection
		
		qinit=%t // if this is the first time through for this q value, just calculate s and continue
		if ~isinf(f(q)) // if f(q) is inf, no intersection of the parabola will be suitable
			while qinit | ((s <= z(k)) & (~isinf(s) | s>0))
				if qinit
					qinit=%f
				else
					k=k-1
				end
				s=( (f(q)+q^2)-(f(v(k))+(v(k))^2) )/(2*q-2*v(k))
				sinf=%t
				if(f(q)==%inf)
					if(f(v(k))==%inf)//both values infinite						
						s=(q+v(k))/2
					else//f(q) infinite
					end
				else
					if (f(v(k))==%inf)//f(v(k)) infinite, s=%inf
					else
						sinf=%f
					end
				end
				if sinf
					if s <= z(k)
						str="<="
					else
						str=">"
					end
				end
						
			end
			k=k+1
			v(k)=q
			z(k)=s
			z(k+1)=%inf
		end
	end
	k=1
	for q = 1:n //* Fill in values of distance transform
		while z(k+1) < q
			k=k+1
		end
		out(q)=(q-v(k))^2+f(v(k))
	end
endfunction