// Moreau Envelope computation using monotone nonexpansive proximal mapping (linear-time)
// only works for CONVEX functions
function [M,P] = pl_me_nep(X, f, S)
	// step 1. calculate the MYE for S(1) (binary search)
	min_val = %inf;
	i = 1;
	m = size(S, 1);n=size(X,1);
	//function phi = v(i), phi = (S(1)-X(i))^2 + f(i), endfunction;
	v = (S(1) - X) .^ 2 + f;
	a = 1; b = n;
	while (b - a > 1)
		mid = round((b + a) / 2);
		l = round((a + mid) / 2);
		r = round((mid + b) / 2);
		
		if (v(mid) < min(v(l), v(r)))
			a = l; b = r;
		elseif (min(v(l), v(r)) < min(v(a), v(b)))
			if (v(l) < v(r))
				b = mid;
			elseif (v(l) > v(r))
				a = mid;
			end
		else
			if (v(a) < v(b))
				b = l;
			else
				a = r;
			end
		end
	end
	
	if (v(b) > v(a))
		i = a;
	else
		i = b;
	end
	M(1) = v(i);P(1)=i;
	// step 2. iterate through the rest of f and build M based on increasing indices
	for j=2:m
		val_1 = (X(i) - S(j)) ^ 2 + f(i);
		if (i < n)
			val_2 = (X(i+1) - S(j)) ^ 2 + f(i+1);
		else
			val_2 = %inf;
		end
		
		if (val_1 < val_2)
			M(j) = val_1;P(j)=i;
		else
			M(j) = val_2;P(j)=i+1;
			if (i < n),i = i + 1;end;
		end
	end
endfunction