function F = op_fitz(A, n, x, xstar)
  // Evaluate the Fitzpatrick function F[A,n](x, xstar) using
  // a recursive algorithm that uses LLT to achieve quartic
  // complexity O(m^3) when m == Nx = Nx*.
  //
  // A = [a; astar];

  if (size(x,1) < size(x,2)) then x = x'; end;
  if (size(xstar,1) < size(xstar,2)) then xstar = xstar'; end;

  if (~and(A(1,2:$) - A(1,1:$-1) >= 0)) then
    [a,k] = gsort(A(1,:), "c", "i");
    A = [a; A(2,k)];
  end;

  Nx = size(x, 1);
  Nxstar = size(xstar, 1);
  m = size(A, 2);

  a = A(1,:)';
  astar = A(2,:)';

  if (n == %inf) then
    // Is the operator monotonically nondecreasing?
    if (and(astar(2:$) - astar(1:$-1) >= 0)) then
      n = m + 1;
    else
      F = %inf * ones(size(x, 1), size(xstar, 1));
      return;
    end;
  end;

  select (n)

  case 1 then
    F = x * xstar';

  case 2 then
    F = %inf * ones(m, m);
    for i = 1:m
      F(i,i) = a(i)*astar(i);
    end;
    F = pl_lft_llt_2d(astar, a, F, x, xstar);

  case 3 then
    f0 = -%inf * ones(m, m);
    for i = 1:m
      for j = 1:m
        v = -a(i)*astar(i) - a(j)*astar(j) + astar(i)*a(j);
        f0(i,j) = max(f0(i,j), v);
      end;
    end;

    F = pl_lft_llt_2d(astar, a, -f0', x, xstar);
    //F = pl_lft_llt_2d(a, astar, f0, x, xstar)';

  else
    F2 = op_fitz(A, n-2, a, astar); // F_{A,n-2}, m x m

    f = zeros(m, m);
    for i = 1:m          // a_1 = a(i)
      a1 = a(i);
      for j = 1:m        // a_{n-1}^* = astar(j)
        anstar = a(j);
        //f(i,j) = -a(i)*astar(i) - a(j)*astar(j) + a(j)*astar(i);
        f(i,j) = -a(i)*astar(i) - a(j)*astar(j) + F2(j, i);
      end;
    end;

    F = pl_lft_llt_2d(astar, a, -f', x, xstar);

  end;
endfunction