function F = op_fitz_brute(A, n, x, xstar)
  // Evaluate the Fitzpatrick function F[A,n](x, xstar) using
  // a brute-force method with complexity O(n*m^(n-1)*N).
  //
  // A = [a; astar];

  if (size(x, 1) < size(x, 2)) then x = x'; end;
  if (size(xstar, 1) < size(xstar, 2)) then xstar = xstar'; end;

  Nx = size(x, 1);
  Nxstar = size(xstar, 1);
  m = size(A, 2);

  if (~and(A(1,2:$) - A(1,1:$-1) >= 0)) then
    [a,k] = gsort(A(1,:), "c", "i");
    A = [a; A(2,k)];
  end;

  if (n == %inf) then
    // Is the operator monotonically nondecreasing?
    if (and(A(2,2:$) - A(2,1:$-1) >= 0)) then
      n = m + 1;
    else
      F = %inf * ones(size(x, 1), size(xstar, 1));
      return;
    end;
  end;

  if (n == 1) then
    F = x * xstar';
    return;
  end;

  F = zeros(Nx, Nxstar);
  a1 = ones(1, n-1);

  // Todo: checksize()
  for i = 1:Nx
    xi = x(i);
    for j = 1:Nxstar
      xstarj = xstar(j);

      a = [A(1,1)*a1; A(2,1)*a1; a1];     // Each column = [A(:, i); i]
      sup = -%inf;
      permuteA = %t;
      while (permuteA)
        z = (a(1,2:(n-1)) - a(1,1:(n-2))) .* a(2,1:(n-2));
        z = sum(z) + (xi - a(1,n-1)) * a(2,n-1) + a(1,1) * xstarj;
        sup = max(sup, z);

        // Permute A with a loop to avoid recursion.
        k = 0;
        wrap = %t;
        while (wrap & k < n)
          k = k + 1;
          if (k == n) then break; end;
          z = a(3,k) + 1;
          if (z <= size(A,2)) then
            wrap = %f;
          else
            wrap = %t;
            z = 1;
          end;
          a(1,k) = A(1,z);
          a(2,k) = A(2,z);
          a(3,k) = z;
        end;

        if (k == n) then permuteA = %f; end;
      end;

      F(i,j) = sup;
    end;
  end;
endfunction