function F = op_fitz_direct(A, n, x, xstar)
  // Evaluate the Fitzpatrick function F[A,n](x, xstar) using
  // a recursive algorithm with quartic complexity O(m^4).
  //
  // A = [a; astar];

  if (size(x,1) < size(x,2)) then x = x'; end;
  if (size(xstar,1) < size(xstar,2)) then xstar = xstar'; end;

  if (~and(A(1,2:$) - A(1,1:$-1) >= 0)) then
    [a,k] = gsort(A(1,:), "c", "i");
    A = [a; A(2,k)];
  end;

  if (n == %inf) then
    // Is the operator monotonically nondecreasing?
    if (and(A(2,2:$) - A(2,1:$-1) >= 0)) then
      n = size(A, 2) + 1;
    else
      F = %inf * ones(size(x, 1), size(xstar, 1));
      return;
    end;
  end;

  F = op_fitz_direct_do(A, n, x, xstar);
endfunction