function [_f,_g,ind] = _costf_co(X,ind,xb)
  //input: 
  //X: point to evaluate at. X=[x;lambda] with x2=x
  //ind=1: not output; ind=2: only compute f; ind=3: only compute g; ind=4: compute f and g
  //xb: parameters used in the computation (real numbers)
  //WARNING: f,df are passed by dynamic scoping (defined in opt_me)
  //output: 
  //_f: function value at y
  //_g: derivative value at y
  //ind<0 cannot compute f at x; ind=0 interrupt optimization
  eps=1E-6;//precision to handle round-off errors
  x=X(1);lambda=X(2);_g=[%nan;%nan];
  if lambda < -eps | lambda > 0.5 + eps then 
    //must have 0<=lambda<=1 but enforce 0<=lambda<=0.5 since cases 0.5<=lambda<=1 are recovered
    //using the symmetry: lambda -> 1-lambda; x1->x2; x2->x1
    //infinite penalization since 'nd' method does not take bound contraints
    //note could also return ind < 0
    _f=%inf;
  else
    if lambda < eps then lambda =0;end;
    lambda1=1-lambda;x1=(xb-lambda*x)/lambda1;
    x2=x;lambda2=lambda;//makes formula easier to read
    _f = lambda1 * f(x1) + lambda2 * f(x2);
    _g1=-lambda*df(x1) + lambda*df(x);//derivative wrt x=x2
    _g2= -f(x1) + df(x1)/lambda1 + f(x);//derivative wrt lambda;
    _g = [_g1;_g2];
  end
  //workaround bug in optim with method 'nd': when gradient is exactly 0 errors out with optim: Wrong input parameters.
  //workaround is to replace 0 with a small value
  if _g==[0;0] then _g=_g+1E-20;end;
  //what do we return for a point outside the domain? The choice below while not the most
  //efficient works on our examples.
  if _f==%inf then 
    _f=%inf;//number_properties('huge');
    _g=[%nan;%nan];
    ind=-1;
  end;    
  //printf("i=%i, x=%f, g=%f, f=%f\n",i,x,_g,_f);
endfunction