function [_f,_g,ind] = _costf_ka(y,ind,x,lambda),
  //input: 
  //y: point to evaluate at
  //ind=1: not output; ind=2: only compute f; ind=3: only compute g; ind=4: compute f and g
  //x,lambda: parameters used in the computation (real numbers)
  //WARNING: f1,df1,f2,df2,g,dg are passed by dynamic scoping (defined in opt_ka)
  //output: 
  //_f: function value at y
  //_g: derivative value at y
  //ind<0 cannot compute f at x; ind=0 interrupt optimization
  lambda1=1-lambda;lambda2=lambda;
  y1 = y;  
  if lambda2>0 then
    y2 = (x - lambda1 * y1)/lambda2;        
    _f = lambda1 * f1(y1) + lambda2 * f2(y2) + lambda1 * lambda2 * g(y1-y2);
    //derivative is computed with respect to lambda1 by substituting y2
    _g = lambda1 * df1(y1) -lambda1 * df2(y2)+ lambda1 * dg(y1-y2);    
  else//lambda1=1, lambda2=0 so y1=x
    _f = f1(x);
    _g = 0;
  end
  //workaround bug in optim with method 'nd': when gradient is exactly 0 errors out with optim: Wrong input parameters.
  //workaround is to replace 0 with a small value
  if _g==0 then _g=1E-20;end;
  //what do we return for a point outside the domain? The choice below while not the most
  //efficient works on our examples.
  if _f==%inf then _f=number_properties('huge');_g=%inf;end;
    
  //printf("i=%i, y=%f, g=%f, f=%f\n",i,y,_g,_f);
endfunction