function y1 = _pl_kaFeasiblePoint(xstart,x,lambda1,lambda2,domf1,domf2,domg)
  //compute a feasible point i.e. point for which the function
  //lambda1 f1(y1) + lambda2 f2(y2) + lambda1 lambda2 g(y1-y2) is finite
  //x: real number
  //lambda1, lambda2: real numbers satisfying lambda1+lambda2=1 and lambda1,lambda2>=0
  //domf1,domf2,domg: intervals in the form I=[lb,ub] storing set of points where the respective functions are finite
  //y1: real number for which the function above is finite (or %nan if such number does not exist)
  if lambda1==1 | lambda2==1 then y1=x;
  else
    I1=domf1;I2=domf2;J=domg;
    //f1(y1) finite requires y1 in I1
    K=I1;
    //f2(y2) finite requires y1 in (x-lambda2*I2)/lambda1
    t=(x-lambda2*I2)/lambda1;//need to revert bound to get true interval
    K=intervalIntersect(K,[t(2),t(1)]);
    //g(y1-y2) finite requires y1 in x+lambda2*J
    K=intervalIntersect(K,x+lambda2*J);
    if intervalisEmpty(K) then y1=%nan
    else
      //project xstart onto interval K
      if xstart < K(1) then
        y1 = K(1);
      elseif xstart > K(2) then
        y1 = K(2);
      else
        y1 = xstart;
      end
    end    
  end
endfunction