//NEED A NONCONVEX NONSMOOTH SOLVER (Neldermead in Scilab 5.2.0:  fminsearch with infinite penalization
//or neldermead with Box complex algorithm). BUT optim WON'T WORK SINCE 'nd' IS A BUNDLE METHOD (only works
//for convex functions)
function [y,argmin] = opt_co(f,df,domf,x,output)
  //computes the convex hull of f evaluated on vector x
  //f: univariate function continuous but (potentially) nonsmooth
  //df: univariate function: derivative of f: df=f'
  //domf: domain of function f (set of points where the function is finite)
  //domains are stored as intervals I=[lb,ub] with ub<lb storing an empty set
  //WARNING: if the domain is wrong i.e. there are x in domf for which f(x)=%inf
  //then a search for a starting point needs to be implemented (simpler than _pl_kaFeasiblePoint above)
  //x: vector; points at which to evaluate the convex hull
  //output: 0: no output, 1: progress bar
  // y = min_{x1,x2,lambda} {(1-lambda) f(x1) + lambda f(x2) : lambda >=0, (1-lambda) x1 + lambda x2 = x }
  //y: vector of the same dimension as x: y = co f(x)
  //x1, x2 are vectors of the same dimension as x
  //lambda: vector of the same dimension as x
  //argmin: length(x) x 2  matrix. argmin = [x2;lambda]
  //then recover lambda1=1-lambda;lambda2=lambda;x1=(x-lambda2*x2)/lambda1
  if argn(2)<5 then output=0;end;
  n=size(x,1);eps=1E-6;
  X0=[x(1);0];//start initially from x
  if output>0 then beginProgress();end;
  for i=1:n,    
    //for PLQ functions, dom co f is a subset dom f (with equality as soon as co f is proper)
    if x(i) < domf(1)-eps | x(i) > domf(2)+eps then
      y(i,1) = %inf; argmin(i,:) = [%nan,%nan];
    else
      //project to boundary up to eps precision
      if x(i) < domf(1) then x(i)=domf(1);if X0(1) < domf(1) then X0(1)=x(i);end;end;
      if x(i) > domf(2) then x(i)=domf(2);if X0(1) > domf(2) then X0(1)=x(i);end;end;

      [fopt,Xopt,gopt] = optim(list(_costf_co,x(i)),X0,'nd');//,'ar',100,100,1E-3,1E-3
      //[y(i,1),argmin(i,:),gopt]=[A,B,C];
      y(i,1)=fopt;
      argmin(i,:)=Xopt';

      //printf("i=%i, fopt=%f, gopt=%f, argminopt=%f\n",i,y(i,1),gopt,argmin(i,1));
      if norm(gopt)>eps then //gradient nonzero but because it is a cutting plane like method we are still at the optimum
        //y(i,1)=%inf;argmin(i,1)=%nan;
      end;
      //printf("--------\ni=%i, fopt=%f, gopt=%f, argminopt=%f\n",i,y(i,1),gopt,argmin(i,1));
      X0=argmin(i,:)';//warmstart
    end
    if output>0 then showProgress(i/n);end;
  end
  if output>0 then finishProgress();end;
endfunction