function [y,argmin] = opt_ka(f1,df1,domf1,f2,df2,domf2,g,dg,domg,x,lambda,output)
  //computes the (kernel average) g-average of f1 and f2 evaluated on vector x
  //f1, f2, g: univariate functions continuous but (potentially) nonsmooth
  //df1, df2, dg: univariate functions: derivatives of f1 df1=f1', f2 df2=f2', and dg=g'
  //domf1, domf2, domg: domain of functions f1, f2, g (set of points where the function is finite)
  //domains are stored as intervals I=[lb,ub] with ub<lb storing an empty set
  //x: vector; points at which to evaluate the kernel average
  //lambda: real number lambda1=1-lambda;lambda2=lambda;
  //output: 0: no output, 1: progress bar
  // y = inf_{lambda1 y1+lambda2 y2=x} [lambda1 f1(y1) + lambda2 f2(y2) + lambda1 lambda2 g(y1-y2)]
  // argmin = argmin_{lambda1 y1+lambda2 y2=x} [lambda1 f1(y1) + lambda2 f2(y2) + lambda1 lambda2 g(y1-y2)]
  if argn(2)<12 then output=0;end;
  n=size(x,1);lambda1=1-lambda;lambda2=lambda;
  xstart=x(1);
  if output>0 then beginProgress();end;
  for i=1:n,    
    if isnan(xstart) then xstart=x(i);end;
    x0 = _pl_kaFeasiblePoint(xstart,x(i),lambda1,lambda2,domf1,domf2,domg)//find a feasible x0 i.e. point for which the objective is finite

    if isnan(x0) then 
      y(i,1)=%inf;argmin(i,1)=%nan;
    else      
      [y(i,1),argmin(i,1),gopt]=optim(list(_costf_ka,x(i),lambda),x0,'nd');//,'ar',100,100,1E-3,1E-3
      //printf("i=%i, fopt=%f, gopt=%f, argminopt=%f\n",i,y(i,1),gopt,argmin(i,1));
      if abs(gopt)>1E-6 then //gradient nonzero but because it is a cutting plane like method we are still at the optimum
        //y(i,1)=%inf;argmin(i,1)=%nan;
      end;
      //printf("--------\ni=%i, fopt=%f, gopt=%f, argminopt=%f\n",i,y(i,1),gopt,argmin(i,1));
      xstart=argmin(i,1);//warmstart
    end
    if output>0 then showProgress(i/n);end;
  end
  if output>0 then finishProgress();end;
endfunction