function [y,argmin] = opt_lft(f,df,domf,s,output)
  //computes the Legendre-Fenchel Transform of f evaluated on vector x
  //f: univariate function continuous but (potentially) nonsmooth
  //df: univariate function: derivatives of f1 df1=f1'
  //domf: domain of function f1 (set of points where the function is finite)
  //domains are stored as intervals I=[lb,ub] with ub<lb storing an empty set
  //s: vector; points at which to evaluate the Legendre-Fenchel transform
  //lambda: real number
  //output: 0: no output, 1: progress bar
  // y = f^*(s) = sup_x [ s * x - f(x) ]
  // argmin = argmin_y [ s * x - f(x) ]
  if argn(2)<5 then output=0;end;
  n=size(s,1);
  xstart=s(1);
  if output>0 then beginProgress();end;
  for i=1:n,
    if isnan(xstart) then xstart=s(i);end;
    //can use _pl_kaFeasiblePoint since only need a point in domf (first constraint in _pl_kaFeasiblePoint)
    //and discard the two other constraints by putting the real line. Return the projection of 
    //xstart on domf
    x0 = _pl_kaFeasiblePoint(xstart,s(i),0.5,0.5,domf,[-%inf,%inf],[-%inf,%inf])//find a feasible x0 i.e. point for which the objective is finite

    if isnan(x0) then 
      y(i,1)=%inf;argmin(i,1)=%nan;
    else      
      [y(i,1),argmin(i,1),gopt]=optim(list(_costf_lft,s(i)),x0,'nd');//,'ar',100,100,1E-3,1E-3
      //compute sup not inf so invert sign
      y(i,1)=-y(i,1);
      //printf("i=%i, fopt=%f, gopt=%f, argminopt=%f\n",i,y(i,1),gopt,argmin(i,1));
      if abs(gopt)>1E-6 then //gradient nonzero but because it is a cutting plane like method we are still at the optimum
        //y(i,1)=%inf;argmin(i,1)=%nan;
      end;
      //printf("--------\ni=%i, fopt=%f, gopt=%f, argminopt=%f\n",i,y(i,1),gopt,argmin(i,1));
      xstart=argmin(i,1);//warmstart
    end
    if output>0 then showProgress(i/n);end;
  end
  if output>0 then finishProgress();end;
endfunction