function _plq_plotEnv(envFlag,L,x,lset,r,rect,isConvex,animate,generateGIF,gifFilename)
//internal function called by plq_plotpa (parameter check done in caller function)
//plot the graph of (x,lambda)->pa(f,lambda) in the current graphic window
//2d graph with colors for each value of lambda
//L: list. If envFlag=='pa' or 'gpa' then L is a list of 2 plq functions. plq0=L(1);plq1=L(2);
//         If envFlag=='ka' then L is a list of 4 (Scilab) functions: f1=L(1),df1=L(2),f2=L(3),df2=L(4),g=L(5),dg=L(6)
//x: column vector
//lset: row vector; lset is the lambda set i.e. the values of lambda from 0 to 1
//rect: rect option passed to plot2d
//isConvex: %t (default) if functions are convex (faster to compute)
//animate: %t shows curve as they are plotted (default %f)
//generateGIF uses ImageMagicks from www.ImageMagick.org to create an animated GIF
//gifFilename: name of the GIF file (default: animate.gif) created in the current directory

  if generateGIF then
    //clean up any *.gif file in TMPDIR
    stat=unix("rm "+TMPDIR+"/*.gif");
    //add .gif if not already in the name
    addSuffix=%f;
    if length(gifFilename)<3 then addSuffix=%t;end;
    if ~addSuffix then
      suffix=strsplit(gifFilename,length(gifFilename)-3);
      if suffix($)~="gif" then addSuffix=%t;end;
    end;
    if addSuffix then gifFilename=gifFilename+".gif";end;
  end
  
  if size(x,2) >1 then x=x';end;
  if size(lset,1) >1 then lset=lset';end;
  //clf();
  th=1;fig=gcf();

  if ~generateGIF then drawlater();  end
  nc=max(round(length(lset)/100),100);//number of colors
  cm=[jetcolormap(nc)];
  fig.color_map = cm;
  colorbar(0,1);//h=gce();h.parent.font_size=4;

  i=1;stp=ceil(length(lset)/10);
  beginProgress();
  for lambda=lset,
      if envFlag=="pa" then
        pa=plq_pa(L(1),L(2),lambda,isConvex);
        y=plq_eval(pa,x);        
      elseif envFlag=="gpa" then
        pa=plq_gpa(L(1),L(2),lambda,r);
        y=plq_eval(pa,x);                
      elseif envFlag=="ka" then
        y=opt_ka(L(1),L(2),L(3),L(4),L(5),L(6),x,lambda);
      else
        error("Unrecognized option: Flag %s is not a valid flag for envFlag in _plq_plotEnv",envFlag);
      end      

      plot2d(x,y,style=ceil(lambda*nc)+1,rect=rect);//lambda no longer equispaced so cannot use style=i
      if generateGIF then
        s=msprintf("%s/img%03.0f.gif",TMPDIR,i);
        //if i==1 then printf("%s\n",s);pause;end;
        xs2gif(fig.figure_id,s);
      end
      //if (i==1) then h=gca(); h.font_size=4; end;
      if modulo(i,stp)==0 then
    	    showProgress(i/length(lset));
    	    if animate & ~generateGIF then drawnow();drawlater();end;
      end;
      i=i+1;
  end;
  finishProgress();
  if ~generateGIF then 
    drawnow();
  else 
    //requires ImageMagicks from www.ImageMagick.org
    stat=unix("convert -delay 10 "+TMPDIR+"/*.gif "+gifFilename);
  end;  
endfunction