///////////////FIRST ORDER FUNCTION APPROXIMATION///////////////
function plqf = plq_build(x,f,df,needEval,exclude,method,extrapolate)
//PLQ function = piecewise linear going through (x,f(x)) with derivative
//df(x) at (x,f(x)) i.e. 1st order approximation of f  
//needEval==%f or not entered means the function is vectorized; otherwise
//evaluate point by point
//exclude: ??
//method: "plq"  normal PLQ approximation
//        "soqs" Shape-Preserving Osculatory Quadratic Spline approximation
//extrapolate: 'constant' or 'bounded' to extend outside of domain by constant or %inf
//NOTE: feval(x,exp) returns an error, but function
//y=f(x),y=exp(x);endfunction;feval(x,f) works
  
  if (argn(2) < 3) then df = %f; dy = %f; end;
  if (argn(2) < 4) then needEval = %f; end;  // Assume function is vectorized.
  if (argn(2) < 5) then exclude = %f; end;
  if (argn(2) < 6) then method = 'plq'; end;
  if (argn(2) < 7) then extrapolate = 'constant'; end;
  
  if size(x,1) < size(x,2) then x=x'; end;
  dfType = typeof(df);

  if (needEval) then
    try, y=feval(x,f);
    catch, y=f(x); end;
    if (dfType=="fptr" | df~=%f) then
      try, dy=feval(x,df);
      catch, dy=df(x); end;
    else
      dy = %f;
    end;
  else
    try, y=f(x);
    catch, y=feval(x,f);
    end;
    if (dfType=="fptr" | df~=%f) then
      try, dy=df(x);
      catch, dy=feval(x,df); end;
    else
      dy = %f;
    end;
  end;

  if method == 'soqs' then
    if (dy==%f) then
      dy = _soqs_slopes(x,y);
    end;
    plqf = _soqs_build(x,y,dy,extrapolate);
  else//anything else default to 'plq'
    plqf = plq_build_d(x,y,dy,exclude);
  end;
endfunction

