function plqf = plq_build_d(x,y,dy,exclude)
  //build PLQ function which is piecewise linear using the points (x(i),y(i)) and maybe dy(i)

  if (argn(2) < 3) then dy = %f; end;
  if (argn(2) < 4) then exclude = %f; end;

  infl = %f;
  infr = %f;
  dyType = typeof(dy);
  if (dyType~="fptr" & dy==%f) then
    if (exclude) then
      while (y(1) == %inf), infl = %t; x = x(2:$);   y = y(2:$);   end;
      while (y($) == %inf), infr = %t; x = x(1:$-1); y = y(1:$-1); end;
    end;
    dy = (y(2:$) - y(1:$-1)) ./ (x(2:$) - x(1:$-1));
    dym=[dy(1);dy(1:$)];//left derivative at (xm,ym)
    plqf=[x,zeros(x),dym,y-dym.*x];
    plqf=[plqf;%inf,0,dy($),y($)-dy($)*x($)];//careful PLQ is represented with a x^2+b x + c
    if (exclude) then
      if (infl) then plqf(1,2:4) = [0,0,%inf]; end;
      if (infr) then plqf($,:) = [%inf,0,0,%inf]; end;
    end;
  else
    //disp([x,y,dy], "initial [x,y,dy]:");
    while (y(1) == %inf | dy(1) == %inf), infl = x(1); x = x(2:$);   y = y(2:$);   dy = dy(2:$);   end;
    while (y($) == %inf | dy($) == %inf), infr = x($); x = x(1:$-1); y = y(1:$-1); dy = dy(1:$-1); end;
    //disp([x,y,dy], "infs removed [x,y,dy]:");
    //when dy(i)==dy(i+1) remove the point x(i) since it does not give any 1st order info
    ni = find(dy(2:$)-dy(1:$-1)~=0);
    ni=[ni,length(x)];//always keep the rightmost point
    x=x(ni);y=y(ni);dy=dy(ni);
    //disp([x,y,dy], "dups removed [x,y,dy]:");
    //(xm,ym) = intersection of tangent at (x(i),y(i)) and at (x(i+1),y(i+1))
    xm=(y(1:$-1)-y(2:$)+dy(2:$).*x(2:$)-dy(1:$-1).*x(1:$-1))./(dy(2:$)-dy(1:$-1));
    ym=y(1:$-1)+dy(1:$-1).*(xm-x(1:$-1));
    dym=dy(1:$-1);//left derivative at (xm,ym)
    //disp([xm,ym,dym], "intersection of tangents [xm,ym,dym]:");
    plqf=[xm,zeros(xm),dym,ym-dym.*xm;%inf,0,dy($),y($)-dy($)*x($)];//careful PLQ is represented with a x^2+b x + c
    //handle %inf values or derivatives
    if (infl ~= %f) then
      if (exclude) then
        plqf = [x(1),0,0,%inf; plqf];
      else
        plqf = [infl,0,0,%inf; plqf];
      end;
    end;
    if (infr ~= %f) then
      if (exclude) then
        plqf($,1) = x($);
      else
        plqf($,1) = infr;
      end;
      plqf = [plqf; %inf,0,0,%inf];
    end;
  end;
  plqf=plq_clean(plqf);//clean up, remove linear parts
endfunction