function [plqic, argmin] = plq_infconv_brute(plqf, plqg, swapArgmin, plotArgmin)
  // Computes the plq_min of the convolutions, as computed by
  // plq_infconv_brute_d(), for all nf*ng pairs of (f,g) pieces.
  // Assumes plqf and plqg are not point indicator functions.
  // Simply ignores non-finite pieces.

  if (argn(2) < 3) then swapArgmin = %t; end;
  if (argn(2) < 4) then plotArgmin = %f; end;

//removed: plq_is_convex is not implemented so crashes unit tests
//why do we need to swap arguments if plqg is not convex anyway??
//  if (~plq_is_convex(plqg)) then
//	temp = plqf;
//	plqf = plqg;
//	plqg = temp;
//  end;

  calcArgmin = (argn(1) >= 2);
  plqic = [%inf, 0, 0, %inf];
  argmin = [%inf, 0, 0, %inf];
  nf = size(plqf, 1);
  ng = size(plqg, 1);

  for i = 1:nf
    if (plqf(i,4) == %inf) then continue; end;

    for j = 1:ng
      if (plqg(j,4) == %inf) then continue; end;
      if (i == 1) then u(1) = -%inf; else u(1) = plqf(i-1,1); end;
      if (j == 1) then u(2) = -%inf; else u(2) = plqg(j-1,1); end;
      v(1) = plqf(i,1);
      v(2) = plqg(j,1);

      //printf("plq_infconv2_brute: i==%d, j==%d\n", i, j);
      [plqh, lb, amin] = plq_infconv_brute_d(plqf(i,:), plqg(j,:), u, v, swapArgmin);
      plqh = [lb,0,0,%inf; plqh; %inf,0,0,%inf];

      if (~calcArgmin) then
        plqic = plq_min(plqic, plqh);
      else
        amin = [lb,0,0,%inf; amin; %inf,0,0,%inf];
        //plq_plotMultiple(-3,15,plqh);
        //printf("%f\n", plqh($-1,1));
        //disp(amin);

        // Update the proximal mapping.
        // Where replace the intervals in the current argmin
        // where minf(:,1)==2 with the new argmin.
        [plqic, minf] = plq_min(plqic, plqh);
        bounds = minf(find(minf(:,1)==2),2:3)';
        for bound = bounds
          argmin = plq_replace(argmin, amin, bound(1), bound(2));
        end;

        if (plotArgmin) then
          disp(argmin);
          clf();
          plq_plotMultiple(-5, 5, argmin, amin);
          if (plqf(i,3) < plqg(j,3)) then
            xtitle(sprintf("(%d,%d) swapped", i, j));
          else
            xtitle(sprintf("(%d,%d)", i, j));
          end;
          scanf("%s");
        end;
      end;
    end;
  end;

  if (calcArgmin) then
    argmin = plq_clean(argmin);
  end;
endfunction