function T = _sheppard_interpn(xyz,x,y,rect)

    verbose = 0;
    
    // remove points outside rectangle
    ky = find(xyz(:,2)<=rect(4));
    xyz=xyz(ky,:);

    instr="S2d = cshep2d(xyz);"
    ierr=execstr(instr,'errcatch');


    if ierr>0 then
    
        //some points are too close: data is ill-conditioned
        xyz=lex_sort(xyz,1:2);
        dxyz=xyz(2:$,2)-xyz(1:$-1,2);
        n=size(xyz,1);
                           
        cratio = 400*(2.e3/n); // conditioning ratio (do not change !!)
        radius = max(max(abs(dxyz))/cratio, 0.027407);  // radius of influence of points

        if verbose == 1 then
            printf("\nradius of influence: %f nodes: %i\n",radius,size(xyz,1));
            printf("max dist: %f, min dist: %f\n",max(abs(dxyz)),min(abs(dxyz)));
        end

        // FIXME: try to replace the while loop with vectorization
        i = 2;
        nxyz = xyz(1,:);
        curx = nxyz(1,1);
        while i < n do
            if abs(nxyz($,2)-xyz(i,2)) >= radius then
                nxyz($+1,:)=xyz(i,:);
            end    
            i = i+1;
        end
        if abs(nxyz($,2)-xyz($,2)) < radius then
            nxyz($,:)=xyz($,:);
        else
            nxyz($+1,:)=xyz($,:);
        end        

        if verbose == 1 then
            dxyz=nxyz(2:$,2)-nxyz(1:$-1,2);                
            printf("new max dist: %f, new min dist: %f\n",max(abs(dxyz)),min(abs(dxyz)));                    
            d2xyz=nxyz(2:$,1)-nxyz(1:$-1,1);
            printf("max x distance: %g, min x distance: %g\n",max(abs(d2xyz)),min(abs(d2xyz)));
            scf(ny-9);clf();subplot(1,2,1);
            plot2d(xyz(:,1),xyz(:,2),style=-1,rect=rect);
            xtitle("dy="+string(min(nxyz(:,3)))+" n="+string(ny-1));
            warning("min: "+string(min(nxyz(:,3))));                                
            warning("plq_pa_Y: removed "+string(size(xyz,1)-size(nxyz,1))+" nodes left "+string(size(nxyz,1))+" (for cshep2d");
        end
    
        xyz=nxyz;
        xyz=lex_sort(xyz,1:2);

        if verbose == 1 then
            subplot(1,2,2);                  
            plot2d(xyz(:,1),xyz(:,2),style=-1,rect=rect);
            xtitle(string(min(xyz(:,3))));
        end
    
        ierr=execstr(instr,'errcatch');                  
    end
    if ierr == 0 then                
        //evaluate on regular grid x,y
        [xp,yp] = ndgrid(x,y);
        // printf("ndgrid fine\n");
        T = eval_cshep2d(xp,yp,S2d);
    else
        warning(lasterror());                  
        T = [];
    end                
endfunction