function [T,y] = plq_pa_Y(x,lambda,Y,ny,rect,interpolationType)
    //TO DO: 
    // - remove finiteness assumption.
    // - remove f0 < f1 assumption
    // - fix boundaries issues. May need to dig into Sgrayplot to remove interpolation on boundary.
    // - use soqs for interpolation instead of interpln
    // - use derivative in soqs instead of inferring derivatives within the  interpolation: 
    //does that change significantly the quality?
    //Then 
    // - compare speed with plq_plotpa
    // - compare quality with plq_plotpa (difference of matrices)

    //compute T(j,i)=lambda(k) such that y(j)=f_lambda(k)(x(i))
    //  x: column vector
    //  lambda: column vector
    //  Y: matrix of size nlambda x nx
    //  ny: integer: number of points on the y-axis to be computed (regularly grid)

    _interpolationTypes=["SOQS","Linear","2D smooth scattered","2D linear interpolate_grid","2D linear grid2Ddata","Exact"];
    //printf("Using %s\n",_interpolationTypes(interpolationType));

    nx=length(x);nlambda=length(lambda);
    if size(Y)~=[nlambda,nx] then 
       error("plq_pa_Y: size(Y) inconsistent with length(x) and length(lambda)");
    end;

    // We don't use +/- infinity
    //maxim = max(Y);
    //minim = min(Y);
    //if maxim == %inf then
    //maxim = max((Y~=%inf).*Y);   
    //end
    //if minim == -%inf then
    //minim = min((Y~= -%inf).*Y);
    //end

    minim=rect(2);maxim=rect(4);y=linspace(minim,maxim,ny)';
    //T=zeros(ny,nx);  
    T=-ones(ny,nx);  

    if interpolationType <=2 then//univariate interpolation
        //Compute the values at points y by interpolation using values at Y(:,i)
        for i=1:nx
            [v,idx]=gsort(Y(:,i),'r','i');
            pts=[v';lambda(idx)'];	    
            //do not extrapolate only interpolate
            j=find((y>=v(1)) & (y<=v($)));//vector of indeces to compute
            if length(j) > 0 then
                if (interpolationType==1) then//use soqs
                    if (or(isinf(pts))) then
                        dy = _soqs_slopes(pts(1,1:$-1),pts(2,1:$-1));
                        pf = _soqs_build(pts(1,1:$-1),pts(2,1:$-1),dy);
                    else
                        dy = _soqs_slopes(pts(1,:),pts(2,:));
                        pf = _soqs_build(pts(1,:),pts(2,:),dy);			      
                    end;
                    T(j,i)=plq_eval(pf,y(j));
                else
                    T(j,i)=interpln(pts,y(j))';
                end;
            end
        end
    else//bivariate interpolation
        //perform bivariate interpolation using the scattered
        //data x,lambda,Y to evaluate on the regular grid x,y,T
        X = ones(nlambda,1) .*. x;//see help kron for detail
        L = lambda .*. ones(nx,1);
        YY = Y';
        xyz = [X YY(:) L];            
        xyz(find(xyz==%inf))=maxim+1;//substitute %inf values

        select interpolationType          
            case 3,
                //bidimensional scattered Sheppard cubic interpolation 
                T = _sheppard_interpn(xyz,x,y,rect);
            case 4,
                //generate a regular grid from scattered input data using quadratic 2D interpolation
                scaling_factor=1;
                XYL = interpolate_grid(xyz,scaling_factor);
                [xp,yp] = ndgrid(x,y);
                //bivariate linear interpolation
                T = linear_interpn(xp,yp, XYL(1),XYL(2), XYL(3));
            case 5,
                //regrid the scattered points into a regular grid then use regular interpolation
                //with grid2Ddata (enrico toolbox)
                T = grid2Ddata(xyz,x',y');
            case 6,
                T = [];      
            else
                [str,n,line,func]=lasterror();
                error(sprintf("Error: undefined interpolationType=%i\n",interpolationType));
        end
        T=T';
    end
endfunction