mode(-1);
__test_set = "PLQ: Plotting functions";

// A test file for PLQ plotting functions.

// How we're actually supposed to test a plotting function,
// I (Bryan Gardiner) am not sure.  As long as it doesn't
// error, the test succeeds?  We could do introspection on
// the plot, but that data could change.

function b = testPlot1()
  b = %f;
  plqf = [-1,0,0,1; 1,1,0,0; %inf,0,2,-1];
  while (winsid() ~= []); xdel(); end;
  plq_plot(plqf);
  while (winsid() ~= []); xdel(); end;
  b = %t;
endfunction

function b = testPlot2()
  b = %f;
  plqf1 = [-1,0,0,1; 1,1,0,0; %inf,0,2,-1];
  plqf2 = [0,0,1,4; %inf,0,-1,4];
  while (winsid() ~= []); xdel(); end;
  plq_plot(plqf1, plqf2);
  while (winsid() ~= []); xdel(); end;
  b = %t;
endfunction

function b = testPlot3()
  b = %f;
  plqf = [-4,0,-1/2,2; -1,1,4,4; 0,0,0,1; 1,0,-2,1; 2,0,3,-4; %inf,1,-6,10];
  plqco = [
      -2.25,0,-0.5,-1.0625;
      -2.1622777,1,4,4; ...
      1,0,-0.3245553,-0.6754447; ...
      3.4494897,0,0.8989795,-1.8989795; ...
      %inf,1,-6,10];
  while (winsid() ~= []); xdel(); end;
  plq_plot(plqf, plqco);
  while (winsid() ~= []); xdel(); end;
  b = %t;
endfunction

function b = testPlotMultiple()
  b = %f;
  f1 = [-4,0,-1/2,2; -1,1,4,4; 0,0,0,1; 1,0,-2,1; 2,0,3,-4; %inf,1,-6,10];
  f2 = [-2.25,0,-0.5,-1.0625;
        -2.1622777,1,4,4; ...
        1,0,-0.3245553,-0.6754447; ...
        3.4494897,0,0.8989795,-1.8989795; ...
        %inf,1,-6,10];
  f3 = [0,0,-1/5,4; 3,0,1/2,0; %inf,0,1/4,2];
  while (winsid() ~= []); xdel(); end;
  plq_plotMultiple(%f, %f, f1, f2, f3);
  while (winsid() ~= []); xdel(); end;
  b = %t;
endfunction

function b = testPlotPA()
  b = %f;
  while (winsid() ~= []); xdel(); end;

  // Code taken from demos/nl2nl.sci:

  function y = f1(x)
    i1 = find(x<=0); y(i1) = %inf*ones(i1);
    i2 = find(x>0);  y(i2) = -log(x(i2));
  endfunction
  function dy = df1(x)
    i1 = find(x<=0); dy(i1) = %inf*ones(i1);
    i2 = find(x>0);  dy(i2) = -x(i2)^(-1);
  endfunction
  x1=linspace(0,10,50)'; f1plq=plq_build(x1,f1,df1); [y1,k]=plq_eval(f1plq,x1); //plot2d(x1,y1);

  function y = f2(x)
    i1 = find(x<0);  y(i1) = -log(-x(i1));
    i2 = find(x>=0); y(i2) = %inf;
  endfunction
  function dy = df2(x)
    i1 = find(x<0);  dy(i1) = -x(i1)^(-1);
    i2 = find(x>=0); dy(i2) = %inf;
  endfunction
  x2=linspace(-10,0,50)'; f2plq=plq_build(x2,f2,df2); [y2,k]=plq_eval(f2plq,x2); //plot2d(x2,y2);

  lset = linspace(0,1,5);
  rect = [-5,-1.5,5,2.5];
  xx = linspace(-3,3,10)';
  plq_plotpa(f1plq, f2plq, xx, lset, rect);

  while (winsid() ~= []); xdel(); end;
  b = %t;
endfunction

function b = testPlotPALambdaSet()
  b = %f;
  while (winsid() ~= []); xdel(); end;

  f1 = [-2,0,1,6;-1,0,-8,-12;0,0,8,4;%inf,0,0,%inf];
  f2 = [0,1/2,0,0;%inf,-1/2,0,0];
  x = -14:2:30;
  lset = 0:0.2:1;
  mu = 0.5;
  rect = [-15,-5,30,20];
  plq_plotpa_lambdaSet(f1, f2, x, lset, mu, rect);

  while (winsid() ~= []); xdel(); end;
  b = %t;
endfunction

function b = testPlotPAMuSet()
  b = %f;
  while (winsid() ~= []); xdel(); end;

  f1 = [-2,0,1,6;-1,0,-8,-12;0,0,8,4;%inf,0,0,%inf];
  f2 = [0,1/2,0,0;%inf,-1/2,0,0];
  x = -14:2:30;
  lambda = 0.5;
  muset = 0.2:0.2:1;
  rect = [-15,-5,30,20];
  plq_plotpa_muSet(f1, f2, x, lambda, muset, rect);

  while (winsid() ~= []); xdel(); end;
  b = %t;
endfunction

function b = runTestFile()
  b = %t;
  b = checkForFail(testWrapper(testPlot1, "testPlot1"), b);
  b = checkForFail(testWrapper(testPlot2, "testPlot2"), b);
  b = checkForFail(testWrapper(testPlot3, "testPlot3"), b);
  b = checkForFail(testWrapper(testPlotMultiple, "testPlotMultiple"), b);
  b = checkForFail(testWrapper(testPlotPA, "testPlotPA"), b);
  b = checkForFail(testWrapper(testPlotPALambdaSet, "testPlotPALambdaSet"), b);
  b = checkForFail(testWrapper(testPlotPAMuSet, "testPlotPAMuSet"), b);
endfunction
