mode(-1);
//only perform unit testing. Faster than coverage testing.
//exec test+cov.sci to perform unit AND coverage testing.
//exec test.sci for a menu.
tic();
clear;
global CCADIR;   // Bring back CCADIR if the user was using it.

function [name] = getMonth(month)
	select month
		case 1 then name = "January";
		case 2 then name = "February";
		case 3 then name = "March";
		case 4 then name = "April";
		case 5 then name = "May";
		case 6 then name = "June";
		case 7 then name = "July";
		case 8 then name = "August";
		case 9 then name = "September";
		case 10 then name = "October";
		case 11 then name = "November";
		case 12 then name = "December";
	end
endfunction



// user set globals
global __test_file;
global __test_exclude;

// automated globals
global __test_set;
global __test_failures;
global __test_passes;
global __base_dir;
global __test_dir;
global __test_coverage;
global __macros_dir;


// define base directory
__cwd_dir = pwd();
__test_dir = get_absolute_file_path('t.sci');
chdir(__test_dir+"..");
__base_dir = pwd();
__macros_dir = pathconvert(__base_dir + '/macros/');
chdir(__test_dir);

// Disable coverage testing
__test_coverage = %f;

//Number of functions and libraries loaded
// chdir(__base_dir+"/c_routines/");
// exec('dt_c.sce');
//chdir(__base_dir);
chdir(__macros_dir);
CCA_lib = ['common_version.sci','me_version.sci','lft_version.sci','plq_version.sci','op_version.sci','gph_version.sci','opt_version.sci','soqs_version.sci']';
CCA_lib_untested = ['algo_version.sci','plot_version.sci'];


directories = [ ..
    "algo"              ; .. 
    "common"            ; .. 
    "copa_compute"      ; .. 
    "copa_display"      ; .. 
    "gph"               ; .. 
    "gph_plot"          ; ..
    "grid2Ddata"        ; ..
    "interpolate_grid"  ;  
    "lft"               ;
    "lft_llt"           ;
    "me"                ; ..
    "op"                ; 
    "opt"               ; .. 
    "plot"              ; .. 
    "plq"               ; 
    "plq_pa_Y"          ;
    "progress_bar"      ;  
    "soqs"             ];.. 
    
directories = pathconvert(__macros_dir+directories);
if (~__test_coverage) then
  Nb_fcts_scilab = size(who('local'),1);
  for i=1:size(directories,"*")
      chdir(directories(i))
      x = ls('*.sci')
      for j=1:size(x,"*")
          exec(x(j),-1)
      end
  end
  Nb_fcts_CCA = size(who('local'),1) - Nb_fcts_scilab
end

chdir(__test_dir);

// this must be defined at the unit test level
__test_set  = "N/A";
__test_failures = 0;
__test_passes = 0;

// set up the log file
__test_filename = pwd()+'/test_CCA.log';//xgetfile("*", "/home",'Please select the location for the log file.');
__test_output = mopen(__test_filename,"w");

// write the header
__test_date = getdate();
fprintf(__test_output, "Scilab Unit Test Log :: %s %d, %d\n", getMonth(__test_date(2)), __test_date(6), __test_date(1));
fprintf(__test_output, "==============================================================================\n");
fprintf(__test_output, "Set Name :: Test Name ................................................[Result]\n");
fprintf(__test_output, "------------------------------------------------------------------------------\n");

// extremely useful for checking if a test failed
// the first call to this must be bool = checkForFail(check, %t), and then following calls
// must then be bool = checkForFail(check, bool)
function [b] = checkForFail(bool, lastBool)
	if (bool == %f | lastBool == %f)
		b = %f;
	else
		b = %t;
	end
endfunction

// In each unit test file's runTestFile() function,
// if b is the boolean representing the success of
// the function, testWrapper should be called like:
// b = checkForFail(testWrapper(testFoo, "testFoo"), b);
// The test succeeds if testFoo returns logical true,
// fails if testFoo returns logical false or errors.
function [b] = testWrapper(testFunc, __test_name)
	global __test_failures;
	global __test_passes;
	r = %f;
	
	errcatch(-1,"continue");  // catch all errors
		r = testFunc();
	if (iserror(-1))
		status = "[ERROR!]"
		__test_failures = __test_failures + 1
		b = %f
	elseif (r)
		status = "[Passed]"
		__test_passes = __test_passes + 1
		b = %t
	else
		status = "[FAILED]"
		__test_failures = __test_failures + 1
		b = %f
	end
	errclear(-1);
	
	line = msprintf("%s :: %s ", __test_set, __test_name);
	while (length(line) < 70), line = msprintf("%s.",line); end;
	
	mfprintf(__test_output, "%s%s\n", line,status);
endfunction

// This function acts like testWrapper, except the
// test succeeds if the function errors, and fails
// if the test returns without erroring.  It is used
// to test for error cases of functions.
function [b] = errorTestWrapper(testFunc, __test_name)
  global __test_failures;
  global __test_passes;
  global __testing_errors;

  __testing_errors = 10001;
  errcatch(-1, "continue", "nomessage");
  testFunc();
  b = iserror(10001);
  __testing_errors = %f;

  if (b) then
    status = "[Passed]";
    __test_passes = __test_passes + 1;
  elseif (iserror(-1)) then
	status = "[ERROR!]";
	__test_failures = __test_failures + 1;
  else
    status = "[FAILED]";
    __test_failures = __test_failures + 1;
  end;

  errclear(-1);

  line = msprintf("%s :: %s ", __test_set, __test_name);
  while (length(line) < 70), line = msprintf("%s.",line); end;

  mfprintf(__test_output, "%s%s\n", line,status);
endfunction

// This function works like errorTestWrapper, but
// when an error occurs, Scilab pauses instead of
// continuing.  This can be used to debug an error
// condition when Scilab freezes instead of returning
// an error cleanly in an error test.
function [b] = breakTestWrapper(testFunc, __test_name)
  global __test_failures;
  global __test_passes;
  global __testing_errors;

  __testing_errors = 10001;
  errcatch(-1, "pause");
  testFunc();
  b = iserror(10001);
  __testing_errors = %f;

  if (b) then
    status = "[Passed]";
    __test_passes = __test_passes + 1;
  elseif (iserror(-1)) then
	status = "[ERROR!]";
	__test_failures = __test_failures + 1;
  else
    status = "[FAILED]";
    __test_failures = __test_failures + 1;
  end;

  errclear(-1);

  line = msprintf("%s :: %s ", __test_set, __test_name);
  while (length(line) < 70), line = msprintf("%s.",line); end;

  mfprintf(__test_output, "%s%s\n", line,status);
endfunction

// run tests - note that these script files will run their tests through function 
// testWrapper already
function [b,testFiles] = runTests()
	// if the user defined __test_file, we'll just test that file
	global __test_file;
	global __test_exclude;
	//displays the name of the unit test file being run
	//useful to know which file is failing when scilab freezes
	isDebug=%f;
	
	if (length(__test_file) == 0)
		if getos() == "Windows"
			fileList=ls("test_*.sc?");
		else
			fileList=ls("test_*.sc?","-1");
		end;
	else
		fileList=__test_file;
	end
	
        msg = "Running unit tests";
    beginProgress(%f, msg);
	b=%t;
    testFiles = size(fileList,1);
    
	for curFileNum=1:size(fileList,1)
		curFile=fileList(curFileNum)
		curFile=stripblanks(curFile);
		// if not excluded, execute test files
		if (length(find(__test_exclude == curFile)) == 0)
			clear runTestFile;
			if isDebug then printf("Testing: %s",curFile);tic();end;
			exec(curFile,-1);
			if isDebug then printf("...");end;
			results = runTestFile();
			if isDebug then printf("Done in %2.2f\n",toc());
            else showProgress(curFileNum/testFiles, %f, msg);
            end;  

           b = checkForFail(results, b);
		
			if (curFileNum < size(fileList, 1))
				mfprintf(__test_output, "------------------------------------------------------------------------------\n");
			end
		end
	end
    finishProgress();
	mfprintf(__test_output, "==============================================================================\n");
	if (b)
		mfprintf(__test_output, "All tests passed!\n");
	else
		mfprintf(__test_output, "%d test(s) failed!\n", __test_failures);
    end
endfunction

//printf("Automated Test Suite\n");
//printf("==================================================================\n");

// Print out all of the global variable settings pertaining to tests
//printf("\nGlobal Variable Settings\n");
//printf("------------------------\n");
if (length(__test_file) > 0)
	//printf("__test_file:     %s\n", __test_file);
else
	//printf("__test_file:     All files starting with test\n");
end
if (size(__test_exclude, 1) > 0)
	//printf("__exclude_file:  %s\n", __test_exclude(1));
	if (size(__test_exclude, 1) > 1)
		for i=2:size(__test_exclude, 1)
			//printf("                 %s\n", __test_exclude(i));
		end
	end
else
	//printf("__exclude_file: No files excluded\n");
end
//printf("------------------------\n");

answer = 1;//x_choose(['Yes';'No'],['Should I run the tests';'with these settings?']);

if (answer(1) == 1)
	// run all tests!

	[__test_results,testFiles] = runTests();
    if (__test_coverage) then
      printf("\nCollecting coverage info...");
      cov = cover_results();
      mfprintf(__test_output, "\n");
      cover_printStmtCoverage(cov, __test_output);
      printf("\n");
      cover_makeHtmlReport(__test_dir + "report", %t, cov);
    end;

	// give final results
	if (__test_results)
		mprintf("\nAll tests passed!");//x_message("All tests passed!");
	else
//		if (__test_failures > 1)
//			smessage = sprintf("%d tests failed, check log file: %s", __test_failures, __test_filename);
//		else
//			smessage = sprintf("%d test failed, check log file: %s", __test_failures, __test_filename);
//		end
//		printf(smessage);//x_message(smesnsage);
		mprintf("\n%d test(s) failed, check log file: %s", __test_failures, __test_filename);
	end
	
	mclose(__test_output)// close file
else
	mprintf("Tests not ran\n");
end

//displaying detail unit test information
__test_total = __test_passes + __test_failures;

mprintf("\n------------------------");
mprintf("\n %d/%d unit tests passed/run",__test_passes,__test_total);
mprintf("\n %d functions loaded/run",Nb_fcts_CCA);
mprintf("\n %d libraries loaded",size(CCA_lib,1))
mprintf("\n %d files run",testFiles);
mprintf("\n------------------------\n");
mprintf(" Tests ran in %2.1f s\n",toc());

//whos -type -function

// Removed since this has already been done - both tested and untested functions
// chdir(__macros_dir);
// for filename = CCA_lib_untested
// exec(filename, -1);
// end;

chdir(__cwd_dir);//restore current directory
