mode(-1);

__test_set = "Common functions";
//WARNING: do not unit test plot_version()
//since that library is plot only and so is not
//unit tested.

function b = testIsEqualZero()
  b = %f;
  b = isEqual([-1E-8,1E-8,0,2,%inf,-%inf], [1E-8,-1E-8,-1E-8,2+1E-8,%inf,-%inf],[0,0,0,2,%inf,-%inf]);
endfunction

function b = testIsEqualEmpty()
  b = %f;
  b = isEqual([], []);
endfunction

function b = testIsEqualFloat()
  b = %f;
  b = isEqual([1/10,2; 0,-5], [(5/10-4/10),2; 0,-5]);
endfunction

function b = testIsEqualInfNegInf()
  b = %f;
  b = ~isEqual(%inf, -%inf);
endfunction

function b = testIsEqualNan()
  b = %f;
  b = ~isEqual(%nan, %nan);
endfunction

function b = testIsEqualMultiple1()
  b = %f;
  b = isEqual([1,2,3], (0:2)+1, [2,4,6]/2);
endfunction

function b = testIsEqualMultiple2()
  b = %f;
  b = ~isEqual([1,2,3], (0:2)+1, [1,2], [1,2,3]);
endfunction

function b = testSumFinites0()
  b = %f;
  b = (sumFinites([]) == 0);
endfunction

function b = testSumFinites1()
  b = %f;
  b = (sumFinites([%inf,%nan,-%inf]) == 0);
endfunction

function b = testSumFinites2()
  b = %f;
  b = (sumFinites([2,30,461;-%inf,0,-3]) == 490);
endfunction

function b = testSumFinites3()
  b = %f;
  x = [];
  x(:,:,1) = [1,2;3,4];
  x(:,:,2) = [%inf,-%inf;%nan,-2];
  b = (sumFinites(x) == 8);
endfunction

function b = testCommonVersion()
  b = %f;
  v = common_version();
  b = (length(strindex(v, ".")) >= 2);
endfunction

function b = testLftVersion()
  b = %f;
  v = lft_version();
  b = (length(strindex(v, ".")) >= 2);
endfunction

function b = testMeVersion()
  b = %f;
  v = me_version();
  b = (length(strindex(v, ".")) >= 2);
endfunction

function b = testPlqVersion()
  b = %f;
  v = plq_version();
  b = (length(strindex(v, ".")) >= 2);
endfunction

function b = testOpVersion()
  b = %f;
  v = op_version();
  b = (length(strindex(v, ".")) >= 2);
endfunction

function b = testRowMap()
  b = %f;
  function row2=doRow(row,i), row2=[row(1)*row(2)+row(3),row(4),i]; endfunction
  m = [1,2,3,4;5,6,7,8];
  desired = [5,4,1;37,8,2];
  result = rowMap(doRow,m);
  b = isEqual(desired, result);
endfunction

function b = testColMap()
  b = %f;
  function col2=doCol(col,i), col2=[size(col,1); sum(col); i]; endfunction
  m = [1,2,3,4;5,6,7,8;10,11,12,13];
  desired = [3,3,3,3;16,19,22,25;1,2,3,4];
  result = colMap(doCol,m);
  b = isEqual(desired, result);
endfunction

//Progress bar is GUI so do not unit test
//function b = testProgress0()
//  global defaultProgressWidth;
//  defaultProgressWidth = [];
//  beginProgress();
//  finishProgress();
//endfunction
//
//function b = testProgress1()
//  beginProgress();
//  showProgress(0.05);
//  abortProgress();
//endfunction
//
//function b = testProgress2()
//  beginProgress(10);
//  showProgress(0.50, 10);
//  finishProgress(10);
//endfunction
//
//function b = testProgress3()
//  beginProgress(10);
//  showProgress(0.5, 10, "Hi!");
//  finishProgress(10, "Bye!");
//endfunction

function b = testGphVersion()
  b = %f;
  v = gph_version();
  b = (length(strindex(v, ".")) >= 2);
endfunction

function b = testOptVersion()
  b = %f;
  v = opt_version();
  b = (length(strindex(v, ".")) >= 2);
endfunction

function b = testIntervalIntersect()
  b=%t;
  I1=[1,5];I2=[2,6];J=[2,5];
  b = b & isEqual(intervalIntersect(I1,I2), J);
  b = b & isEqual(intervalIntersect(I2,I1), J);    
  b = b & ~intervalisEmpty(J);
  I1=[1,5];I2=[2,4];J=[2,4];
  b = b & isEqual(intervalIntersect(I1,I2), J);
  b = b & isEqual(intervalIntersect(I2,I1), J);      
  I1=[1,5];I2=[6,8];J=[6,5];
  b = b & isEqual(intervalIntersect(I1,I2), J);
  b = b & isEqual(intervalIntersect(I2,I1), J); 
  b =b & intervalisEmpty(J);
  I1=[1,5];I2=[2,6];J=[2,5];
  b = b & isEqual(intervalIntersect(I1,I2), J);
  b = b & isEqual(intervalIntersect(I2,I1), J);
endfunction


function b = runTestFile()
  b = %t;
  b = checkForFail(testWrapper(testIsEqualZero, "testIsEqualZero"), b);  
  b = checkForFail(testWrapper(testIsEqualEmpty, "testIsEqualEmpty"), b);
  b = checkForFail(testWrapper(testIsEqualFloat, "testIsEqualFloat"), b);
  b = checkForFail(testWrapper(testIsEqualInfNegInf, "testIsEqualInfNegInf"), b);
  b = checkForFail(testWrapper(testIsEqualNan, "testIsEqualNan"), b);
  b = checkForFail(testWrapper(testIsEqualMultiple1, "testIsEqualMultiple1"), b);
  b = checkForFail(testWrapper(testIsEqualMultiple2, "testIsEqualMultiple2"), b);
  b = checkForFail(testWrapper(testSumFinites0, "testSumFinites0"), b);
  b = checkForFail(testWrapper(testSumFinites1, "testSumFinites1"), b);
  b = checkForFail(testWrapper(testSumFinites2, "testSumFinites2"), b);
  b = checkForFail(testWrapper(testSumFinites3, "testSumFinites3"), b);
  b = checkForFail(testWrapper(testCommonVersion, "testCommonVersion"), b);
  b = checkForFail(testWrapper(testLftVersion, "testLftVersion"), b);
  b = checkForFail(testWrapper(testMeVersion, "testMeVersion"), b);
  b = checkForFail(testWrapper(testPlqVersion, "testPlqVersion"), b);
  b = checkForFail(testWrapper(testOpVersion, "testOpVersion"), b);
  b = checkForFail(testWrapper(testRowMap, "testRowMap"), b);
  b = checkForFail(testWrapper(testColMap, "testColMap"), b);
//  b = checkForFail(testWrapper(testProgress0, "testProgress0"), b);
//  b = checkForFail(testWrapper(testProgress1, "testProgress1"), b);
//  b = checkForFail(testWrapper(testProgress2, "testProgress2"), b);
//  b = checkForFail(testWrapper(testProgress3, "testProgress3"), b);
  b = checkForFail(testWrapper(testGphVersion, "testGphVersion"), b);  
  b = checkForFail(testWrapper(testOptVersion, "testOptVersion"), b);    
endfunction
