mode(-1);

// Author: Bryan Gardiner
// For: Yves Lucet
// Date: Summer 2008

__test_set = "Operators: Fitzpatrick functions";

function [b] = testOpFitz1To3Linear()
  b = %f;
  a = -2:2;
  astar = 2*a - 7;
  A = [a;astar];
  x = -3:3;
  xstar = -3:3;
  for n = 1:3
    if (n <= 2) then dresult = op_fitz_brute(A, n, x, xstar); end;
    rresult = op_fitz_direct(A, n, x, xstar);
    lresult = op_fitz(A, n, x, xstar);
    b = and(abs(rresult - lresult) < 1E-5);
    if (n <= 2) then b = b & and(abs(rresult - dresult) < 1E-5); end;
    if (~b) then return; end;
  end;
endfunction

function [b] = testOpFitz12Exp()
  b = %f;
  a = -1:2;
  astar = exp(a);
  A = [a;astar];
  x = -3:3;
  xstar = -3:3;
  for n = 1:2
    dresult = op_fitz_brute(A, n, x, xstar);
    rresult = op_fitz_direct(A, n, x, xstar);
    lresult = op_fitz(A, n, x, xstar);
    b = and(abs(rresult - lresult) < 1E-5);
    b = b & and(abs(rresult - dresult) < 1E-5);
    if (~b) then return; end;
  end;
endfunction

function [b] = testOpFitz1234Log()
  b = %f;
  a = 1:3;
  astar = log(a);
  A = [a;astar];
  x = 0:4;
  xstar = 0:4;
  for n = 1:4
    if (n <= 2) then dresult = op_fitz_brute(A, n, x, xstar); end;
    rresult = op_fitz_direct(A, n, x, xstar);
    lresult = op_fitz(A, n, x, xstar);
    b = and(abs(rresult - lresult) < 1E-5);
     if (n <= 2) then b = b & and(abs(rresult - dresult) < 1E-5); end;
    if (~b) then return; end;
  end;
endfunction

function [b] = testOpFitz1234Quad()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 0:3;
  astar = a^2 - 2;
  A = [a;astar];
  A = A(:,$:-1:1);
  x = -1:4;
  xstar = 0:4;
  for n = 1:4
    if (n <= 2) then dresult = op_fitz_brute(A, n, x, xstar); end;
    rresult = op_fitz_direct(A, n, x, xstar);
    lresult = op_fitz(A, n, x, xstar);
    b = and(abs(rresult - lresult) < 1E-5);
    if (n <= 2) then b = b & and(abs(rresult - dresult) < 1E-5); end;
    if (~b) then return; end;
  end;
endfunction

function [b] = testOpFitz45Direct()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  astar = [-2:0.5:-1, 0:2];
  a = 1:length(astar);
  A = [a;astar];
  x = [0:1, 3.5, 6:7];
  xstar = -2:3;
  for n = 4:5
    rresult = op_fitz_direct(A, n, x, xstar);
    lresult = op_fitz(A, n, x, xstar);
    b = and(abs(rresult - lresult) < 1E-5);
    if (~b) then return; end;
  end;
endfunction

function [b] = testOpFitzInfQuad()
  b = %f;
  a = 1:3;
  astar = a/2 + 4;
  A = [a;astar];
  B = [a;astar;astar];
  x = 0:1;
  xstar = 0:1;
  dresult = op_fitz_brute(A, %inf, x, xstar);
  rresult = op_fitz_direct(A, %inf, x, xstar);
  lresult = op_fitz(A, %inf, x, xstar);
  iresult = op_fitzinf(B, x, xstar);
  if (~isEqual(dresult, rresult)) then return; end;
  if (~isEqual(dresult, lresult)) then return; end;
  b = %t;
endfunction

function [b] = testOpFitzInfHoriz()
  b = %f;
  astar = [1,2,2,3,3];
  a = 1:length(astar);
  A1 = [a;astar];
  A2 = [a;astar;astar];
  x = 0:5;
  xstar = 0:5;
  // desired was computed with op_fitz_brute.
  desired = [-1,0,1,4,9,14; 0,1,2,5,10,15; 1,2,4,7,12,17; 3,4,6,9,14,19; 5,6,8,12,17,22; 8,9,11,15,20,25];
  rresult = op_fitz_direct(A1, %inf, x, xstar);
  //lresult = op_fitz(A1, n, x, xstar);        LLT can't handle duplicate a values.
  iresult = op_fitzinf(A2, x, xstar);
  b = and(abs(desired - rresult) < 1E-5);
  if (~b) then return; end;
  b = and(abs(desired - iresult) < 1E-5);
endfunction

function [b] = testOpFitzDirectVert()
  b = %f;
  a = [1,2,2,3,3];
  astar = 1:length(a);
  A1 = [a;astar];
  A2 = [a;astar;astar];
  x = 0:5;
  xstar = 0:5;
  // desired was computed with op_fitz_brute.
  desired = [-1,0,1,4,9,14; 0,1,2,5,10,15; 1,2,4,7,12,17; 3,4,6,9,14,19; 5,6,8,12,17,22; 8,9,11,15,20,25]';
  rresult = op_fitz_direct(A1, %inf, x, xstar);
  //lresult = op_fitz(A1, n, x, xstar);        LLT can't handle duplicate a values.
  //iresult = op_fitzinf(A2, x, xstar);        Requires strictly increasing a values.
  b = and(abs(desired - rresult) < 1E-5);
endfunction

function [b] = testOpFitzInf0Quad()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = -1:2;
  astar = (a + 1).^2 - 2;    // [-2,-1,2,7]
  A1 = [a;astar];
  A2 = [a;astar;astar];
  x = 0:5;
  rresult = op_fitz_direct(A1, %inf, x, 0);
  lresult = op_fitz(A1, %inf, x, 0);
  iresult = op_fitzinf(A2, x, 0);
  Lplqf = plq_fitzinf0(A2); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A2);
  b = and(abs(rresult - lresult) < 1E-5);
  c = and(abs(rresult - iresult) < 1E-5);
  d = and(abs(rresult - Lresult) < 1E-5);
  e = isEqual(plq_clean(Lplqf), plq_clean(F));
  [rresult, lresult, iresult, Lresult], Lplqf, Lk, F
  b = b & c & d & e;
endfunction

function [b] = testOpFitzInf0Log()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 0.5:0.25:2;
  astar = log(a);
  A1 = [a;astar];
  A2 = [a;astar;astar];
  x = 0:5;
  lresult = op_fitz(A1, %inf, x, 0);
  iresult = op_fitzinf(A2, x, 0);
  Lplqf = plq_fitzinf0(A2); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A2);
  b = and(abs(lresult - iresult) < 1E-5);
  c = and(abs(lresult - Lresult) < 1E-5);
  d = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c & d;
endfunction

function [b] = testOpFitzInf0Pos()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:4;
  bl = [0.5, 2, 3.5, 5.5];
  br = [1, 3, 5, 7.5];
  A = [a;bl;br];
  x = 0.5:4.5;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0Pos2()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:4;
  bl = [0.5, 1, 2, 5];
  br = [1, 2, 5, 5];
  A = [a;bl;br];
  x = [0.25, 0.5, 1, 1.5, 2, 3, 5, 5.5];
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0PosTouch()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:2;
  bl = [1,2];
  br = [2,4];
  A = [a;bl;br];
  x = -2:2;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0Neg()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1;
  bl = [-3];
  br = [-2];
  A = [a;bl;br];
  x = -2:2;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk, F
endfunction

function [b] = testOpFitzInf0Neg2()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:4;
  bl = [-8, -6, -4, -2];
  br = [-7, -5, -3, -1];
  A = [a;bl;br];
  x = -8.5:-0.5;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0NegTouch()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:2;
  bl = [-3,-2];
  br = [-2,-1];
  A = [a;bl;br];
  x = 0.5:0.5:2.5;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0NegPos()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:4;
  bl = [-1, 0, 3.5, 5.5]
  br = [ 0, 3, 5  , 7.5];
  A = [a;bl;br];
  x = -0.5:4.5;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0NegPosTch()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:3;
  bl = [-1.5, -0.5, 0.5];
  br = [-0.5,  0.5, 1.5];
  A = [a;bl;br];
  x = -2:2;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0NegPosFlat()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:5;
  bl = [-1.5, -1.5, -1.5, 0.5, 0.5];
  br = [-1.5, -1.5,  0.5, 0.5, 0.5];
  A = [a;bl;br];
  x = -2:2;
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [x', iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0NegPosDup()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = 1:5;
  bl = [-0.5, -0.5, 1, 1, 2];
  br = [-0.5,  1  , 1, 1, 2];
  A = [a;bl;br];
  A = A(:,$:-1:1);
  x = [0.25, 0.5, 1, 1.5, 2, 3];
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [iresult, Lresult], Lplqf, Lk
endfunction

function [b] = testOpFitzInf0Single()
  b = %f;
  //getf("../op.sci"); getf("../plq.sci");
  a = [2];
  bl = [1];
  br = [5];
  A = [a;bl;br];
  x = [0.5, 1, 1.5, 2, 3, 5, 5.5];
  iresult = op_fitzinf(A, x, 0);
  Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
  F = plq_fitzinf0_direct(A);
  b = and(abs(iresult - Lresult) < 1E-5);
  c = isEqual(plq_clean(Lplqf), plq_clean(F));
  b = b & c;
  b, [iresult, Lresult]
endfunction

function [b] = testOpFitzNonmonotoneInf()
  //exec("loader.sce");
  b = %f;
  a = [1,2,3,4];
  astar = [3,5,4,2];
  A = [a;astar];
  B = [a;astar;astar+0.5];
  x = 0:5;
  xstar = 0:5;
  F0 = %inf * ones(length(x), length(xstar));
  F1 = op_fitz_brute(A, %inf, x, xstar);
  F2 = op_fitz_direct(A, %inf, x, xstar);
  F3 = op_fitz(A, %inf, x, xstar);
  F4 = op_fitzinf(B, x, xstar);
  if (~and(isEqual(F0, F1))) then return; end;
  if (~and(isEqual(F0, F2))) then return; end;
  if (~and(isEqual(F0, F3))) then return; end;
  if (~and(isEqual(F0, F4))) then return; end;
  b = %t;
endfunction

function [b] = testPlqFitzNonmonotone()
  b = %f;
  a = [1,2,3,4];
  astar = [3,5,4,2];
  B = [a;astar;astar+0.5];
  F0 = [%inf,0,0,%inf];
  F1 = plq_fitzinf0(B);
  F2 = plq_fitzinf0_direct(B);
  F3 = plq_rock(B, 1);
  if (~and(isEqual(F0, F1))) then return; end;
  if (~and(isEqual(F0, F2))) then return; end;
  if (~and(isEqual(F0, F3))) then return; end;
  b = %t;
endfunction

function testOpFitzInfNonIncrA()
  a  = [1,2,2,3];
  bl = [1,3,5,7];
  br = [2,4,6,8];
  A = [a;bl;br];
  x = 0;
  xstar = 0;
  op_fitzinf(A, x, xstar);
endfunction

function testPlqFitzInf0NonIncrA()
  a  = [1,2,2,3];
  bl = [1,3,5,7];
  br = [2,4,6,8];
  B = [a;bl;br];
  plq_fitzinf0(B);
endfunction

function testPlqFitzInf0DNonIncrA()
  a  = [1,2,2,3];
  bl = [1,3,5,7];
  br = [2,4,6,8];
  B = [a;bl;br];
  plq_fitzinf0_direct(B);
endfunction

function [b] = runTestFile()
  b = %t;
  b = checkForFail(testWrapper(testOpFitz1To3Linear, "testOpFitz1To3Linear"), b);
  b = checkForFail(testWrapper(testOpFitz12Exp, "testOpFitz12Exp"), b);
  b = checkForFail(testWrapper(testOpFitz1234Log, "testOpFitz1234Log"), b);
  b = checkForFail(testWrapper(testOpFitz1234Quad, "testOpFitz1234Quad"), b);
  b = checkForFail(testWrapper(testOpFitz45Direct, "testOpFitz45Direct"), b);
  b = checkForFail(testWrapper(testOpFitzInfQuad, "testOpFitzInfQuad"), b);
  b = checkForFail(testWrapper(testOpFitzInfHoriz, "testOpFitzInfHoriz"), b);
  b = checkForFail(testWrapper(testOpFitzDirectVert, "testOpFitzDirectVert"), b);
  b = checkForFail(testWrapper(testOpFitzInf0Quad, "testOpFitzInf0Quad"), b);
  b = checkForFail(testWrapper(testOpFitzInf0Log, "testOpFitzInf0Log"), b);
  b = checkForFail(testWrapper(testOpFitzInf0Pos, "testOpFitzInf0Pos"), b);
  b = checkForFail(testWrapper(testOpFitzInf0Pos2, "testOpFitzInf0Pos2"), b);
  b = checkForFail(testWrapper(testOpFitzInf0PosTouch, "testOpFitzInf0PosTouch"), b);
  b = checkForFail(testWrapper(testOpFitzInf0Neg, "testOpFitzInf0Neg"), b);
  b = checkForFail(testWrapper(testOpFitzInf0Neg2, "testOpFitzInf0Neg2"), b);
  b = checkForFail(testWrapper(testOpFitzInf0NegTouch, "testOpFitzInf0NegTouch"), b);
  b = checkForFail(testWrapper(testOpFitzInf0NegPos, "testOpFitzInf0NegPos"), b);
  b = checkForFail(testWrapper(testOpFitzInf0NegPosTch, "testOpFitzInf0NegPosTch"), b);
  b = checkForFail(testWrapper(testOpFitzInf0NegPosFlat, "testOpFitzInf0NegPosFlat"), b);
  b = checkForFail(testWrapper(testOpFitzInf0NegPosDup, "testOpFitzInf0NegPosDup"), b);
  b = checkForFail(testWrapper(testOpFitzInf0Single, "testOpFitzInf0Single"), b);
  b = checkForFail(testWrapper(testOpFitzNonmonotoneInf, "testOpFitzNonmonotoneInf"), b);
  b = checkForFail(testWrapper(testPlqFitzNonmonotone, "testPlqFitzNonmonotone"), b);
  b = checkForFail(errorTestWrapper(testOpFitzInfNonIncrA, "testOpFitzInfNonIncrA"), b);
  b = checkForFail(errorTestWrapper(testPlqFitzInf0NonIncrA, "testPlqFitzInf0NonIncrA"), b);
  b = checkForFail(errorTestWrapper(testPlqFitzInf0DNonIncrA, "testPlqFitzInf0DNonIncrA"), b);
endfunction
