mode(-1);

__test_set = "GPH: gph_eval - gph evaluation";

function [b] = testIndicator()
	eps = 1e-10;
	gph = [-1 -1; -1 1; -3 -3];//I_{-1)-3
	X = [-3; -2; -1; 0; 1; 2];
	Z = gph_eval(gph, X);
	expected = [%inf; %inf; -3; %inf; %inf; %inf];
	b = and(expected == Z | abs(expected - Z) < eps);
endfunction

function [b] = testSingleRegion()
	eps = 1e-10;
	b = %f;
	// f(x) = -3*X^2 + 5   (subdifferential: -6*X)
	gph = [-3 -1; 18 6; -22 2];
	X = (-4:0.5:1)';
	Z = gph_eval(gph, X);

	expected = -3*X.^2 + 5;
	b = and(abs(expected - Z) < eps);
endfunction

function [b] = testAbs()
	eps = 1e-10;
	b = %f;
	//abs function
	gph = [-1, 0, 0, 1; ..
	       -1, -1, 1, 1; ..
	       1, 0, 0, 1];
	X = [-100; -5; -1; -0.5; 0; 0.1; 0.8; 1; 1.01; 12];
	Z = gph_eval(gph, X);

	expected = abs(X);
	b = and(abs(expected - Z) < eps);
endfunction

function [b] = testL_bounded()
	gph = [-1, -1, 1, 1; ..
	       -1, 0, 0, 1; ..
	       %inf, 0, 0, %inf];//I_[-1,1].
	X = [-5; -1.01; -1; -1;-1;-0.8; -0.2; 0; 0.5; 0.999; 1; 1.001; 5; 100];
	Z = gph_eval(gph, X);
	expected = [%inf; %inf; 0; 0; 0; 0; 0; 0; 0; 0; 0; %inf; %inf; %inf];
	b = and(expected == Z);
endfunction

function [b] = testPL()
	eps = 1e-10;
	// A "\_/" function.
	gph = [-1, 0, 0, 1, 1, 2; ..
	       -1, -1, 0, 0, 1, 1; ..
	       1, 0, 0, 0, 0, 1];
	X = [-6; -1; -0.5; 0; 0.5; 1; 1.5; 2; 4; 20];
	Z = gph_eval(gph, X);

	expected = [6; 1; 0.5; 0; 0; 0; 0.5; 1; 3; 19];
	b = and(abs(expected - Z) < eps);
endfunction

function [b] = testPLQ1()
	eps = 1e-10;
	b = %f;
	// A PLQ function:
	//     x<-1: y=(x+1)^2
	//   -1<x<1: y=0
	//      1<x: y=(x-1)^2
	gph = [-2, -1, 1, 2; ..
	       -2,  0, 0, 2; ..
	       1, 0, 0, 1];
	X = [-6; -4; -2.5; -1.75; -1; -0.5; 0; 0.75; 1; 1.5; 2; 2.5; 3; 5; 11];
	Z = gph_eval(gph, X);

	expected = [25; 9; 2.25; 0.5625; 0; 0; 0; 0; 0; 0.25; 1; 2.25; 4; 16; 100];
	b = and(abs(expected - Z) < eps);
endfunction

function [b] = testPLQ2()
	eps = 1e-10;
	b = %f;
	// A PLQ function:
	//   x in [-1,1]: 0
	//     otherwise: x^2 - 1
	gph = [-2, -1, -1, 1, 1, 2; ..
	       -4, -2, 0, 0, 2, 4; ..
	       3, 0, 0, 0, 0, 3];
	X = [-9; -5; -2; -1.01; -1; -0.25; 0.1; 0.99; 1; 1.5; 2; 2.5; 3; 7];
	Z = gph_eval(gph, X);

	expected = [80; 24; 3; 0.0201; 0; 0; 0; 0; 0; 1.25; 3; 5.25; 8; 48];
	b = and(abs(expected - Z) < eps);
endfunction

function [b] = testEmpty()
	gph = [-1, 0, 0, 1;-1, -1, 1, 1; 1, 0, 0, 1];//abs
	X = [];
	Z = gph_eval(gph, X);
	expected = [];
	b = and(expected == Z);
endfunction

function b=test1()
  //evaluate on G(1,:) only
  G = [-1, 0, 0, 1;-1, -1, 1, 1; 1, 0, 0, 1];//abs
  x=[-1;0;0;1];
  y=gph_eval(G,x);
  b = isEqual(y,[1;0;0;1]);
endfunction

 
function b = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testIndicator,"testIndicator"), b);//error
	b = checkForFail(testWrapper(testSingleRegion,"testSingleRegion"), b);
	b = checkForFail(testWrapper(testAbs,"testAbs"), b);
	b = checkForFail(testWrapper(testL_bounded,"testL_bounded"), b);//fail
	b = checkForFail(testWrapper(testPL,"testPL"), b);
	b = checkForFail(testWrapper(testPLQ1,"testPLQ1"), b);
	b = checkForFail(testWrapper(testPLQ2,"testPLQ2"), b);
	b = checkForFail(testWrapper(testEmpty,"testEmpty"), b);
	b = checkForFail(testWrapper(test1,"test1"), b);	//fail
endfunction
