// Unit test file for gph_lft

mode(-1);

__test_set = "GPH: gph_lft - Fenchel Conjugate";

function b = testIsBounded()
  g = [-1, 0, 0, 1;-1, -1, 1, 1;1, 0, 0, 1];//abs
  b = and(gph_isBounded(g)==[%f %f]);
  g = [-1 -1 1 1;-1 0 0 1;%inf 0 0 %inf];//I_[-1,1]
  b = b & and(gph_isBounded(g)==[%t %t]);
  g = [-1 -1 0 1;-1 0 0 1;%inf 0 0 0.5];//(x<0?-x:x^2)^*
  b = b & and(gph_isBounded(g)==[%t %f]);
  g = [-1 0 1 1;-1 0 0 1;0.5 0 0 %inf];//(x<0?x^2:-x)^*
  b = b & and(gph_isBounded(g)==[%f %t]);  
endfunction

function [b] = testAbs()
	b = %f;
	//abs function
	G = [-1, 0, 0, 1; ..
	     -1, -1, 1, 1; ..
	     1, 0, 0, 1];
	if ~_gph_check(G) then return; end;
	Gs = gph_lft(G);
    p = plq_gph(G);
    q=plq_lft(p);// I_[0,1]
    expected=gph_plq(q);	
	b = and(Gs == expected);
endfunction

function [b] = testInd()
	b = %f;
	//indicator function I_[-1,1].
	G = [-1, -1, 1, 1; ..
	     -1, 0, 0, 1; ..
	     %inf, 0, 0, %inf];
	if ~_gph_check(G) then return; end;
	Gs = gph_lft(G);

	// Abs function
	expected = [-1, 0, 0, 1; ..
		    -1, -1, 1, 1; ..
		    1, 0, 0, 1];
	b = and(Gs == expected);
endfunction

function [b] = testPL()
	b = %f;
	// A piecewise linear function:
	//     x<0: y=-x
	//   0<x<1: y=0
	//     1<x: y=x-1
	G = [-1, 0, 0, 1, 1, 2; ..
	     -1, -1, 0, 0, 1, 1; ..
	     1, 0, 0, 0, 0, 1];
	if ~_gph_check(G) then return; end;
	Gs = gph_lft(G);

	expected = [-1, -1, 0, 0, 1, 1; ..
		    -1, 0, 0, 1, 1, 2; ..
		    %inf, 0, 0, 0, 1, %inf];
	b = and(Gs == expected);
endfunction

function [b] = testPLQ1()
	b = %f;
	// A PLQ function:
	//     x<-1: y=(x+1)^2
	//   -1<x<1: y=0
	//      1<x: y=(x-1)^2
	G = [-2, -1, 1, 2; ..
	     -2,  0, 0, 2; ..
	     1, 0, 0, 1];
	if ~_gph_check(G) then return; end;
	// The conjugate of the PLQ function is:
	//      x<0: y = 0.25*x^2 - x
	//      x>0: y = 0.25*x^2 + x	 
	Gs = gph_lft(G);
    p = plq_gph(G);q = plq_lft(p);Ge = gph_plq(q);
	b = gph_isEqual(Gs,Ge);
endfunction

function [b] = testPLQ2()
	b = %f;
	// A PLQ function:
	//   x in [-1,1]: 0
	//     otherwise: x^2 - 1
	G = [-2, -1, -1, 1, 1, 2; ..
	     -4, -2, 0, 0, 2, 4; ..
	     3, 0, 0, 0, 0, 3];
	if ~_gph_check(G) then return; end;
    // The conjugate of the PLQ function is:
    //     x<-2: y = 0.25*x^2 + 1
    //   -2<x<0: y = -x
    //    0<x<2: y = x
    //      2<x: y = 0.25*x^2 + 1
	Gs = gph_lft(G);
    p = plq_gph(G);q=plq_lft(p);Ge = gph_plq(q);
	b = gph_isEqual(Gs,Ge);
endfunction

function [b] = testQuadratic()
  p=[%inf,1,0,0];P=gph_plq(p);b=_gph_check(P);
  Q=gph_lft(P);q=plq_lft(p);QQ=gph_plq(q);
  b = b & gph_isEqual(Q,QQ);
endfunction

function [b] = testLinear()
  p=[%inf,0,1,0];P=gph_plq(p);b=_gph_check(P);
  Q=gph_lft(P);q=plq_lft(p);QQ=gph_plq(q);
  b = b & gph_isEqual(Q,QQ);
endfunction

function [b] = testIndicatorSingleton()
  p=[1,0,0,2];P=gph_plq(p);b=_gph_check(P);
  Q=gph_lft(P);q=plq_lft(p);QQ=gph_plq(q);
  b = b & gph_isEqual(Q,QQ);
endfunction

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testIsBounded,"testIsBounded"), b);	
	b = checkForFail(testWrapper(testAbs,"testAbs"), b);
	b = checkForFail(testWrapper(testInd,"testInd"), b);
	b = checkForFail(testWrapper(testPL,"testPL"), b);
	b = checkForFail(testWrapper(testPLQ1,"testPLQ1"), b);
	b = checkForFail(testWrapper(testPLQ2,"testPLQ2"), b);
	b = checkForFail(testWrapper(testQuadratic,"testQuadratic"), b);
	b = checkForFail(testWrapper(testLinear,"testLinear"), b);
	b = checkForFail(testWrapper(testIndicatorSingleton,"testIndicatorSingleton"), b);	
endfunction
