mode(-1);
// Unit test file for gph_me

__test_set = "GPH: gph_me";

//ME-Line Y=X+1
function [b] = testMELineWrapper()
	b = %f;
	lambda = 1;
	gph = [0 1; 1 1; 1 2];    // f(x) = x + 1
	gphme = gph_me(gph, lambda)
	X = (-5:0.5:5)';
	result = gph_eval(gphme, X)
	desired = X + 1/2;    // e_f(x) = x + 1/2
	b = and(result == desired);
endfunction

function [b] = testMEQuadraticWrapper()
	b = %f;
	lambda=1;
	gph = [-1 1; -1 1; 0.5 0.5];    // f(x) = 1/2*x^2
	gphme = gph_me(gph, lambda)
	X = (-3:0.5:3)';
	result = gph_eval(gphme, X)
	desired = 1/4 * X.^2;
	b = and(result == desired);
endfunction

function [b] = testMEIndicatorWrapper()
	b = %f;
	lambda = 1;
	gph = [5 5; -1 1; 1 1];    // Ind. f(5) = 1
	gphme = gph_me(gph, lambda)
	X = (-3:0.5:3)';
	result = gph_eval(gphme, X)
	desired = 1/2*X.^2 - 5.*X + 13.5;
	b = and(result == desired);
endfunction

function [b] = testUnbSmall0()
	eps = 1e-10
	b = %f;
	lambda = 1;
	plq = [0,0,-1,0; %inf,0,1,0];//abs
	expected = plq_me(plq, lambda);
	gph = gph_plq(plq);
	gphme = gph_me(gph, lambda);
	actual = plq_gph(gphme);
	b = and(actual == expected | abs(actual - expected) < eps);

	//scf();
	//plq_plot(expected, actual);
	//gph_plot(gphme, gph_plq(expected));
endfunction

function [b] = testUnbSmall1()
	eps = 1e-10;
	b = %f;
	lambda = 1;
	plq = [0,0,-1,0; 1,0,0,0; %inf,1,-2,1];
	expected = plq_me(plq, lambda);
	gph = gph_plq(plq);
	gphme = gph_me(gph, lambda);
	actual = plq_gph(gphme);
	b = and(actual == expected | abs(actual - expected) < eps);

	//scf();
	//plq_plot(expected, actual);
	//gph_plot(gphme, gph_plq(expected));
endfunction

function [b] = testUnbBig()
	eps = 1e-10;
	b = %f;
	lambda = 1;
	plq = [-2,0,-2,-2; -1,0,-1.5,-1; 0,0,-0.5,0; 1,0,0,0; 2,0,1,-1; %inf,1,-2,1];
	expected = plq_me(plq, lambda);
	gph = gph_plq(plq);
	gphme = gph_me(gph, lambda);
	actual = plq_gph(gphme);
	b = and(actual == expected | abs(actual - expected) < eps);

	//scf();
	//plq_plot(expected, actual);
	//gph_plot(gphme, gph_plq(expected));
endfunction

function [b] = testLowerBounded()
	eps = 1e-10;
	b = %f;
	lambda = 2;
	// (x-5)^2+1 = x^2 - 10x + 25 + 1
	plq = [2,0,0,%inf; 5,0,-1,6; %inf,1,-10,26;]
	expected = plq_me(plq, lambda);
	gph = gph_plq(plq);
	gphme = gph_me(gph, lambda);
	actual = plq_gph(gphme);
	b = and(actual == expected | abs(actual - expected) < eps);
endfunction

function [b] = testUpperBounded()
	eps = 1e-10;
	b = %f;
	lambda = 3;
	// (x-1)^2+1 = x^2 - 2x + 1 + 1
	plq = [1,1,-2,2; 5,0,1,0; %inf,0,0,%inf];
	expected = plq_me(plq, lambda);
	gph = gph_plq(plq);
	gphme = gph_me(gph, lambda);
	actual = plq_gph(gphme);
	b = and(actual == expected | abs(actual - expected) < eps);
endfunction

function [b] = testBounded()
	eps = 1e-10;
	b = %f;
	lambda = 1.5;
	plq = [-3,0,0,%inf; 0,1,0,3; 2,0,0,3; 5,0,1,1; %inf,0,0,%inf];
	expected = plq_me(plq, lambda);
	gph = gph_plq(plq);
	gphme = gph_me(gph, lambda);
	actual = plq_gph(gphme);
	b = and(actual == expected | abs(actual - expected) < eps);
endfunction

//function [b] = testMEnonconvex()
//	b = %f;
//	lambda = 1;
//	plqf=[-1,0,-1,-1;0,0,1,1;1,0,-1,1;%inf,0,1,-1];
//	result = plq_me(plqf,lambda,%f);
//	desired = [-2,0,-1,-1.5;0,0.5,1,0.5;2,0.5,-1,0.5;%inf,0,1,-1.5];
//	b = (result==desired);
//endfunction

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testMELineWrapper,"testMELineWrapper"), b);
	b = checkForFail(testWrapper(testMEIndicatorWrapper,"testMEIndicatorWrapper"), b);
	b = checkForFail(testWrapper(testMEQuadraticWrapper,"testMEQuadraticWrapper"), b);
	b = checkForFail(testWrapper(testUnbSmall0,"testUnbSmall0"), b);
	b = checkForFail(testWrapper(testUnbSmall1,"testUnbSmall1"), b);
	b = checkForFail(testWrapper(testUnbBig,"testUnbBig"), b);
	b = checkForFail(testWrapper(testLowerBounded,"testLowerBounded"), b);
	b = checkForFail(testWrapper(testUpperBounded,"testUpperBounded"), b);
	b = checkForFail(testWrapper(testBounded,"testBounded"), b);
	//b = checkForFail(testWrapper(testMEnonconvex,"testMEnonconvex"), b);
endfunction
