// Unit test file for gph_plq
mode(-1);

__test_set = "GPH: gph_plq - plq to gph convertion";

function b = testAbs()
	//abs function
  p = [0,0,-1,0;%inf,0,1,0];
  G = gph_plq(p);
  E = [-1, 0, 0, 1; -1, -1, 1, 1; 1, 0, 0, 1];
  b = and(G==E)
endfunction

function b = testQuartic()
  function y=f(x), y=x.^4;endfunction
  function dy=df(x), dy=4*x.^3;endfunction
  x=linspace(-2,2,4)';
  
  p=plq_build(x,f,df,%t,%f,'soqs','bounded');
  P=gph_plq(p);
  pg=plq_gph(P);
  b=plq_isEqual(p,pg);
endfunction

 

function [b] = testInd()
	b = %f;
	// The function I_[-1,1].
        plq = [-1,0,0,%inf; 1,0,0,0; %inf,0,0,%inf];
        gph = gph_plq(plq);

        expected = [-1, -1, 1, 1; ..
	            -1, 0, 0, 1; ..
                    %inf, 0, 0, %inf];
        b = and(gph == expected);
endfunction

function [b] = testPL()
	b = %f;
	// A "\_/" function.
        plq = [0,0,-1,0; 1,0,0,0; %inf,0,1,-1];
        gph = gph_plq(plq);

	expected = [-1, 0, 0, 1, 1, 2; ..
                    -1, -1, 0, 0, 1, 1; ..
                    1, 0, 0, 0, 0, 1];
        b = and(gph == expected);
endfunction

function [b] = testPLQ1()
	b = %f;
	// A PLQ function:
	//     x<-1: y=(x+1)^2
	//   -1<x<1: y=0
	//      1<x: y=(x-1)^2
        plq = [-1,1,2,1; 1,0,0,0; %inf,1,-2,1];
        gph = gph_plq(plq);

	expected = [-2, -1, 1, 2; ..
                    -2,  0, 0, 2; ..
                    1, 0, 0, 1];
        b = and(gph == expected);
endfunction

function b = testPLQ2()
	b = %f;
	// A PLQ function:
	//   x in [-1,1]: 0
	//     otherwise: x^2 - 1
        plq = [-1,1,0,-1; 1,0,0,0; %inf,1,0,-1];
        gph = gph_plq(plq);

	expected = [-2, -1, -1, 1, 1, 2; ..
                    -4, -2, 0, 0, 2, 4; ..
                    3, 0, 0, 0, 0, 3];
        b = and(gph == expected);
endfunction

function b = testFullDomain()
  p = [%inf,1,0,0];
  gph = gph_plq(p);
  ge = [-1 1;-2 2;1 1];
  b = gph_isEqual(gph,ge);
endfunction

function b = testIndicator()
  p = [2,0,0,1];
  gph = gph_plq(p);
  ge = [2 2;-1 1;1 1];
  b = gph_isEqual(gph,ge);
endfunction


function [b] = runTestFile()
          b = %t;
          b = checkForFail(testWrapper(testAbs,"testAbs"), b);
          b = checkForFail(testWrapper(testQuartic,"testQuartic"), b);          
          b = checkForFail(testWrapper(testInd,"testInd"), b);
          b = checkForFail(testWrapper(testPL,"testPL"), b);
          b = checkForFail(testWrapper(testPLQ1,"testPLQ1"), b);
          b = checkForFail(testWrapper(testPLQ2,"testPLQ2"), b);
          b = checkForFail(testWrapper(testFullDomain,"testFullDomain"), b);
          b = checkForFail(testWrapper(testIndicator,"testIndicator"), b);          
endfunction
