mode(-1);
__test_set = "GPH: gph_prox, gph_me";

//perform all tests
//P: input gph function
//lambda: input real number lambda >0
function b = _test(p,lambda)
  P=gph_plq(p);
  
  M=gph_me(P,lambda);//m=plq_me(p,lambda);
  R=gph_prox(P,lambda);//r=plq_prox(p,lambda);//monotone but nonconvex
  //since there is no plq_compose function
  //we numerically evaluate the moreau envelope
  //computed directly and through the proximal mapping
  dx=linspace(-10,10)';
//  printf("1\n");
  dy=gph_eval(R,dx);//dy=plq_eval(r,dx);
//  printf("2\n");
  dp=gph_eval(P,dy);//dp=plq_eval(p,dy);
  dz=dp+(dx-dy).^2/(2*lambda);
//    printf("3\n");
  dm=gph_eval(M,dx);//dm=plq_eval(m,dx);
  b=and(clean(dm-dz)+1);
endfunction

//while we could avoid using plq functions, we prefer to use exactly
//the same tests as plg_prox without modifying unit test functions
//these functions are duplicated below.

function b = testLambdaNegative()
  p=[0 0 -1 0; %inf 0 1 0];lambda =-2;
  P=gph_plq(p);
  b=%f;
//  try
    R=gph_prox(P,lambda);
//  catch
//    b=%t;
//  end
endfunction

function b = testLambdaNegative2()
  p=[0 0 -1 0; %inf 0 1 0];lambda =-2;
  b=%f;
  b=_test(p,lambda);
endfunction


function b = testAbs()
  p=[0 0 -1 0; %inf 0 1 0];lambda =2;
  b=_test(p,lambda);
endfunction

function b = testPLQ0()
	p=[%inf 1 0 0];lambda=3;
    b=_test(p,lambda);
endfunction

function b = testPLQ1()
  // A PLQ function:
  //     x<-1: y=(x+1)^2
  //   -1<x<1: y=0
  //      1<x: y=(x-1)^2
  p=[-1 1 2 1;1 0 0 0;%inf 1 -2 1];
  lambda=3;
  b=_test(p,lambda);
endfunction

function b = testPL()
	// A piecewise linear function:
	//     x<0: y=-x
	//   0<x<1: y=0
	//     1<x: y=x-1
	p = [0 0 -1 0;1 0 0 0;%inf 0 1 -1];
  lambda=20;
  b=_test(p,lambda);
endfunction

function b = testPLQ2()
	// A PLQ function:
	//   x in [-1,1]: 0
	//     otherwise: x^2 - 1
  p=[-1 1 0 -1;1 0 0 0;%inf 1 0 -1];
  lambda=3/4;
  b=_test(p,lambda);

endfunction

function b = testQuadratic()
  p=[%inf,1,0,0];
  lambda=1/10;
  b=_test(p,lambda);
endfunction

function b = testLinear()
  p=[%inf,0,1,0];
  lambda=1/3;
  b=_test(p,lambda);
endfunction

function b = testIndicatorSingleton()
  p=[1,0,0,2];
  lambda=30.5;
  b=_test(p,lambda);
endfunction

function b = testInd()
  p=[-1 0 0 %inf;1 0 0 0 ;%inf 0 0 %inf];//indicator function I_[-1,1].
  lambda=10;
  b=_test(p,lambda);
endfunction

function b = test1()
  p=[0 0 -1 0; %inf 0 1 0];lambda =2;
  P=gph_plq(p);  
  R=gph_prox(P,lambda);//R is monotone nonconvex
  b=~_gph_check(R,%t);
endfunction

function b = runTestFile()
	b = %t;
	b = checkForFail(errorTestWrapper(testLambdaNegative,"testLambdaNegative"), b);		
	b = checkForFail(errorTestWrapper(testLambdaNegative2,"testLambdaNegative2"), b);			
	b = checkForFail(testWrapper(testPLQ0,"testPLQ0"), b);	
	b = checkForFail(testWrapper(testAbs,"testAbs"), b);
	b = checkForFail(testWrapper(testPL,"testPL"), b);
	b = checkForFail(testWrapper(testPLQ1,"testPLQ1"), b);
	b = checkForFail(testWrapper(testPLQ2,"testPLQ2"), b);
	b = checkForFail(testWrapper(testQuadratic,"testQuadratic"), b);
	b = checkForFail(testWrapper(testLinear,"testLinear"), b);
	b = checkForFail(testWrapper(testIndicatorSingleton,"testIndicatorSingleton"), b);	
	b = checkForFail(testWrapper(testInd,"testInd"), b);		
	b = checkForFail(testWrapper(test1,"test1"), b);	
endfunction
