mode(-1);
__test_set = "PLQ: pl_gph";

//perform all tests
//p: input plq function
//isError: %t if error is expected
//dp: output plq function when isError=%f
function b = _test(p,isError,dp)
  isErrorComputed=%f;
  G = gph_plq(p);
  //error generated if the subdifferential of PLQ is not a function - handled by cerror
  dpComputed=pl_gph(G);
  if ~isError then b=plq_isEqual(dp,dpComputed);end;
endfunction

function b = testAbs()
  p=[0 0 -1 0; %inf 0 1 0];
  dp=[];
  b=_test(p,%t,dp);
endfunction

function b = testPLQ0()
	p=[%inf 1 0 0];dp=[%inf 0 2 0];
    b=_test(p,%f,dp);
endfunction

function b = testPLQ1()
  // A PLQ function:
  //     x<-1: y=(x+1)^2
  //   -1<x<1: y=0
  //      1<x: y=(x-1)^2
  p=[-1 1 2 1;1 0 0 0;%inf 1 -2 1];
  dp=[-1 0 2 2;1 0 0 0 ;%inf 0 2 -2];
  b=_test(p,%f,dp);
endfunction


function b = testPL()
	// A piecewise linear function:
	//     x<0: y=-x
	//   0<x<1: y=0
	//     1<x: y=x-1
	p = [0 0 -1 0;1 0 0 0;%inf 0 1 -1];
    b=_test(p,%t,[]);
endfunction

function b = testPLQ2()
	// A PLQ function:
	//   x in [-1,1]: 0
	//     otherwise: x^2 - 1
  p=[-1 1 0 -1;1 0 0 0;%inf 1 0 -1];
  b=_test(p,%t,[]);
endfunction

function b = testQuadratic()
  p=[%inf,1,0,0];
  dp=[%inf 0 2 0];
  b=_test(p,%f,dp);
endfunction

function b = testLinear()
  p=[%inf,0,1,0];
  dp=[%inf 0 0 1];
  b=_test(p,%f,dp);	
endfunction

function b = testIndicatorSingleton()
  p=[1,0,0,2];
  b=_test(p,%t,[]);
endfunction

function b = testInd()
  p=[-1 0 0 %inf;1 0 0 0 ;%inf 0 0 %inf];//indicator function I_[-1,1].
  b=_test(p,%t,[]);
endfunction

function b = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testPLQ0,"testPLQ0"), b);	
	b = checkForFail(errorTestWrapper(testAbs,"testAbs"), b);
	b = checkForFail(errorTestWrapper(testInd,"testInd"), b);
	b = checkForFail(errorTestWrapper(testPL,"testPL"), b);
	b = checkForFail(testWrapper(testPLQ1,"testPLQ1"), b);
	b = checkForFail(errorTestWrapper(testPLQ2,"testPLQ2"), b);
	b = checkForFail(testWrapper(testQuadratic,"testQuadratic"), b);
	b = checkForFail(testWrapper(testLinear,"testLinear"), b);
	b = checkForFail(errorTestWrapper(testIndicatorSingleton,"testIndicatorSingleton"), b);	
endfunction
