// Author: 	    Mike Trienis
// Whatsit:     A wrapper function for various legendre pl_lft_generic tranforms functions
//
// S=Seperable Function, N=Non-Seperable Function

mode(-1);

__test_set = "LFT: Fenchel - General lft Function";

function y = f(x), y = x .^ 2 / 2; endfunction
function s = df(x), s = x; endfunction

function [b] = testlft_fenchel_1d()
	X = [1:1:10]'
	Y = 0.5.*(X.^2)
	[Conjw] = pl_lft_generic(X, Y, X);
	[Conj] = pl_lft_llt(X, Y, X);
	[Conjp] = pl_lft_plt(X, f, df);
	// Y in should be Y out for this particular set
	b = isequal(Conj, Conjw, Conjp);
endfunction

function [b] = testlltdirect_fenchel_2d()
    //xr and xc is the grid which f is evaluated on
    Xr=(-2:1:2)'; //x
    Xc=(-2:1:2)'; //y

    //sr and sc is the grid which f* is evaluated on
    Sr=(-2:1:2)';
    Sc=(-2:1:2)';
   
    f = rand(size(Xr,1),size(Xc,1));

    result_direct = pl_lft_direct_2d (Xr, Xc, f, Sr, Sc);
    result_llt = pl_lft_llt_2d(Xr,Xc,f,Sr,Sc);
    result_fenchel = pl_lft_generic(Xr,Xc,f,Sr,Sc);
    
    b = and(result_direct==result_llt & result_fenchel==result_llt);
endfunction

function [b] = testlft_direct2dS()
    //xr and xc is the grid which f is evaluated on
    Xr=(-2:0.1:2)'; //x
    Xc=(-2:0.1:2)'; //y

    //sr and sc is the grid which f* is evaluated on
    Sr=(-2:0.1:2)';
    Sc=(-2:0.1:2)';

    deff('[z]=f(Xr,Xc)',['z= Xr^2 + Xc^2']);
    z=eval3d(f,Xr,Xc);

    //f must be a matrix size(Xr,Xc')
    result = pl_lft_direct_2d (Xr, Xc, z, Sr, Sc);
    
    clear f
    //Desired
    deff('[z]=f(Sr,Sc)',['z= (1/4)*Sr^2 + (1/4)*Sc^2']);
    desired = eval3d(f,Xr,Xc);
    
    b = and((result-desired)<1.000D-3);
endfunction


function [b] = testlft_direct2dN()
    //xr and xc is the grid which f is evaluated on
    Xr=(-2:0.1:2)'; //x
    Xc=(-2:0.1:2)'; //y

    //sr and sc is the grid which f* is evaluated on
    Sr=(-2:0.1:2)';
    Sc=(-2:0.1:2)';

    deff('[z]=f(Xr,Xc)',['z= -Xr.*Xc + Xr^2 + Xc^2']);
    z=eval3d(f,Xr,Xc);
    
    //f must be a matrix size(Xr,Xc')
    result = pl_lft_direct_2d (Xr, Xc, z, Sr, Sc);

    //surface z=x*y
    clear f
    deff('[z]=f(Sr,Sc)',['z= (Sr.*Sc)/3 + (Sr^2)/3 + (Sc^2)/3']);
    desired = eval3d(f,Sr,Sc);
    
    b = and((result-desired)<1.000D-3);
endfunction


function [b] = testlft_llt3dS()
    //xr and xc is the grid which f is evaluated on
    Xr=(-2:1:2)'; //x
    Xc=(-2:1:2)'; //y
    Xt=(-2:1:2)'; //y

    //sr and sc is the grid which f* is evaluated on
    Sr=(-2:1:2)';
    Sc=(-2:1:2)';
    St=(-2:1:2)';
    
    f = rand(size(Xr,1),size(Xc,1),size(Xt,1));
    result = pl_lft_llt_3d (Xr, Xc, Xt, f, Sr, Sc, St);
    
    //Note the Conjugate of f(x,y,z)=x^2+y^2+z^2 ---> f*(s,r,t) = 1/4*s^2 + 1/4*y^2 + 1/4*t^2 //This function is convex! since f**=f
endfunction

function testCcaFenchelTooFewArgs()
  X = [1,3,5,7];
  Y = [3,7,11,13,17];
  pl_lft_generic(X, Y);
endfunction

function testLftDirect2dWrongSize()
  Xr = 1:4; Xc = 1:5; f = eye(5, 5);
  Sr = 1:3; Sc = 2:5;
  pl_lft_llt_2d(Xr, Xc, f, Sr, Sc);
endfunction

function testLft2dWrongSize()
  Xr = 1:4; Xc = 1:5; f = eye(5, 5);
  Sr = 1:3; Sc = 2:5;
  pl_lft_llt_2d(Xr, Xc, f, Sr, Sc);
endfunction

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testlft_fenchel_1d, "testlft_fenchel_1d"), b);	
	b = checkForFail(testWrapper(testlltdirect_fenchel_2d, "testlltdirect_fenchel_2d"), b);
	b = checkForFail(testWrapper(testlft_direct2dS, "testlft_direct2dS"), b);	
	b = checkForFail(testWrapper(testlft_direct2dN, "testlft_direct2dN"), b);
	b = checkForFail(testWrapper(testlft_llt3dS, "testlft_llt3dS"), b);
	b = checkForFail(errorTestWrapper(testCcaFenchelTooFewArgs, "testCcaFenchelTooFewArgs"), b);
	b = checkForFail(errorTestWrapper(testLftDirect2dWrongSize, "testLftDirect2dWrongSize"), b);
	b = checkForFail(errorTestWrapper(testLft2dWrongSize, "testLft2dWrongSize"), b);
endfunction
