mode(-1);
__test_set = "LLT: Linear Time Legendre Transform";

function y = f(x), y = x .^ 2 / 2; endfunction
function s = df(x), s = x; endfunction

function b = testPartialConj_direct()
//compute the conjugate of a function going through (i,row(i))
//with pl_lft_direct_d
	row = [1 2.5 5 8.5 13]';//values of the function on {1,2,3,4,5}
	conj_part = pl_lft_direct_d(row);//conjugate computation
	b = isequal(conj_part, [0 1.5 4 7.5 12]');//verify the result
endfunction

function b = testQuadratic_direct_plt()
//conjugate of the energy function f(x)=x^2/2
//with pl_lft_direct
	X = [1:1:10]';//primal grid
	Y = 0.5.*(X.^2);//sample the function on X
	//compute the conjugate with dual grid = primal grid
	[Conj] = pl_lft_direct(X, Y, X);
	[Conjp] = pl_lft_plt(X, f, df);
	// Check that the conjugate of the energy function is itself
	b = isequal(Y, Conj, Conjp);
endfunction

function b = testPartialConj_llt()
//compute the conjugate of a function going through (i,row(i))
//with pl_lft_llt
	row = [1 2.5 5 8.5 13]';//values of the function on {1,2,3,4,5}
	conj_part = pl_lft_llt_d(row);//conjugate computation
	b = isequal(conj_part, [0 1.5 4 7.5 12]');//verify the result
endfunction

function b = testQuadratic_llt()
//conjugate of the energy function f(x)=x^2/2
//with pl_lft_llt
	X = [1:1:10]';//primal grid
	Y = 0.5.*(X.^2);//sample the function on X
	//compute the conjugate with dual grid = primal grid
	[Conj] = pl_lft_llt(X, Y, X, %t, 0);
	// Check that the conjugate of the energy function is itself
	b = isequal(Y, Conj);
endfunction

function b = testCompareDirLLTFusion()
//compare pl_lft_llt with option _pl_fusion,  pl_lft_llt with option _pl_fusionsci, 
//and pl_lft_direct when computing the conjugate of discrete data 
	X = [1:1:10]';//primal grid
	Y = [34 3 1 15 13 12 22 30 32 58]';//values of an ungiven function on X
	ConjA = pl_lft_llt(X, Y, X, %f);//result with llt algorithm option _pl_fusionsci
	ConjAa= pl_lft_llt(X, Y, X, %f, 0);//result with llt algorithm option _pl_fusion
	ConjB = pl_lft_direct(X, Y, X);//result with direct algorithm (quadratic)
	ConjC = pl_lft_generic(X, Y, X, %f);
	b = isequal(ConjA, ConjAa, ConjB, ConjC);//verify the result
endfunction

function b = testLft2dEmptyS()
	X = (1:4)';
	Y = (1:4)';
	f = X * Y';
	ConjA1 = pl_lft_direct_2d(X, Y, f, [], []);
	ConjA2 = pl_lft_llt_2d   (X, Y, f, [], [], %t);
	ConjB1 = pl_lft_direct_2d(X, Y, f,  0, []);
	ConjB2 = pl_lft_llt_2d   (X, Y, f,  0, [], %t);
	ConjC1 = pl_lft_direct_2d(X, Y, f, [],  0);
	ConjC2 = pl_lft_llt_2d   (X, Y, f, [],  0, %t);
	ConjD  = pl_lft_generic (X, Y, f, [], [], %t);
	b = isEqual([], ConjA1, ConjA2, ConjB1, ConjB2, ConjC1, ConjC2, ConjD);
endfunction

function testLftLlt2dWrongSize()
  Xr = 1:4; Xc = 1:5; f = eye(5, 5);
  Sr = 1:3; Sc = 2:5;
  pl_lft_llt_2d(Xr, Xc, f, Sr, Sc);
endfunction

function b = testLlt2dvsdirect2d()
	x = linspace(-2,2,5)';y=x;	s=x;t=s;
	function z=sq(x,y)
	 z=0.5*(x.^2+y.^2);
  endfunction
  [X,Y]=ndgrid(x,y);
  Z=sq(X,Y);
	C1 = pl_lft_direct_2d(x,y,Z, s, t);
	C2 = pl_lft_llt_2d(x, y, Z, s, t, %t);
	b = isEqual(C1, C2,Z);
endfunction

function b = testLlt2dunsym()
	x = linspace(-1,1,5)';	y = linspace(-2,2,5)';
  s=x;t=2*y;
	function z=sq(x,y), z=0.5*x.^2+y.^2;endfunction
	function z=sqstar(x,y), z=0.5*x.^2+0.25*y.^2;endfunction
  [X,Y]=ndgrid(x,y);  Z=sq(X,Y);
  [S,T]=ndgrid(s,t);  Zs=sqstar(S,T);
	C1 = pl_lft_direct_2d(x,y,Z, s, t);
	C2 = pl_lft_llt_2d(x, y, Z, s, t, %t);
	b = isEqual(C1,C2,Zs)
endfunction

function b = testLlt3dSq()
	x = linspace(-2,2,5)';y=x;z=x;s=x;t=s;u=s;
	function w=sq(x,y,z), w=0.5*(x.^2+y.^2+z.^2); endfunction
  [X,Y,Z]=ndgrid(x,y,z); W=sq(X,Y,Z);
	C1 = pl_lft_llt_3d(x,y,z,W,s,t,u,%t);
	C2 = pl_lft_llt_3d(x,y,z,W,s,t,u,%f);	
	b = isEqual(C1,C2,W)
endfunction

function b = testLlt3dSq2()
	x=linspace(-1,1,5)';y=linspace(-2,2,5)';z=linspace(-4,4,5)';;
	s=x;t=2*y;u=4*z;
	function w=sq(x,y,z),w=0.5*x.^2+y.^2+2*z.^2; endfunction
	function w=sqstar(x,y,z),w=0.5*x.^2+0.25*y.^2+0.125*z.^2;endfunction
  [X,Y,Z]=ndgrid(x,y,z);  W=sq(X,Y,Z);
  [S,T,U]=ndgrid(s,t,u);  Ws=sqstar(S,T,U);
	C1 = pl_lft_llt_3d(x,y,z,W,s,t,u,%t);
	C2 = pl_lft_llt_3d(x,y,z,W,s,t,u,%f);	
	b = isEqual(C1,C2,Ws)
endfunction

function b = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testPartialConj_direct, "testPartialConj_direct"), b);
	b = checkForFail(testWrapper(testQuadratic_direct_plt, "testQuadratic_direct_plt"), b);
	b = checkForFail(testWrapper(testPartialConj_llt, "testPartialConj_llt"), b);
	b = checkForFail(testWrapper(testQuadratic_llt, "testQuadratic_llt"), b);
	b = checkForFail(testWrapper(testCompareDirLLTFusion, "testCompareDirLLTFusion"), b);
	b = checkForFail(testWrapper(testLft2dEmptyS, "testLft2dEmptyS"), b);
	b = checkForFail(errorTestWrapper(testLftLlt2dWrongSize, "testLftLlt2dWrongSize"), b);
	b = checkForFail(testWrapper(testLlt2dvsdirect2d, "testLlt2dvsdirect2d"), b);	
	b = checkForFail(testWrapper(testLlt2dunsym, "testLlt2dunsym"), b);		
	b = checkForFail(testWrapper(testLlt3dSq, "testLlt3dSq"), b);		
	b = checkForFail(testWrapper(testLlt3dSq2, "testLlt3dSq2"), b);			
endfunction
