// Author:   Mike Trienis
// For:      Dr. Yves Lucet
// Whatsit:  A test file for plq_add, adds two piecewise linear quadratic functions. 

mode(-1);
//L: Linear, Q: Quadratic, I: Indicator, PW: Piecewise

__test_set = "PLQ: plq_add";

function [b] = testQLQNonsmPlusQLQNonsm()
	b = %f;
	plqf = [-1,1,0,0;1,0,0,1;%inf,1,0,0];
	plqg = [0,1,0,0;2,0,1,0;%inf,1,0,-2];
	result=plq_add(plqf,plqg);
	desired = [-1,2,0,0;0,1,0,1;1,0,1,1;2,1,1,0;%inf,2,0,-2];          
	b = and(result==desired);
endfunction

function [b] = testQQSmoothPlusQQSmooth()
	b = %f;
	plqf = [-1,1,1,1; %inf,1,1,1];
	plqg = [-2,1,1,0; %inf,1,1,0];
	result = plq_add(plqf,plqg);
	desired = [%inf,2,2,1];          
	b = and(result==desired);
endfunction

//Note this function tests when there is a common point between the two PLQ functions
function [b] = testQLSmoothPlusQLQNonsm()
	b = %f;
	plqf=[1,1,0,0;3, 0, 2, -1;%inf,3,-16,26];
	plqg=[0,1,0,0;1, 0, 1, 0;%inf,3, -4, 2];
	result = plq_add(plqf,plqg);
	desired = [0,2,0,0;1,1,1,0;3,3,-2,1;%inf,6,-20,28];          
	b = and(result==desired);
endfunction

function [b] = testIPlusQ()
	b = %f;
	plqf=[4,0,0,5];
	plqg=[%inf,1/2,0,0];
	result = plq_add(plqf,plqg);
	desired = [4,0,0,13];          
	b = and(result==desired);
endfunction

function [b] = testIPlusQPW()
	b = %f;
	plqf=[4,0,0,5];
	plqg=[0,0,-1,0;%inf,0,1,0];
	result = plq_add(plqf,plqg);
	desired = [4,0,0,9];          
	b = and(result==desired);
endfunction

function [b] = testIPlusQPW2()
	b = %f;
	plqg=[4,0,0,5];
	plqf=[0,0,-1,0;%inf,0,1,0];
	result = plq_add(plqf,plqg);
	desired = [4,0,0,9];          
	b = and(result==desired);
endfunction

function [b] = testIPlusPL()
	b = %f;
	Ii=[-0.5,0,0,%inf;0.5,0,0,0;%inf,0,0,%inf];
	plq=[-1,0,-1,-1;0,0,1,1;1,0,-1,1;%inf,0,1,-1];
	result = plq_add(Ii,plq);
	desired = [-0.5,0,0,%inf;0,0,1,1;0.5,0,-1,1;%inf,0,0,%inf];
	b = and(result==desired);
endfunction

function [b] = testIPlusI()
  b = %f;
  f1 = [1,0,0,3];
  f2 = [1,0,0,10];
  result = plq_add(f1, f2);
  desired = [1,0,0,13];
  b = isEqual(desired, result);
endfunction

function [b] = testIPlusI2()
  b = %f;
  f1 = [1,0,0,3];
  f2 = [2,0,0,10];
  result = plq_add(f1, f2);
  desired = [%inf,0,0,%inf];
  b = isEqual(desired, result);
endfunction

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testQLQNonsmPlusQLQNonsm,"testQLQNonsmPlusQLQNonsm"), b);
	b = checkForFail(testWrapper(testQLSmoothPlusQLQNonsm,"testQLSmoothPlusQLQNonsm"), b);
	b = checkForFail(testWrapper(testQQSmoothPlusQQSmooth,"testQQSmoothPlusQQSmooth"), b);
	b = checkForFail(testWrapper(testIPlusQ,"testIPlusQ"), b);
	b = checkForFail(testWrapper(testIPlusQPW,"testIPlusQPW"), b);
	b = checkForFail(testWrapper(testIPlusQPW2,"testIPlusQPW2"), b);
	b = checkForFail(testWrapper(testIPlusPL,"testIPlusPL"), b);
	b = checkForFail(testWrapper(testIPlusI,"testIPlusI"), b);
	b = checkForFail(testWrapper(testIPlusI2,"testIPlusI2"), b);
endfunction
