__test_set = "PLQ: plq_build";
mode(-1);

// unit tests ==========================================================
//1st order model
function [b] = testexp()
	x=linspace(0,4)';
	plqf = plq_build(x,exp,exp);
	y=plq_eval(plqf,x);
	b = norm(y-exp(x))<1E-8;
endfunction

function [b] = testlinear()
	function y=f(x),y=2*x+1;endfunction;
	function dy=df(x),dy=2*ones(x);endfunction;
	x=linspace(-2,2,5)';
	plqf = plq_build(x,f,df);
	y=plq_eval(plqf,x);
	b = norm(y-f(x))<1E-8;
endfunction

function [b] = testpartiallylinear()
	function y=f(x),i=find(x>0);y=zeros(x);y(i)=x(i).^2;endfunction
	function y=df(x),i=find(x>0);y=zeros(x);y(i)=2*x(i);endfunction
	x=linspace(-2,2,5)';
	plqf = plq_build(x,f,df);
	y=plq_eval(plqf,x);
	b = norm(y-f(x))<1E-8;
endfunction

function [b] = testconjexp()
	function y=f(x),i1=find(x<0);i2=find(x==0);i3=find(x>0);y(i1)=%inf*ones(i1);y(i2)=0;y(i3)=x(i3).*log(x(i3))-x(i3);endfunction
	function y=df(x),i1=find(x<=0);i2=find(x>0);y(i1)=%inf*ones(i1);y(i2)=log(x(i2));endfunction
	x=linspace(0,20,3)';
	plqf = plq_build(x,f,df);
	y = plq_eval(plqf,x);
	yy=f(x);i=find(x==0);yy(i)=%inf*ones(i);//plq evaluates %inf at 0
	i=isinf(y);ii=isinf(yy);//need to test %inf values separately
	b = (norm(y(~i)-yy(~ii))<1E-8) & and(y(i)==yy(i));
endfunction

function [b] = testconjexp_d()
	function y=f(x),i1=find(x<0);i2=find(x==0);i3=find(x>0);y(i1)=%inf*ones(i1);y(i2)=0;y(i3)=x(i3).*log(x(i3))-x(i3);endfunction
	function y=df(x),i1=find(x<=0);i2=find(x>0);y(i1)=%inf*ones(i1);y(i2)=log(x(i2));endfunction
	x=linspace(0,20,3)';
	y=f(x);dy=df(x);
	plqf = plq_build_d(x,y,dy);
	y = plq_eval(plqf,x);
	yy=f(x);i=find(x==0);yy(i)=%inf*ones(i);//plq evaluates %inf at 0
	i=isinf(y);ii=isinf(yy);//need to test %inf values separately
	b = (norm(y(~i)-yy(~ii))<1E-8) & and(y(i)==yy(i));
endfunction


//0th order model
function [b] = testexp0()
	x=linspace(0,4)';
	plqf = plq_build(x,exp);
	y=plq_eval(plqf,x);
	b = norm(y-exp(x))<1E-8;
endfunction

function [b] = testabs0()
	x=linspace(-4,4,5)';
	plqf = plq_build(x,abs);
	b = [0,0,-1,0;%inf,0,1,0]==plqf;
endfunction

function [b] = testabs0_d()
	x=linspace(-4,4,5)';
	y=abs(x);
	plqf = plq_build_d(x,y);
	b = [0,0,-1,0;%inf,0,1,0]==plqf;
endfunction


function [b] = testx0()
	x=linspace(-4,4)';
	function y=f(x),y=x;endfunction;
	plqf = plq_build(x,f);
	b = [%inf,0,1,0]==plqf;
endfunction

function [b] = testlinear0()
	x=linspace(-2,2,5)';function y=f(x),y=2*x+1;endfunction;
	plqf = plq_build(x,f);
	b = [%inf,0,2,1]==plqf;
endfunction

function [b] = testpartiallylinear0()
	function y=f(x),i=find(x>0);y=zeros(x);y(i)=x(i).^2;endfunction
	x=linspace(-2,2,5)';
	plqf = plq_build(x,f);
	y=plq_eval(plqf,x);
	b = norm(y-f(x))<1E-8;
endfunction

function [b] = testInf0()
  function y = f(x); ys = [%inf;4/1;4/2;4/3;4/4;%inf]; y = ys(int(x*4+1)); endfunction

  x = 0:0.25:1.25;
  plqf = plq_build(x,f,%f,%f,%t);
  desired = [1/4,0,0,%inf; 2/4,0,-8,6; 3/4,0,-8/3,10/3; 1,0,-4/3,7/3; %inf,0,0,%inf];
  b = isEqual(plqf, desired);
endfunction

function [b] = testInf1()
  //function y = f(x); i=ieee(); ieee(2); y=ones(x)./x; ieee(i); endfunction
  //function s = df(x); i=ieee(); ieee(2); s=-ones(x)./(x.^2); ieee(i); endfunction
  function y = f(x); ys = [%inf;4/1;4/2;4/3;4/4;%inf]; y = ys(int(x*4+1)); endfunction
  function dy = df(x); dys = -[%inf;[4/1;4/2;4/3;4/4].^2;%inf]; dy = dys(int(x*4+1)); endfunction

  x = 0:0.25:1.25;
  plqf = plq_build(x,f,df,%f,%t);
  //desired = [0,0,0,%inf; 1/3,0,-16,8; 0.6,0,-4,4; 6/7,0,-16/9,8/3; %inf,0,-1,2];
  desired = [1/4,0,0,%inf; 1/3,0,-16,8; 3/5,0,-4,4; 6/7,0,-16/9,8/3; 1,0,-1,2; %inf,0,0,%inf];
  b = isEqual(plqf, desired);
endfunction

function [b] = testNeedEvalFptr()
  b = %f;
  x = 0:0.5:3;
  f1 = plq_build(x, exp, %f, %f);
  f2 = plq_build(x, exp, %f, %t);
  b = isEqual(f1, f2);
endfunction

function [b] = testNeedEvalFunc()
  function y=f(x)
    if (size(x) ~= [1,1]) then error("blah"); end;
	y=exp(x);
  endfunction;
  b = %f;
  x = 0:0.5:3;
  f1 = plq_build(x, f, %f, %f);
  f2 = plq_build(x, f, %f, %t);
  b = isEqual(f1, f2);
endfunction

function [b] = testNonconvexfctn()
  b=%f;
  x=linspace(0,2*%pi,4)';
  result = plq_build(x,sin,cos);
  desired = [] //The desired result requires the x(i) values to be ordered. See result.
endfunction

function [b] = testsoqs()
  function y = f(x);  ys=[.0;.3;.5;.2;.6;1.2;1.3;1.0;1.0;1.0;0;-1.0];y=ys(int(x+1)); endfunction
  function dy = df(x); dys = [.5;1;0;-.2;5;4;0;-.2;-.2;.3;-.9999;-1.001]; dy = dys(int(x+1)); endfunction
	x=linspace(0,11,12)';
	plqf = plq_build(x,f,df,%t,%f,'soqs');
	desired = [0,0,0,0; .9,-.25,.5,0; 1,4.75,-8.5,4.05; 1.2,-2,5,-2.7]
	b = isEqual(plqf(1:4,:), desired);
endfunction

function [b] = testsoqs_noderiv()
  function y = f(x);  ys=[.0;.3;.5;.2;.6;1.2;1.3;1.0;1.0;1.0;0;-1.0];y=ys(int(x+1)); endfunction
  function dy = df(x); dys = [.5;1;0;-.2;5;4;0;-.2;-.2;.3;-.9999;-1.001]; dy = dys(int(x+1)); endfunction
	x=linspace(0,11,12)';
	plqf = plq_build(x,f,%f,%t,%f,'soqs');
	desired = [0,0,0,0; .5,-.06,.36,0; 1,-.06,.36,0]
	b = isEqual(plqf(1:3,:), desired);
endfunction

function b = testsoqs1()
  function y=f(x), y=x.^4;endfunction
  function dy=df(x), dy=4*x.^3;endfunction
  x=linspace(-10,10,2)';
	p = plq_build(x,f,df,%f,%f,'soqs','bounded');
	d = [-10 0 0 %inf;0 200 0 -10000;10 200 0 -10000;%inf 0 0 %inf];
	b = plq_isEqual(p, d);
endfunction

function b = testsoqs2()
  function y=f(x), y=x.^4;endfunction
  function dy=df(x), dy=4*x.^3;endfunction
  x=linspace(-10,10,2)';
  b=%f;
  ierr=execstr("plq_build(x,f,df,%f,%f,""soqs"",""wrongOption"");",'errcatch');
  b=ierr<>0;
endfunction

function [b] = runTestFile()
  b = %t;
  b = checkForFail(testWrapper(testsoqs1, "testsoqs1"), b);  
  b = checkForFail(testWrapper(testsoqs2, "testsoqs2"), b);    
  b = checkForFail(testWrapper(testexp, "testexp"), b);
  b = checkForFail(testWrapper(testlinear, "testlinear"), b);
  b = checkForFail(testWrapper(testpartiallylinear, "testpartiallylinear"), b);
  b = checkForFail(testWrapper(testconjexp, "testconjexp"), b);
  b = checkForFail(testWrapper(testexp0, "testexp0"), b);
  b = checkForFail(testWrapper(testabs0, "testabs0"), b);
  b = checkForFail(testWrapper(testx0, "testx0"), b);
  b = checkForFail(testWrapper(testlinear0, "testlinear0"), b);
  b = checkForFail(testWrapper(testpartiallylinear0, "testpartiallylinear0"), b);
  b = checkForFail(testWrapper(testconjexp_d, "testconjexp_d"), b);
  b = checkForFail(testWrapper(testabs0_d, "testabs0_d"), b);
  b = checkForFail(testWrapper(testInf0, "testInf0"), b);
  b = checkForFail(testWrapper(testInf1, "testInf1"), b);
  b = checkForFail(testWrapper(testNeedEvalFptr, "testNeedEvalFptr"), b);
  b = checkForFail(testWrapper(testNeedEvalFunc, "testNeedEvalFunc"), b);
  //b = checkForFail(testWrapper(testNonconvexfctn, "testNonconvexfctn"),//b);
  b = checkForFail(testWrapper(testsoqs, "testsoqs"), b);  
  b = checkForFail(testWrapper(testsoqs_noderiv, "testsoqs_noderiv"), b);  
endfunction
