// Author:   Mike Trienis
// For:      Dr. Yves Lucet
// Whatsit:  A test file for plq convex hull functions plq_coDirect and _plq_conv_on_interval

// I = Indicator, E = Exponential, C = Conjugate, L = Linear, Q = Quadratic

__test_set = "PLQ: plq_coDirect, _plq_conv_on_interval";
mode(-1);

function [b] = testNCExt_1()
	// testL2Lch_1 in test_plq_coSplit.sci.
	// Bounded on both sides.
	b = %f;
	plqf = [-1,0,0,%inf; 1,0,1,-1; 4,0,0,0; %inf,0,0,%inf];
	[result, iters] = plq_coDirect(plqf);
	desired = [-1,0,0,%inf; 4,0,0.4,-1.6; %inf,0,0,%inf];
	if size(result) == size(desired) then
		b = and(result == desired | abs(result - desired) < 1E-5);
		b = b & (iters == 1);
	else
		b = %f;
	end;
endfunction

function [b] = testNCExt_2()
	// Bounded on the left, co is slanted.
	b = %f;
	plqf = [1,0,1,-1; 4,0,0,0; %inf,0,0,%inf];
	[result, iters] = plq_coDirect(plqf);
	desired = [4,0,1,-4; %inf,0,0,%inf];
	if size(result) == size(desired) then
		b = and(result == desired | abs(result - desired) < 1E-5);
		b = b & (iters == 1);
	else
		b = %f;
	end;
endfunction

function [b] = testNCExt_3()
	// Bounded on the right, co is flat.
	b = %f;
	plqf = [-1,0,0,3; 3,0,-1,2; %inf,0,0,%inf];
	[result, iters] = plq_coDirect(plqf);
	desired = [3,0,0,-1; %inf,0,0,%inf];
	if size(result) == size(desired) then
		b = and(result == desired | abs(result - desired) < 1E-5);
		b = b & (iters == 1);
	else
		b = %f;
	end;
endfunction

function [b] = testNCExt_4()
	// Bounded upside down quadratic.
	b = %f;
	plqf = [-1,0,0,%inf; 2,-1,0,1; %inf,0,0,%inf];
	[result, iters] = plq_coDirect(plqf);
	desired = [-1,0,0,%inf; 2,0,-1,-1; %inf,0,0,%inf];
	if size(result) == size(desired) then
		b = and(result == desired | abs(result - desired) < 1E-5);
		b = b & (iters == 0);
	else
		b = %f;
	end;
endfunction

//---------------------------//

function [b] = testQ2Lch_1()
	b = %f;
	plqf = [0,1,0,0;3,0,0,0];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2.5;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
  desired = plqf;          
	b = and(abs(result-desired) < 1E-5);
endfunction


function [b] = testL2Qch_2()
	b = %f;
	plqf = [-1,0,0,1;3,1,0,0];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2.5;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
  desired = [-0.2087122,0,-0.4174243,-0.0435608;3,1,0,0];          
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testL2Qch_3()
	b = %f;
	plqf = [0,0,0,0;2,-1,0,0];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
  desired =[2,0,-1,-2];         
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testQ2Lch_3()
	b = %f;
	plqf = [1,1,0,0;4,0,0,1]; 
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2.5;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
  desired = [0.1270167,1,0,0;4,0,0.2540333,-0.0161332];          
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testQ2Lch_4()
	b = %f;
	plqf = [1,-1,0,1;3,0,0,0];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
	desired = [3,0,0.6,-1.8];  
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testQ2Lch_5()
    // This test was added to test the x(1)==-%inf & x(5)==%inf &
    // a(1)<>0 & a(2)==0 case of _plq_conv_buildl.

    // _plq_conv_on_interval:
    // a(1) >= 0, a(2) >= 0
    // a(1) <> 0, a(2) == 0
    // a(1) > 0, a(2) == 0

    // _conv_interval_func:
    // (a(1) == 0 | x(5) == %inf) & (a(2) == 0 | x(1) <> -%inf)
    // x(5) == %inf, a(2) == 0

    // _plq_conv_buildl:
    // a(1) > 0, a(2) == 0
    // x(1) == -%inf, x(5) == %inf

    b = %f;
    plqf = [0,1,1,0;%inf,0,0,0];
    result = _plq_conv_on_interval(plqf(1,:), plqf(2,:), -%inf, 0, %inf);
    result($,1)=0;
    desired = [-0.5,1,1,0;0,0,0,-0.25];
    b = and(norm(result-desired) < 1E-5);
endfunction

function [b] = testQ2Qch_1()
	b = %f;
	plqf = [3/2,1,0,0;3,1,-4,6];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2.5;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
  desired=[0.5,1,0,0;2.5,0,1,-0.25;3,1,-4,6];         
	b = and(abs(result-desired) < 1E-5);
endfunction

   
function [b] = testQ2Qch_2()
	b = %f;
	plqf = [0,1,1,2;5,1,-1,2];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2.5;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
	desired=[-0.5,1,1,2;0.5,0,0,1.75;5,1,-1,2];      
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testQ2Qch_3()
	b = %f;
	plqf = [0,2,4,0;2,2,-4,0];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
	desired =[-1,2,4,0;1,0,0,-2;2,2,-4,0];       
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testQ2Qch_4()
	b = %f;
	plqf = [1,-1,0,2;5,1,-4,4];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
	desired=[2.2426407,0,0.4852814,-1.0294373;5,1,-4,4];        
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testQ2Qch_5()
	b = %f;
	plqf = [0,-1,-4,0;2,-1,4,0];
	f1 = plqf(1,:);
  f2 = plqf(2,:);
  x1=-2;
  x3=f1(1,1);
  x5=f2(1,1);
	result = _plq_conv_on_interval(f1,f2,x1,x3,x5);
	desired=[0,0,-2,0;2,0,2,0];       
	b = and(abs(result-desired) < 1E-5);
endfunction

function [b] = testQ2Qch_6()
	b = %f;
	plqf = [0,2,4,0;%inf,2,-4,0];
	[result, iters] = plq_coDirect(plqf); result($,1)=0;
	desired=[-1,2,4,0;1,0,0,- 2;%inf,2,-4,0]; desired($,1)=0;
	b = (iters == 1) & and(norm(result-desired) < 1E-5);
endfunction

function [b] = testQ2Qch_7()
    // This test was added to test the x(1)==-%inf & x(5)==%inf &
    // a(1)<>0 & a(2)==0 case of _plq_conv_buildl.

    // _plq_conv_on_interval:
    // a(1) <> 0, a(2) <> 0

    // _conv_interval_func:
    // (a(1) == 0 | x(5) == %inf) & (a(2) == 0 | x(1) <> -%inf)
    // x(5) == %inf, a(2) == 0

    // _plq_conv_buildl:
    // a(1) <> 0, a(2) <> 0

    b = %f;
    plqf = [0,2,1,0;%inf,1,0,0];
    result = _plq_conv_on_interval(plqf(1,:), plqf(2,:), -%inf, 0, %inf);
    desired = [-0.1464466,2,1,0;0.2071068,0,0.4142136,-0.0428932;%inf,1,0,0];
    b = isEqual(result, desired);
endfunction

//---------------------------//

function [b] = testL2L2Qch_7()
	b = %f;
	plqf = [0,0,1,0;2,0,0,0;%inf,1,-4,4];
	[result, iters] = plq_coDirect(plqf); result($,1)=0;
	desired  = [2.5,0,1,-2.25;%inf,1,-4,4];  desired($,1)=0;
	b = (iters == 2) & and(norm(result-desired) < 1E-5);
endfunction

function [b] = testQ2Q2Qch_8()
	b = %f;
	plqf =[-2,1,8,16;2,-1,0,8;%inf,1,-8,16]; 
	[result, iters] = plq_coDirect(plqf); result($,1)=0;
	desired  = [-4,1,8,16;4,0,0,0;%inf,1,-8, 16]; desired($,1)=0; 
	b = (iters == 3) & and(norm(result-desired) < 1E-5);
endfunction

//---------------------------//

function [b] = testQ2L2L2Qch_10()
	b = %f;
	plqf =[-3,1,8,16;0,0,1,4;3,0,-1,4;%inf,1,-8,16]; 
	[result, iters] = plq_coDirect(plqf); result($,1)=0;
	desired =  [-4,1,8,16;4,0,0,0;%inf,1,-8,16];  desired($,1)=0; 
	b = (iters == 3) & and(norm(result-desired) < 1E-5);
endfunction

function [b] = testQ2L2L2Qch_11()
	b = %f;
	plqf =[-3,1,8,16;0,0,-1,-2;3,0,1,-2;%inf,1,-8,16]; 
	[result, iters] = plq_coDirect(plqf); result($,1)=0;
	desired = [-4.2426407,1,8,16;0,0,-0.4852814,-2;4.2426407,0,0.4852814,-2;%inf,1,-8,16]; desired($,1)=0; 
	b = (iters == 3) & and(norm(result-desired) < 1E-5);
endfunction

//---------------------------//

function [b] = testL2Q2Lch_12()
	b = %f;
	plqf =[-2,0,-1,10;2,1,0,8;%inf,0,1,10];  
	[result, iters] = plq_coDirect(plqf); result($,1)=0;
	desired=[-0.5,0,-1,7.75;0.5,1,0,8;%inf,0,1,7.75];  desired($,1)=0;     
	b = (iters == 2) & and(norm(result-desired) < 1E-5);
endfunction

function [b] = testL2Q2Lch_13()
	b = %f;
	plqf =[-2,0,-1,2;1,-1,0,8;%inf,0,1,6]; 
	[result, iters] = plq_coDirect(plqf); result($,1)=0;
	desired=[- 2,0,-1,2;%inf,0,1,6]; desired($,1)=0; 
	b = (iters == 1) & and(norm(result-desired) < 1E-5);
endfunction

//---------------------------//

function [b] = testPosInfHull_0()
	b = %f;
	plqf = [%inf,0,0,%inf];
	[result, iters] = plq_coDirect(plqf);
	desired = [%inf,0,0,%inf];
	b = (iters == 0) & (result == desired);
endfunction

function [b] = testNegInfHull_0()
	b = %f;
	plqf = [%inf,0,0,-%inf];
	[result, iters] = plq_coDirect(plqf);
	desired = [%inf,0,0,-%inf];
	b = (iters == 0) & (result == desired);
endfunction

function [b] = testNegInfHull_1()
    b = %f;
	plqf = [0,-1,-2,1;1,0,0,1;%inf,1,1,-1];
	[result, iters] = plq_coDirect(plqf);
	desired = [%inf,0,0,-%inf];
	b = (iters == 0) & (result == desired);
endfunction

function [b] = testNegInfHull_2()
    b = %f;
	plqf = [-1,1,4,3;1,0,2.5,2.5;3,1,-2,6;%inf,-2,12,-9];
	[result, iters] = plq_coDirect(plqf);
	desired = [%inf,0,0,-%inf];
	b = (iters == 0) & (result == desired);
endfunction

function [b] = testNegInfHull_3()
    b = %f;
	plqf = [0,0,-1,0;1,1,0,0;%inf,0,-2,3];
	[result, iters] = plq_coDirect(plqf);
	desired = [%inf,0,0,-%inf];
	b = (iters == 0) & (result == desired);
endfunction

                                     
//---------------------------//

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testNCExt_1, "testNCExt_1"), b);
	b = checkForFail(testWrapper(testNCExt_2, "testNCExt_2"), b);
	b = checkForFail(testWrapper(testNCExt_3, "testNCExt_3"), b);
	b = checkForFail(testWrapper(testNCExt_4, "testNCExt_4"), b);
	b = checkForFail(testWrapper(testQ2Lch_1, "testQ2Lch_1"), b);
	b = checkForFail(testWrapper(testL2Qch_2, "testL2Qch_2"), b);
	b = checkForFail(testWrapper(testL2Qch_3, "testL2Qch_3"), b);
	b = checkForFail(testWrapper(testQ2Lch_3, "testQ2Lch_3"), b);
	b = checkForFail(testWrapper(testQ2Lch_4, "testQ2Lch_4"), b);
	b = checkForFail(testWrapper(testQ2Lch_5, "testQ2Lch_5"), b);
	b = checkForFail(testWrapper(testQ2Qch_1, "testQ2Qch_1"), b);
	b = checkForFail(testWrapper(testQ2Qch_2, "testQ2Qch_2"), b);
	b = checkForFail(testWrapper(testQ2Qch_3, "testQ2Qch_3"), b);
	b = checkForFail(testWrapper(testQ2Qch_4, "testQ2Qch_4"), b);
  	b = checkForFail(testWrapper(testQ2Qch_5, "testQ2Qch_5"), b);
  	b = checkForFail(testWrapper(testQ2Qch_6, "testQ2Qch_6"), b);
	b = checkForFail(testWrapper(testQ2Qch_7, "testQ2Qch_7"), b);
  	b = checkForFail(testWrapper(testL2L2Qch_7, "testL2L2Qch_7"), b);
  	b = checkForFail(testWrapper(testQ2Q2Qch_8, "testQ2Q2Qch_8"), b);
  	b = checkForFail(testWrapper(testQ2L2L2Qch_10, "testQ2L2L2Qch_10"), b);
  	b = checkForFail(testWrapper(testQ2L2L2Qch_11, "testQ2L2L2Qch_11"), b);
  	b = checkForFail(testWrapper(testL2Q2Lch_12, "testL2Q2Lch_12"), b);
  	b = checkForFail(testWrapper(testL2Q2Lch_13, "testL2Q2Lch_13"), b);
  	b = checkForFail(testWrapper(testPosInfHull_0, "testPosInfHull_0"), b);
  	b = checkForFail(testWrapper(testNegInfHull_0, "testNegInfHull_0"), b);
  	b = checkForFail(testWrapper(testNegInfHull_1, "testNegInfHull_1"), b);
  	b = checkForFail(testWrapper(testNegInfHull_2, "testNegInfHull_2"), b);
  	b = checkForFail(testWrapper(testNegInfHull_3, "testNegInfHull_3"), b);
endfunction
