// Author:   Mike Trienis
// For:      Dr. Yves Lucet
// Whatsit:  A test file for plq

mode(-1);
//L: Linear, Q: Quadratic, I: Indicator

__test_set = "PLQ: Convolutions";

function b = testInfconvLft()
  b = %f;
  plqf = [%inf,1/4,0,0];
  plqg = [%inf,1,0,0];
  result = plq_infconv_lft(plqf,plqg);
  desired = [%inf,1/5,0,0];
  b = and(result==desired);
endfunction

function b = testDeconvLft()
  b = %f;
  plqf = [%inf,1/4,0,0];
  plqg = [%inf,1,0,0];
  result = plq_deconv_lft(plqf,plqg);
  desired = [%inf,1/3,0,0];
  b = and(result==desired);
endfunction

function b = testInfconvBruteDL1()
  b = %f;
  f = [-1,0,0,%inf;1,0,-1,0;%inf,0,0,%inf];
  g = [-1,0,0,%inf;1,0,1,0;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1, lb1] = plq_infconv_brute_d(f(2,:), g(2,:), [-1;-1], [1;1]);
  [result2, lb2] = plq_infconv_brute_d(g(2,:), f(2,:), [-1;-1], [1;1]);
  result1 = [lb1,0,0,%inf;result1;%inf,0,0,%inf];
  result2 = [lb2,0,0,%inf;result2;%inf,0,0,%inf];
  b = isEqual(desired, result1, result2),
endfunction

function b = testInfconvBruteDL2()
  b = %f;
  f = [-0.5,0,0,%inf;1.5,0,-1,0.5;%inf,0,0,%inf];
  g = [-1,0,0,%inf;1,0,1,0;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1, lb1] = plq_infconv_brute_d(f(2,:), g(2,:), [-0.5;-1], [1.5;1]);
  [result2, lb2] = plq_infconv_brute_d(g(2,:), f(2,:), [-1;-0.5], [1;1.5]);
  result1 = [lb1,0,0,%inf;result1;%inf,0,0,%inf];
  result2 = [lb2,0,0,%inf;result2;%inf,0,0,%inf];
  b = isEqual(desired, result1, result2),
endfunction

function b = testInfconvBruteDL3()
  b = %f;
  f = [-1,0,0,%inf;1,0,-1,0;%inf,0,0,%inf];
  g = [2,0,0,%inf;4,0,1,-3;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1, lb1] = plq_infconv_brute_d(f(2,:), g(2,:), [-1;2], [1;4]);
  [result2, lb2] = plq_infconv_brute_d(g(2,:), f(2,:), [2;-1], [4;1]);
  result1 = [lb1,0,0,%inf;result1;%inf,0,0,%inf];
  result2 = [lb2,0,0,%inf;result2;%inf,0,0,%inf];
  b = isEqual(desired, result1, result2),
endfunction

function b = testInfconvBruteL0()
  // nf==1, ng==1, f up, g up.
  b = %f;
  f = [-1,0,0,%inf;1,0,1,0;%inf,0,0,%inf];
  g = [0,0,0,%inf;2,0,2,-1/2;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1, am1] = plq_infconv_brute(f, g);
  [result2, am2] = plq_infconv_brute(g, f);
  b = isEqual(desired, result1, result2),
  //b = b & isEqual(am1, am2),
endfunction

function b = testInfconvBruteL1()
  // nf==2, ng==1, f v, g up.
  b = %f;
  f = [-1,0,0,%inf;1,0,-1,0;3,0,1,-2;%inf,0,0,%inf];
  g = [-1,0,0,%inf;1,0,1,0;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1, am1] = plq_infconv_brute(f, g);
  [result2, am2] = plq_infconv_brute(g, f);
  b = isEqual(desired, result1, result2),
  //b = b & isEqual(am1, am2),
endfunction

function b = testInfconvBruteL2()
  // nf==2, ng==2, f v, g v.
  b = %f;
  f = [-1,0,0,%inf;1,0,-1,0;3,0,1,-2;%inf,0,0,%inf];
  g = [-3,0,0,%inf;-1,0,-1,-1;1,0,1,1;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1, am1] = plq_infconv_brute(f, g);
  [result2, am2] = plq_infconv_brute(g, f);
  b = isEqual(desired, result1, result2),
  //b = b & isEqual(am1, am2),
endfunction

function b = testInfconvBruteL3()
  // nf==3, ng==3, f u, g down, disjoint domains.
  b = %f;
  f = [-4,0,0,%inf;-3,0,-1,-2;-2,0,0,1;-1,0,1,3;%inf,0,0,%inf];
  g = [0,0,0,%inf;1,0,-5,8;2,0,-2,5;3,0,-1/2,2;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1,am1] = plq_infconv_brute(f, g);
  [result2,am2] = plq_infconv_brute(g, f);
  b = isEqual(desired, result1, result2),
  //b = b & isEqual(am1, am2);
endfunction

function plqf = plq_pl_from(x0, y0, dx, ms)
  if (size(ms, 1) > size(ms, 2)) then ms = ms'; end;
  plqf = zeros(length(ms) + 2, 4);
  x = x0;
  y = y0;
  plqf(1,:) = [x0, 0, 0, %inf];
  i = 2;
  for m = ms
    //if (%t | m < 0 & grand(1,'uin',0,1) == 1) then y = y - grand(1,1,'unf',0,10); end;
    //if (%t | m > 0 & grand(1,'uin',0,1) == 1) then y = y + grand(1,1,'unf',0,10); end;
    plqf(i,:) = [x + dx, 0, m, y - m*x];
    x = x + dx;
    y = plqf(i,3)*x + plqf(i,4);
    i = i + 1;
  end;
  plqf($,:) = [%inf, 0, 0, %inf];
  //[z, order] = gsort(plqf(:,1), 'r', 'i');
  //for i = 2:size(plqf,1)-2
  //  x = plqf(i,1);
  //  d = plqf(i,2)*x^2 + plqf(i,3)*x + plqf(i,4) - (plqf(i+1,2)*x^2 + plqf(i+1,3)*x + plqf(i+1,4));
  //  if (abs(d) > 1e-10) then
  //    disp(plqf);
  //    cerror(sprintf("plq_build2 error, pieces %d, %d, diff %f.", i, i+1, d));
  //    return;
  //  end;
  //end;
  //plqf = plqf(order,:);
endfunction

function b = testInfconvBruteL4()
  b = %f;
  //f = [0,0,0,%inf;2,0,0,1;3,0,0,0;6,0,0,1;%inf,0,0,%inf];
  f = [0.5,0,0,%inf;1.5,0,-1,1.5;2.5,0,0,0;5,0,0.4,-1;%inf,0,0,%inf];
  g = [0,0,0,%inf;1,0,-1,2;3,0,1,0;%inf,0,0,%inf];
  desired = plq_infconv_lft(f, g);
  [result1, am1] = plq_infconv_brute(f, g);
  [result2, am2] = plq_infconv_brute(g, f);
  b = isEqual(desired, result1, result2),
  //b = b & isEqual(am1, am2);
endfunction

function b = testInfconvBruteLCommut1()
  b = %f;
  f = plq_pl_from(0, 0, 1, 1:4);
  g = plq_pl_from(0.1, -2, 0.3, 1:8);
  desired = plq_infconv_lft(f, g);
  [result1, am1] = plq_infconv_brute(f, g);
  [result2, am2] = plq_infconv_brute(g, f);
  b = isEqual(desired, result1, result2),
  //b = b & isEqual(am1, am2);
endfunction

function b = runTestFile()
    b = %t;
    b = checkForFail(testWrapper(testInfconvLft,"testInfconvLft"), b);
    b = checkForFail(testWrapper(testDeconvLft,"testDeconvLft"), b);
    b = checkForFail(testWrapper(testInfconvBruteDL1,"testInfconvBruteDL1"), b);
    b = checkForFail(testWrapper(testInfconvBruteDL2,"testInfconvBruteDL2"), b);
    b = checkForFail(testWrapper(testInfconvBruteDL3,"testInfconvBruteDL3"), b);
    b = checkForFail(testWrapper(testInfconvBruteL0,"testInfconvBruteL0"), b);
    b = checkForFail(testWrapper(testInfconvBruteL1,"testInfconvBruteL1"), b);
    b = checkForFail(testWrapper(testInfconvBruteL2,"testInfconvBruteL2"), b);
    b = checkForFail(testWrapper(testInfconvBruteL3,"testInfconvBruteL3"), b);
    b = checkForFail(testWrapper(testInfconvBruteL4,"testInfconvBruteL4"), b);
    b = checkForFail(testWrapper(testInfconvBruteLCommut1,"testInfconvBruteLCommut1"), b);
endfunction
