// Unit test file for plq_gph
mode(-1);

__test_set = "PLQ: plq_gph - gph to plq convertion";

function [b] = testAbs()
	b = %f;
	//abs function
	gph = [-1, 0, 0, 1; ..
	       -1, -1, 1, 1; ..
	       1, 0, 0, 1];
	plq = plq_gph(gph);

        expected = [0,0,-1,0; %inf,0,1,0];
        b = and(plq == expected);
endfunction

function [b] = testInd()
	b = %f;
	// The function I_[-1,1].
        gph = [-1, -1, 1, 1; ..
	       -1, 0, 0, 1; ..
	       %inf, 0, 0, %inf];
	plq = plq_gph(gph);

        expected = [-1,0,0,%inf; 1,0,0,0; %inf,0,0,%inf];
        b = and(plq == expected);
endfunction

function [b] = testPL()
	b = %f;
	// A "\_/" function.
	gph = [-1, 0, 0, 1, 1, 2; ..
	       -1, -1, 0, 0, 1, 1; ..
	       1, 0, 0, 0, 0, 1];
	plq = plq_gph(gph);

        expected = [0,0,-1,0; 1,0,0,0; %inf,0,1,-1];
        b = and(plq == expected);
endfunction

function [b] = testPLQ1()
	b = %f;
	// A PLQ function:
	//     x<-1: y=(x+1)^2
	//   -1<x<1: y=0
	//      1<x: y=(x-1)^2
	gph = [-2, -1, 1, 2; ..
	       -2,  0, 0, 2; ..
	       1, 0, 0, 1];
	plq = plq_gph(gph);

        expected = [-1,1,2,1; 1,0,0,0; %inf,1,-2,1];
        b = and(plq == expected);
endfunction

function [b] = testPLQ2()
	b = %f;
	// A PLQ function:
	//   x in [-1,1]: 0
	//     otherwise: x^2 - 1
	gph = [-2, -1, -1, 1, 1, 2; ..
	       -4, -2, 0, 0, 2, 4; ..
	       3, 0, 0, 0, 0, 3];
	plq = plq_gph(gph);

        expected = [-1,1,0,-1; 1,0,0,0; %inf,1,0,-1];
        b = and(plq == expected);
endfunction

function [b] = testFullDomain()
  b = %f;
  p = [%inf, 1, 0, 0];f=gph_plq(p);
  e=[-1 1;-2 2;1 1];
  b=and(f==e);
endfunction

function [b] = testIndicator()
  b = %f;
  p = [1, 0, 0, 1];f=gph_plq(p);
  e=[1 1;-1 1;1 1];
  b=and(f==e);
endfunction

function [b] = runTestFile()
  b = %t;
  b = checkForFail(testWrapper(testAbs,"testAbs"), b);
  b = checkForFail(testWrapper(testInd,"testInd"), b);
  b = checkForFail(testWrapper(testPL,"testPL"), b);
  b = checkForFail(testWrapper(testPLQ1,"testPLQ1"), b);
  b = checkForFail(testWrapper(testPLQ2,"testPLQ2"), b);
  b = checkForFail(testWrapper(testFullDomain,"testFullDomain"), b);  
  b = checkForFail(testWrapper(testIndicator,"testIndicator"), b);  
endfunction
