// Author:   Mike Trienis
// For:      Dr. Yves Lucet
// Whatsit:  A test file for plq

mode(-1);
//L: Linear, Q: Quadratic, I: Indicator

__test_set = "PLQ: plq_lft - Linear Fenchel Conjugate";

function [b] = testLLNonsmooth()
	b = %f;
	plqf = [ -1, 0, -1, -1; %inf, 0, 1, 1];
	result = plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	desired = [-1, 0, 0, %inf; 1, 0, -1, 0; %inf, 0, 0, %inf];          
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction


function [b] = testQQSmooth()
	b = %f;
	plqf = [-2,1,1,1;%inf,1,1,1];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	//desired = [-3, 1/4, -1/2, -3/4; %inf, 1/4, -1/2, -3/4];  
	desired = [%inf, 1/4, -1/2, -3/4];
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction

function [b] = testQQQSmooth()
	b = %f;
	plqf = [-2,1,1,1;2,1,1,1;%inf,1,1,1];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	//desired = [-3, 1/4, -1/2, -3/4; 5, 1/4, -1/2, -3/4; %inf,1/4,-1/2,-3/4];
	desired = [%inf,1/4,-1/2,-3/4];
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction

function [b] = testabs()
	b = %f;
	plqf = [0,0,-1,0;%inf,0,1,0];//abs
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	desired = [-1, 0, 0, %inf; 1, 0, 0, 0; %inf, 0, 0, %inf];          
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction

function [b] = testL()
	b = %f;
	plqf=[%inf,0,1,1];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	desired = [1, 0, 0, -1];          
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction

function [b] = testI()
	b = %f;
	plqf=[1,0,0,-1];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	desired = [%inf, 0, 1, 1];          
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction

function [b] = testI2()
	b = %f;
	plqf=[1,0,0,-1];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	desired = [%inf, 0, 1, 1];          
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction


function [b] = testNormSquared()
	b = %f;
	plqf=[%inf,1/2,0,0];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	desired = [%inf, 1/2, 0, 0];          
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction	

function [b] = testQQNonsmooth()
	b = %f;
	plqf=[0,1,0,0;%inf,2,0,0];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	desired = [0 , 1/4, 0, 0; %inf, 1/8, 0, 0];          
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction	

function [b] = testQLQSmooth()
	b = %f;
	plqf=[1,1,0,0;3, 0, 2, -1;%inf,3,-16,26];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	s=result(:,1);sa=result(:,2);sb=result(:,3);sc=result(:,4);
	resultLHS = [s];
	result2LHS = result2(:,1);
	desiredLHS = [2; %inf];
	resultRHS = [sa, sb, sc];
	result2RHS = result2(:,2:4);
	desiredRHS =[1/4, 0, 0; 1/12, 8/3, -14/3];
	b = and((resultLHS==desiredLHS) & (norm(resultRHS - desiredRHS)<1E-8));
endfunction	


function [b] = testQLQNonsmooth()
	b = %f;
	plqf=[0,1,0,0;1, 0, 1, 0;%inf,3, -4, 2];
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	s=result(:,1);sa=result(:,2);sb=result(:,3);sc=result(:,4);
	resultLHS = [s];
	result2LHS = result2(:,1);
	desiredLHS = [0; 1; 2; %inf];
	resultRHS = [sa, sb, sc];
	result2RHS = result(:,2:4);
	desiredRHS =[1/4, 0, 0; 0, 0, 0; 0,1,-1;1/12,2/3,-2/3];
	b = and((resultLHS==desiredLHS) & (norm(resultRHS - desiredRHS)<1E-8));
endfunction	


function [b] = testPLQboundedLHS()
	b = %f;
	plqf=[-5,0,0,%inf;%inf,1,0,5];
	desired = [-10, 0, -5, -30;%inf,1/4,0,-5];          
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction	

function [b] = testPLQboundedRHS()
	b = %f;
	plqf=[-5,1,0,5;%inf,0,0,%inf];
	desired = [-10, 1/4, 0, -5;%inf,0,-5,-30];          
	result= plq_lft(plqf);
	result2 = mikeplq_lft(plqf);
	b = and(result==desired);
	if (~b) then return; end;
	b = and(result2==desired);
endfunction	

function [b] = testPiecewiseLinear()
	b = %f;
	plqf=plq_build(linspace(-2,2,10),exp,exp,%f);
	result= plq_lft(plqf);
	desired = mikeplq_lft(plqf);
	b = and(result==desired);
endfunction	

function [b] = testNonconvex()
	b = %f;
	plqf=[-1,0,-1,-1;0,0,1,1;1,0,-1,1;%inf,0,1,-1];
	result= plq_lft(plqf,%f);
	desired = [-1,0,0,%inf;0,0,-1,0;1,0,1,0;%inf,0,0,%inf];
	b = and(result==desired);
endfunction	
function [b]= testNonconvexSingleInputParameter()
    function y=fct(x),y=(x.^2-1)^2;endfunction
    function y=dfct(x),y=4*x.*(x.^2-1);endfunction
    x=linspace(-3,3,5);x=[x 0 -1 1];x=unique(gsort(x,'c','i'));
    f=plq_build(x,fct);
    fsdirect=plq_lft(f);
    b=plq_check(fsdirect)
endfunction

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testLLNonsmooth,"testLLNonsmooth"), b);
	b = checkForFail(testWrapper(testQQSmooth,"testQQSmooth"), b);
	b = checkForFail(testWrapper(testQQQSmooth,"testQQQSmooth"), b);
	b = checkForFail(testWrapper(testabs,"testabs"), b);
	b = checkForFail(testWrapper(testL,"testL"), b);
	b = checkForFail(testWrapper(testI,"testI"), b);
	b = checkForFail(testWrapper(testNormSquared,"testNormSquared"), b);
	b = checkForFail(testWrapper(testQQNonsmooth,"testQQNonsmooth"), b);
	b = checkForFail(testWrapper(testQLQSmooth,"testQLQSmooth"), b);	
	b = checkForFail(testWrapper(testQLQNonsmooth,"testQLQNonsmooth"), b);
	b = checkForFail(testWrapper(testPLQboundedLHS,"testPLQboundedLHS"), b);
	b = checkForFail(testWrapper(testPLQboundedRHS,"testPLQboundedRHS"), b);
	b = checkForFail(testWrapper(testPiecewiseLinear,"testPiecewiseLinear"), b);
	b = checkForFail(testWrapper(testNonconvex,"testNonconvex"), b);
    b = checkForFail(testWrapper(testNonconvexSingleInputParameter,"testNonconvexSingleInputParameter"), b);
endfunction


