mode(-1);

__test_set = "PLQ: plq_split";

//perform all tests by comparing with plq functions
function b = _test(p)
  P=gph_plq(p);
  q=plq_scalar(p,3);Q=gph_plq(q);
  QQ=gph_scalar(P,3);
  b = gph_isEqual(Q,QQ);

  q=plq_scalar(p,0);Q=gph_plq(q);
  QQ=gph_scalar(P,0);
  b = b & gph_isEqual(Q,QQ);
endfunction

function [b] = testAbs()
    p =[0 0 -1 0; %inf 0 1 0];//abs function
    b=_test(p);
endfunction

function [b] = testInd()
  p=[-1 0 0 %inf;1 0 0 0 ;%inf 0 0 %inf];//indicator function I_[-1,1].
  b=_test(p);
endfunction

function [b] = testPL()
	// A piecewise linear function:
	//     x<0: y=-x
	//   0<x<1: y=0
	//     1<x: y=x-1
	G = [-1, 0, 0, 1, 1, 2; ..
	     -1, -1, 0, 0, 1, 1; ..
	     1, 0, 0, 0, 0, 1];
	p=plq_gph(G);
	b=_test(p);
endfunction

function [b] = testPLQ1()
	b = %f;
	// A PLQ function:
	//     x<-1: y=(x+1)^2
	//   -1<x<1: y=0
	//      1<x: y=(x-1)^2
	p=[-1 1 2 1;1 0 0 0;%inf 1 -2 1];
	b=_test(p);
endfunction

function [b] = testPLQ2()
	b = %f;
	// A PLQ function:
	//   x in [-1,1]: 0
	//     otherwise: x^2 - 1
	p=[-1 1 0 -1;1 0 0 0;%inf 1 0 -1];
    b=_test(p);
endfunction

function [b] = testQuadratic()
  p=[%inf,1,0,0];
  b=_test(p);
endfunction

function [b] = testLinear()
    p=[%inf,0,1,0];
    b=_test(p);		
endfunction

function [b] = testIndicatorSingleton()
  p=[1,0,0,2];
  b=_test(p);
endfunction

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testAbs,"testAbs"), b);
	b = checkForFail(testWrapper(testInd,"testInd"), b);
	b = checkForFail(testWrapper(testPL,"testPL"), b);
	b = checkForFail(testWrapper(testPLQ1,"testPLQ1"), b);
	b = checkForFail(testWrapper(testPLQ2,"testPLQ2"), b);
	b = checkForFail(testWrapper(testQuadratic,"testQuadratic"), b);
	b = checkForFail(testWrapper(testLinear,"testLinear"), b);
	b = checkForFail(testWrapper(testIndicatorSingleton,"testIndicatorSingleton"), b);	
endfunction
