// Author: Bryan Gardiner
// For: Yves Lucet
// Date: Summer 2008
// Test general Fitzpatrick function algorithms.

// <-- NO CHECK REF -->

printf("Testing with a linear function.\n");
b = %f;
a = -2:2;
astar = 2*a - 7;
A = [a;astar];
x = -3:3;
xstar = -3:3;
for n = 1:3
	if (n <= 2) then dresult = op_fitz_brute(A, n, x, xstar); end;
	rresult = op_fitz_direct(A, n, x, xstar);
	lresult = op_fitz(A, n, x, xstar);
	b = and(abs(rresult - lresult) < 1E-5);
	if (n <= 2) then b = b & and(abs(rresult - dresult) < 1E-5); end;
	if (~b) then pause; end;
end;

printf("Testing with an exponential function.\n");
b = %f;
a = -1:2;
astar = exp(a);
A = [a;astar];
x = -3:3;
xstar = -3:3;
for n = 1:2
	dresult = op_fitz_brute(A, n, x, xstar);
	rresult = op_fitz_direct(A, n, x, xstar);
	lresult = op_fitz(A, n, x, xstar);
	b = and(abs(rresult - lresult) < 1E-5);
	b = b & and(abs(rresult - dresult) < 1E-5);
	if (~b) then pause; end;
end;

printf("Testing with a logarithmic function.\n");
b = %f;
a = 1:3;
astar = log(a);
A = [a;astar];
x = 0:4;
xstar = 0:4;
for n = 1:4
	if (n <= 2) then dresult = op_fitz_brute(A, n, x, xstar); end;
	rresult = op_fitz_direct(A, n, x, xstar);
	lresult = op_fitz(A, n, x, xstar);
	b = and(abs(rresult - lresult) < 1E-5);
	if (n <= 2) then b = b & and(abs(rresult - dresult) < 1E-5); end;
	if (~b) then pause; end;
end;

printf("Testing with a quadratic function.\n");
b = %f;
a = 0:3;
astar = a^2 - 2;
A = [a;astar];
A = A(:,$:-1:1);
x = -1:4;
xstar = 0:4;
for n = 1:4
	if (n <= 2) then dresult = op_fitz_brute(A, n, x, xstar); end;
	rresult = op_fitz_direct(A, n, x, xstar);
	lresult = op_fitz(A, n, x, xstar);
	b = and(abs(rresult - lresult) < 1E-5);
	if (n <= 2) then b = b & and(abs(rresult - dresult) < 1E-5); end;
	if (~b) then pause; end;
end;

printf("Testing 4,5-order Fitzpatrick functions.\n");
b = %f;
astar = [-2:0.5:-1, 0:2];
a = 1:length(astar);
A = [a;astar];
x = [0:1, 3.5, 6:7];
xstar = -2:3;
for n = 4:5
	rresult = op_fitz_direct(A, n, x, xstar);
	lresult = op_fitz(A, n, x, xstar);
	b = and(abs(rresult - lresult) < 1E-5);
	if (~b) then pause; end;
end;

printf("Testing with a vertical line segment.\n");
a = [1,2,2,3,3];
astar = 1:length(a);
A1 = [a;astar];
A2 = [a;astar;astar];
x = 0:5;
xstar = 0:5;
// desired was computed with op_fitz_brute.
desired = [-1,0,1,4,9,14; 0,1,2,5,10,15; 1,2,4,7,12,17; 3,4,6,9,14,19; 5,6,8,12,17,22; 8,9,11,15,20,25]';
rresult = op_fitz_direct(A1, %inf, x, xstar);
//lresult = op_fitz(A1, n, x, xstar);        LLT can't handle duplicate a values.
//iresult = op_fitzinf(A2, x, xstar);        Requires strictly increasing a values.
if (~and(abs(desired - rresult) < 1E-5)) then pause; end;
