// Author: Bryan Gardiner
// For: Yves Lucet
// Date: Summer 2008
// Test %inf-order Fitzpatrick functions with 2nd argument = [0].

// <-- NO CHECK REF -->

printf("Testing with a quadratic operator.\n");
b = %f;
a = -1:2;
astar = (a + 1).^2 - 2;    // [-2,-1,2,7]
A1 = [a;astar];
A2 = [a;astar;astar];
x = 0:5;
rresult = op_fitz_direct(A1, %inf, x, 0);
lresult = op_fitz(A1, %inf, x, 0);
iresult = op_fitzinf(A2, x, 0);
Lplqf = plq_fitzinf0(A2); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A2);
if (~and(abs(rresult - lresult) < 1E-5)) then pause; end;
if (~and(abs(rresult - iresult) < 1E-5)) then pause; end;
if (~and(abs(rresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a logarithmic operator.\n");
a = 0.5:0.25:2;
astar = log(a);
A1 = [a;astar];
A2 = [a;astar;astar];
x = 0:5;
lresult = op_fitz(A1, %inf, x, 0);
iresult = op_fitzinf(A2, x, 0);
Lplqf = plq_fitzinf0(A2); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A2);
disp(lresult); disp(iresult); disp(Lresult);
if (~and(abs(lresult - iresult) < 1E-5)) then pause; end;
if (~and(abs(lresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a strictly positive operator.\n");
a = 1:4;
bl = [0.5, 2, 3.5, 5.5];
br = [1, 3, 5, 7.5];
A = [a;bl;br];
x = 0.5:4.5;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a strictly positive, ""gapless"" operator.\n");
a = 1:4;
bl = [0.5, 1, 2, 5];
br = [1, 2, 5, 5];
A = [a;bl;br];
x = [0.25, 0.5, 1, 1.5, 2, 3, 5, 5.5];
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a simple, positive, gapless operator.\n");
a = 1:2;
bl = [1,2];
br = [2,4];
A = [a;bl;br];
x = -2:2;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a strictly negative operator.\n");
a = 1;
bl = [-3];
br = [-2];
A = [a;bl;br];
x = -2:2;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a longer, strictly negative operator.\n");
a = 1:4;
bl = [-8, -6, -4, -2];
br = [-7, -5, -3, -1];
A = [a;bl;br];
x = -8.5:-0.5;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a negative, gapless operator.\n");
a = 1:2;
bl = [-3,-2];
br = [-2,-1];
A = [a;bl;br];
x = 0.5:0.5:2.5;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with an operator crossing 0.\n");
a = 1:4;
bl = [-1, 0, 3.5, 5.5]
br = [ 0, 3, 5  , 7.5];
A = [a;bl;br];
x = -0.5:4.5;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a gapless operator crossing 0.\n");
a = 1:3;
bl = [-1.5, -0.5, 0.5];
br = [-0.5,  0.5, 1.5];
A = [a;bl;br];
x = -2:2;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with an operator with flat sections.\n");
a = 1:5;
bl = [-1.5, -1.5, -1.5, 0.5, 0.5];
br = [-1.5, -1.5,  0.5, 0.5, 0.5];
A = [a;bl;br];
x = -2:2;
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with more flat sections.\n");
a = 1:5;
bl = [-0.5, -0.5, 1, 1, 2];
br = [-0.5,  1  , 1, 1, 2];
A = [a;bl;br];
A = A(:,$:-1:1);
x = [0.25, 0.5, 1, 1.5, 2, 3];
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;

printf("Testing with a single-segment operator.\n");
a = [2];
bl = [1];
br = [5];
A = [a;bl;br];
x = [0.5, 1, 1.5, 2, 3, 5, 5.5];
iresult = op_fitzinf(A, x, 0);
Lplqf = plq_fitzinf0(A); [Lresult,Lk] = plq_eval(Lplqf, x);
F = plq_fitzinf0_direct(A);
if (~and(abs(iresult - Lresult) < 1E-5)) then pause; end;
if (~isEqual(plq_clean(Lplqf), plq_clean(F))) then pause; end;
