// Original Author: Yves Lucet
// Author:          Jeff Dicker
// Whatsit:         Unit tests for pl_lft_direct, pl_lft_direct_d, pl_lft_llt, pl_lft_llt_d
// Date:            Summer 2005

// <-- NO CHECK REF -->

function y = f(x), y = x .^ 2 / 2; endfunction
function s = df(x), s = x; endfunction

printf("testPartialConj_direct\n");
//compute the conjugate of a function going through (i,row(i))
//with pl_lft_direct_d
row = [1 2.5 5 8.5 13]';//values of the function on {1,2,3,4,5}
conj_part = pl_lft_direct_d(row);//conjugate computation
if (~isequal(conj_part, [0 1.5 4 7.5 12]')) then pause; end;//verify the result

printf("testQuadratic_direct_plt\n");
//conjugate of the energy function f(x)=x^2/2
//with pl_lft_direct
X = [1:1:10]';//primal grid
Y = 0.5.*(X.^2);//sample the function on X
//compute the conjugate with dual grid = primal grid
[Conj] = pl_lft_direct(X, Y, X);
[Conjp] = pl_lft_plt(X, f, df);
// Check that the conjugate of the energy function is itself
if (~isequal(Y, Conj, Conjp)) then pause; end;

printf("testPartialConj_llt\n");
//compute the conjugate of a function going through (i,row(i))
//with pl_lft_llt
row = [1 2.5 5 8.5 13]';//values of the function on {1,2,3,4,5}
conj_part = pl_lft_llt_d(row);//conjugate computation
if (~isequal(conj_part, [0 1.5 4 7.5 12]')) then pause; end;//verify the result

printf("testQuadratic_llt\n");
//conjugate of the energy function f(x)=x^2/2
//with pl_lft_llt
X = [1:1:10]';//primal grid
Y = 0.5.*(X.^2);//sample the function on X
//compute the conjugate with dual grid = primal grid
[Conj] = pl_lft_llt(X, Y, X, %t, 0);
// Check that the conjugate of the energy function is itself
if (~isequal(Y, Conj)) then pause; end;

printf("testCompareDirLLTFusion\n");
//compare pl_lft_llt with option _pl_fusion,  pl_lft_llt with option _pl_fusionsci, 
//and pl_lft_direct when computing the conjugate of discrete data 
X = [1:1:10]';//primal grid
Y = [34 3 1 15 13 12 22 30 32 58]';//values of an ungiven function on X
ConjA = pl_lft_llt(X, Y, X, %f);//result with llt algorithm option _pl_fusionsci
ConjAa= pl_lft_llt(X, Y, X, %f, 0);//result with llt algorithm option _pl_fusion
ConjB = pl_lft_direct(X, Y, X);//result with direct algorithm (quadratic)
ConjC = pl_lft_generic(X, Y, X, %f);
if (~isequal(ConjA, ConjAa, ConjB, ConjC)) then pause; end;//verify the result

printf("testLft2dEmptyS\n");
X = (1:4)';
Y = (1:4)';
f = X * Y';
ConjA1 = pl_lft_direct_2d(X, Y, f, [], []);
ConjA2 = pl_lft_llt_2d   (X, Y, f, [], [], %t);
ConjB1 = pl_lft_direct_2d(X, Y, f,  0, []);
ConjB2 = pl_lft_llt_2d   (X, Y, f,  0, [], %t);
ConjC1 = pl_lft_direct_2d(X, Y, f, [],  0);
ConjC2 = pl_lft_llt_2d   (X, Y, f, [],  0, %t);
ConjD  = pl_lft_generic (X, Y, f, [], [], %t);
if (~isEqual([], ConjA1, ConjA2, ConjB1, ConjB2, ConjC1, ConjC2, ConjD)) then pause; end;

printf("testLftLlt2dWrongSize\n");
b = %t;
try
	Xr = 1:4; Xc = 1:5; f = eye(5, 5);
	Sr = 1:3; Sc = 2:5;
	pl_lft_llt_2d(Xr, Xc, f, Sr, Sc);
catch
	disp(lasterror(%f));
	if (lasterror() == "Inconsistent size of f in pl_lft_llt_2d. size(Xr)=4, but size(f,1)=5")
		b = %f;
	end
end
if b then pause; end;
