// Author:   Jeff Dicker
// For:      Dr. Yves Lucet
// Whatsit:  Unit tests for pl_me_nep_2d, pl_me_nep, pl_me_direct

// <-- NO CHECK REF -->

printf("testdirect_Quadratic1d\n");
//compare NEP Moreau envelope algorithm with result
//computed manually for the function f(x)=x^2
X=(1:4)';//primal grid
F=X.^2;
result = pl_me_nep(X,F,X);
correct = [1;2;5;8];
if (~and(correct == result)) then pause; end;

printf("testdirect2dnep2d_Quad\n");
//compare results of nep and direct algorithm for the
//bivariate function f(x,y)=4*x^2+4*y^2
Xr = (1:5)'; Xc = Xr; //primal grid
Sr = Xr; Sc = Sr;//dual grid
F = (4 .* (Xr.^2) * ones(1, size(Xr, 1))) + (ones(size(Xc, 1), 1) * (4 .* (Xc').^2));
result = pl_me_nep_2d(Xr, Xc, F, Sr, Sc);
correct = pl_me_direct_2d(Xr, Xc, F, Sr, Sc);	
if (~and(correct == result)) then pause; end;

printf("testdirect2d_QuadUnbal\n");
//compare the result of the bivariate direct2d algorithm with
//the univariate direct algorithm for the separable function
//f(x,y)=x^2+2*y^2
function f=f(lambda,x),f=lambda * x.^2,endfunction
function g=g(lambda1,lambda2,x,y),g=f(lambda1,x)+f(lambda2,y),endfunction
lambda1=1;lambda2=2;
x1=(-10:10)';x2=(-5:5)';//primal grid
[X, Y]=ndgrid(x1,x2);F=g(lambda1,lambda2,X,Y);//evaluate g on the primal grid
s1=(-4:4)';s2=(-5:5)';//dual grid
desired=pl_me_direct_2d(x1,x2,F,s1,s2);
//check the results by using the fact g is separable
//so its bivariate Moreau envelope is the sum of the 
//univariate Moreau envelopes for each of its variables
Ms1=pl_me_direct(x1,f(lambda1,x1),s1);//univariate Moreau envelope for x1
Ms2=pl_me_direct(x2,f(lambda2,x2),s2);//univariate Moreau envelope for x2
//build the bivariate result
t1 = Ms1 * ones(1,size(Ms2,1));
t2 = ones(size(Ms1,1),1) * Ms2';
correct=t1+t2;
if (~and(correct == desired)) then pause; end;

printf("testNEP_Quadratic1d\n");
//compare the result of the NEP algorithm with
//manual computation for the function f(x)=x^2
//with primal grid=dual grid
X=(1:4)';//primal grid
F=X.^2;
result = pl_me_nep(X,F,X);
correct = [1;2;5;8];
if (~and(correct == result)) then pause; end;

printf("testNEP_Quad1d_XdiffS\n");
//compare the result of the NEP algorithm with
//manual computation for the function f(x)=x^2
//with different primal and dual grids
X = [1;1.2;1.4;1.6;1.8;2];//primal grid
S = [-0.5;-0.2;0;0.2;0.5];//dual grid
F=X.^2;
result = pl_me_nep(X,F,S);
correct = pl_me_direct(X,F,S);
if (~and(correct == result)) then pause; end;

printf("testNEP_Quadratic\n");
//Compare direct with nep algorithm to compute the 
//Moreau envelope of the bivariate function
//f(x,y)=4*x^2+4*y^2
Xr = (1:5)'; Xc = Xr; //primal grid
Sr = Xr; Sc = Sr;//dual grid
F = (4 .* (Xr.^2) * ones(1, size(Xr, 1))) + (ones(size(Xc, 1), 1) * (4 .* (Xc').^2));
result = pl_me_nep_2d(Xr, Xc, F, Sr, Sc);
correct = pl_me_direct_2d(Xr, Xc, F, Sr, Sc);	
if (~and(correct == result)) then pause; end;

printf("testNEP_QuadraticTwo\n");
//Compare direct with nep algorithm to compute the 
//Moreau envelope of the bivariate function
//f(x,y)=4*x^2+4*y^2
Xr = (-10:10)'; Xc = Xr; //primal grid
Sr = Xr; Sc = Sr;//dual grid
F = (4 .* (Xr.^2) * ones(1, size(Xr, 1))) + (ones(size(Xc, 1), 1) * (4 .* (Xc').^2));	
result = pl_me_nep_2d(Xr, Xc, F, Sr, Sc);
correct = pl_me_direct_2d(Xr, Xc, F, Sr, Sc);
if (~and(correct == result)) then pause; end;

printf("testNEP_Cubic\n");
//Compare direct with nep algorithm to compute the 
//Moreau envelope of the bivariate function
//f(x,y)=4*x^3+4*y^3
Xr = (-10:10)'; Xc = Xr; //primal grid
Sr = Xr; Sc = Sr;//dual grid
F = (4 .* (Xr.^3) * ones(1, size(Xr, 1))) + (ones(size(Xc, 1), 1) * (4 .* (Xc').^3));	
result = pl_me_nep_2d(Xr, Xc, F, Sr, Sc);
correct = pl_me_direct_2d(Xr, Xc, F, Sr, Sc);
if (~and(correct == result)) then pause; end;

printf("testNEP_rand\n");
//Compare direct with nep algorithm to compute the 
//Moreau envelope of a randon univariate function
X = [-10:0.5:10]';//primal grid
S=[-7:0.4:7]';//dual grid
f = rand(X);//random values for f
[X, f] = pl_bb(X,f);//convexify the function (needed to apply the NEP algorithm)
desired = pl_me_direct(X, f, S);
result = pl_me_nep(X, f, S);
//verify the result taking into account roundoff errors
if (norm(result - desired) > 1E-10) then pause; end;

printf("testMeNep2dWrongSize\n");
b = %t;
try
	Xr = 1:4; Xc = 1:5; f = eye(5, 5);
	Sr = 1:3; Sc = 2:5;
	pl_me_nep_2d(Xr, Xc, f, Sr, Sc);
catch
	if (lasterror() == 'Inconsistent size of f in pl_me_nep_2d. size(Xr)=4, but size(f,1)=5')
		b = %f;
	end
end
if b then pause; end;
