// Test-script for plq_eval (Yves Lucet - 2006-06-21)
// Author:   Mike Trienis
// For:      Dr. Yves Lucet
// Whatsit:  A test file for plq_build, builds a piecewise linear function when given a set of points and the derivative at those points.

// <-- NO CHECK REF -->

printf("testexp\n");
x=linspace(0,4)';
plqf = plq_build(x,exp,exp);
y=plq_eval(plqf,x);
if norm(y-exp(x))>1E-8 then pause; end;

printf("testlinear\n");
function y=f(x),y=2*x+1;endfunction;
function dy=df(x),dy=2*ones(x);endfunction;
x=linspace(-2,2,5)';
plqf = plq_build(x,f,df);
y=plq_eval(plqf,x);
if norm(y-f(x))>1E-8 then pause; end;

printf("testpartiallylinear\n");
function y=f(x),i=find(x>0);y=zeros(x);y(i)=x(i).^2;endfunction
function y=df(x),i=find(x>0);y=zeros(x);y(i)=2*x(i);endfunction
x=linspace(-2,2,5)';
plqf = plq_build(x,f,df);
y=plq_eval(plqf,x);
if norm(y-f(x))>1E-8 then pause; end;

printf("testconjexp\n");
function y=f(x),i1=find(x<0);i2=find(x==0);i3=find(x>0);y(i1)=%inf*ones(i1);y(i2)=0;y(i3)=x(i3).*log(x(i3))-x(i3);endfunction
function y=df(x),i1=find(x<=0);i2=find(x>0);y(i1)=%inf*ones(i1);y(i2)=log(x(i2));endfunction
x=linspace(0,20,3)';
plqf = plq_build(x,f,df);
y = plq_eval(plqf,x);
yy=f(x);i=find(x==0);yy(i)=%inf*ones(i);//plq evaluates %inf at 0
i=isinf(y);ii=isinf(yy);//need to test %inf values separately
if (norm(y(~i)-yy(~ii))>1E-8) | ~and(y(i)==yy(i)) then pause; end;

printf("testconjexp_d\n");
function y=f(x),i1=find(x<0);i2=find(x==0);i3=find(x>0);y(i1)=%inf*ones(i1);y(i2)=0;y(i3)=x(i3).*log(x(i3))-x(i3);endfunction
function y=df(x),i1=find(x<=0);i2=find(x>0);y(i1)=%inf*ones(i1);y(i2)=log(x(i2));endfunction
x=linspace(0,20,3)';
y=f(x);dy=df(x);
plqf = plq_build_d(x,y,dy);
y = plq_eval(plqf,x);
yy=f(x);i=find(x==0);yy(i)=%inf*ones(i);//plq evaluates %inf at 0
i=isinf(y);ii=isinf(yy);//need to test %inf values separately
if (norm(y(~i)-yy(~ii))>1E-8) | ~and(y(i)==yy(i)) then pause; end;

printf("testexp0\n");
x=linspace(0,4)';
plqf = plq_build(x,exp);
y=plq_eval(plqf,x);
if norm(y-exp(x))>1E-8 then pause; end;

printf("testabs0\n");
x=linspace(-4,4,5)';
plqf = plq_build(x,abs);
if ~and([0,0,-1,0;%inf,0,1,0]==plqf) then pause; end;

printf("testabs0_d\n");
x=linspace(-4,4,5)';
y=abs(x);
plqf = plq_build_d(x,y);
if ~and([0,0,-1,0;%inf,0,1,0]==plqf) then pause; end;

printf("testx0\n");
x=linspace(-4,4)';
function y=f(x),y=x;endfunction;
plqf = plq_build(x,f);
if ~and([%inf,0,1,0]==plqf) then pause; end;

printf("testlinear0\n");
x=linspace(-2,2,5)';function y=f(x),y=2*x+1;endfunction;
plqf = plq_build(x,f);
if ~and([%inf,0,2,1]==plqf) then pause; end;

printf("testpartiallylinear0\n");
function y=f(x),i=find(x>0);y=zeros(x);y(i)=x(i).^2;endfunction
x=linspace(-2,2,5)';
plqf = plq_build(x,f);
y=plq_eval(plqf,x);
if norm(y-f(x))>1E-8 then pause; end;

printf("testInf0\n");
function y = f(x); ys = [%inf;4/1;4/2;4/3;4/4;%inf]; y = ys(int(x*4+1)); endfunction
x = 0:0.25:1.25;
plqf = plq_build(x,f,%f,%f,%t);
desired = [1/4,0,0,%inf; 2/4,0,-8,6; 3/4,0,-8/3,10/3; 1,0,-4/3,7/3; %inf,0,0,%inf];
if ~isEqual(plqf, desired) then pause; end;

printf("testInf1\n");
//function y = f(x); i=ieee(); ieee(2); y=ones(x)./x; ieee(i); endfunction
//function s = df(x); i=ieee(); ieee(2); s=-ones(x)./(x.^2); ieee(i); endfunction
function y = f(x); ys = [%inf;4/1;4/2;4/3;4/4;%inf]; y = ys(int(x*4+1)); endfunction
function dy = df(x); dys = -[%inf;[4/1;4/2;4/3;4/4].^2;%inf]; dy = dys(int(x*4+1)); endfunction
x = 0:0.25:1.25;
plqf = plq_build(x,f,df,%f,%t);
//desired = [0,0,0,%inf; 1/3,0,-16,8; 0.6,0,-4,4; 6/7,0,-16/9,8/3; %inf,0,-1,2];
desired = [1/4,0,0,%inf; 1/3,0,-16,8; 3/5,0,-4,4; 6/7,0,-16/9,8/3; 1,0,-1,2; %inf,0,0,%inf];
if ~isEqual(plqf, desired) then pause; end;

printf("testNeedEvalFptr\n");
x = 0:0.5:3;
f1 = plq_build(x, exp, %f, %f);
f2 = plq_build(x, exp, %f, %t);
if ~isEqual(f1, f2) then pause; end;

printf("testNeedEvalFunc\n");
function y=f(x)
	if (size(x) ~= [1,1]) then error("blah"); end;
	y=exp(x);
endfunction;
b = %f;
x = 0:0.5:3;
f1 = plq_build(x, f, %f, %f);
f2 = plq_build(x, f, %f, %t);
if ~isEqual(f1, f2) then pause; end;

// This isn't testing anything!!
//printf("testNonconvexfctn\n");
//b=%f;
//x=linspace(0,2*%pi,4)';
//result = plq_build(x,sin,cos);
//desired = [] //The desired result requires the x(i) values to be ordered. See result.

printf("testsoqs\n");
function y = f(x);  ys=[.0;.3;.5;.2;.6;1.2;1.3;1.0;1.0;1.0;0;-1.0];y=ys(int(x+1)); endfunction
function dy = df(x); dys = [.5;1;0;-.2;5;4;0;-.2;-.2;.3;-.9999;-1.001]; dy = dys(int(x+1)); endfunction
x=linspace(0,11,12)';
plqf = plq_build(x,f,df,%t,%f,'soqs');
desired = [0,0,0,0; .9,-.25,.5,0; 1,4.75,-8.5,4.05; 1.2,-2,5,-2.7]
if ~isEqual(plqf(1:4,:), desired) then pause; end;

printf("testsoqs_noderiv\n");
function y = f(x);  ys=[.0;.3;.5;.2;.6;1.2;1.3;1.0;1.0;1.0;0;-1.0];y=ys(int(x+1)); endfunction
function dy = df(x); dys = [.5;1;0;-.2;5;4;0;-.2;-.2;.3;-.9999;-1.001]; dy = dys(int(x+1)); endfunction
x=linspace(0,11,12)';
plqf = plq_build(x,f,%f,%t,%f,'soqs');
desired = [0,0,0,0; .5,-.06,.36,0; 1,-.06,.36,0]
if ~isEqual(plqf(1:3,:), desired) then pause; end;
